/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  dashboard_definition_exports
} from "./chunk-UUGRNOSW.js";
import {
  dashboard_ui_exports
} from "./chunk-V6LCPHRJ.js";
import {
  dashboard_utils_exports
} from "./chunk-QCW3O7QD.js";

// src/absoluteLayoutOptionsSchema.ts
var absoluteLayoutOptionsSchema_default = {
  backgroundColor: {
    anyOf: [
      {
        pattern: dashboard_definition_exports.COLOR_PATTERN_WITH_RGBA_AND_TOKEN,
        type: "string",
        description: `Specify the color of the layout background color using using a hex code, rgb/rgba value, an HTML color name, or a token.

The default for light mode is "#F2F4f5". The default for dark mode is "#3C444D". The default for SCP mode is "#15161B"`
      },
      {
        type: "string",
        enum: dashboard_ui_exports.namedColors,
        description: `Specify the color of the layout background color using a valid web color such as 'blue'`
      }
    ]
  },
  display: {
    type: "string",
    default: "auto",
    enum: ["actual-size", "fit-to-width", "auto", "auto-scale"],
    description: `Specify the display behavior. Valid options are:


actual-size: The dashboard will not change size when the browser is resized.


fit-to-width: The dashboard will change in size as the browser is resized, always filling the available width of the browser.


auto: The dashboard will change in size as the browser is resized, to a maximum zoom of 100%.`
  },
  backgroundImage: {
    type: "object",
    properties: {
      w: {
        type: "number",
        description: `Specify the image's h (height) and w (width) in pixels. These options should not be specified if you set the sizeType option. If they are, they will overrule that setting.`
      },
      h: {
        type: "number",
        description: `Specify the image's h (height) and w (width) in pixels. These options should not be specified if you set the sizeType option. If they are, they will overrule that setting.`
      },
      x: {
        type: "number",
        description: "Specify, in pixels, the position of the background image on the x-axis and y-axis. These options should not be specified if you set the sizeType option. If they are, they will overrule that setting."
      },
      y: {
        type: "number",
        description: "Specify, in pixels, the position of the background image on the x-axis and y-axis. These options should not be specified if you set the sizeType option. If they are, they will overrule that setting."
      },
      src: {
        type: "string",
        description: "The url of background image."
      },
      opacity: {
        type: "number",
        minimum: 0,
        maximum: 1,
        description: "The opacity of the image."
      },
      sizeType: {
        type: "string",
        enum: ["auto", "contain", "cover"],
        default: "contain",
        description: `Specify one of the following:


auto: Scales the image while maintaining the image proportion.


contain: Increases the size of the of the image as much as possible without cropping or stretching it.


cover: Increases the size of the image without stretching it. The image may be cropped vertically and horizontally so that no empty space is shown.`
      }
    },
    description: `A background image must be one of the following types: jpg, jpeg, png, svg, or gif

Specify the location of a background image using the option src. For example: "src": "https://www.myImageLocation.com/galleryX/image1.png`,
    additionalProperties: false
  },
  width: {
    type: "number",
    default: dashboard_utils_exports.DEFAULT_CANVAS_WIDTH,
    description: "Canvas width in pixels."
  },
  height: {
    type: "number",
    default: dashboard_utils_exports.DEFAULT_CANVAS_HEIGHT,
    description: "Canvas height in pixels."
  }
};

export {
  absoluteLayoutOptionsSchema_default
};
//# sourceMappingURL=chunk-2VLZOQVK.js.map
