/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  ConnectedLineEditorConfig_default
} from "./chunk-SQHLMK4Z.js";
import {
  ConnectedLineOptionsSchema_default
} from "./chunk-4AMF6GJS.js";
import {
  dashboard_context_exports,
  themes_exports
} from "./chunk-2OPMAP46.js";
import {
  react_default,
  react_exports
} from "./chunk-D2UIVRV7.js";
import {
  dashboard_utils_exports
} from "./chunk-QCW3O7QD.js";
import {
  i18n_exports
} from "./chunk-PLOB2GBC.js";
import {
  __export,
  __reExport,
  __spreadValues
} from "./chunk-W3Q7A2F6.js";

// esm-externals:@splunk/react-icons/Line
var Line_exports = {};
__export(Line_exports, {
  default: () => Line_default
});
__reExport(Line_exports, Line_star);
import * as defaultImport from "@splunk/react-icons/Line";
import * as Line_star from "@splunk/react-icons/Line";
var Line_default = "default" in defaultImport ? defaultImport.default : defaultImport;

// esm-externals:@splunk/visualization-encoding/Options
var Options_exports = {};
__export(Options_exports, {
  default: () => Options_default
});
__reExport(Options_exports, Options_star);
import * as defaultImport2 from "@splunk/visualization-encoding/Options";
import * as Options_star from "@splunk/visualization-encoding/Options";
var Options_default = "default" in defaultImport2 ? defaultImport2.default : defaultImport2;

// src/visualizations/utils/lineUtils.ts
var computeArrowTriangle = (w, h, l, L) => {
  if (l === 0) {
    return { arrowW: 0, arrowH: 0, arrowL: 0 };
  }
  const arrowW = w * L / l;
  const arrowH = h * L / l;
  return { arrowW, arrowH, arrowL: L };
};
var computeFromArrowAngle = (from, to) => {
  if (to.x - from.x === 0) {
    return from.y > to.y ? 90 : 270;
  }
  const angle = Math.atan((to.y - from.y) / (to.x - from.x)) * (180 / Math.PI);
  return from.x > to.x ? angle : 180 + angle;
};
var computeToArrowAngle = (from, to) => {
  if (to.x - from.x === 0) {
    return from.y > to.y ? 270 : 90;
  }
  const angle = Math.atan((to.y - from.y) / (to.x - from.x)) * (180 / Math.PI);
  return from.x > to.x ? 180 + angle : angle;
};
var getNewFrom = (from, to, arrowW, arrowH, fromArrow) => {
  let newFromX;
  let newFromY;
  if (fromArrow) {
    newFromX = from.x > to.x ? from.x - arrowW : from.x + arrowW;
    newFromY = from.y > to.y ? from.y - arrowH : from.y + arrowH;
  } else {
    newFromX = from.x;
    newFromY = from.y;
  }
  return { newFrom: { x: newFromX, y: newFromY } };
};
var getNewTo = (from, to, arrowW, arrowH, toArrow) => {
  let newToX;
  let newToY;
  if (toArrow) {
    newToX = from.x > to.x ? to.x + arrowW : to.x - arrowW;
    newToY = from.y > to.y ? to.y + arrowH : to.y - arrowH;
  } else {
    newToX = to.x;
    newToY = to.y;
  }
  return { newTo: { x: newToX, y: newToY } };
};
var computePathTriangle = (from, to) => {
  const w = Math.abs(from.x - to.x);
  const h = Math.abs(from.y - to.y);
  const l = Math.sqrt(w * w + h * h);
  return { w, h, l };
};

// src/visualizations/ConnectedLine.tsx
var rootSvgStyle = {
  pointerEvents: "none",
  overflow: "visible"
};
var lineStyle = {
  pointerEvents: "auto"
};
var dataContract = {
  requiredDataSources: ["primary"],
  initialRequestParams: {}
};
var vizConfig = {
  optionsSchema: ConnectedLineOptionsSchema_default,
  editorConfig: ConnectedLineEditorConfig_default,
  dataContract,
  key: "abslayout.line",
  name: (0, i18n_exports._)("Connected Line"),
  category: dashboard_context_exports.vizCategories.get("shapes"),
  icon: Line_default,
  events: {},
  // no onEventTrigger called by connected line
  supports: ["dynamic-options"],
  // what capabilities are supported?
  requiredProps: []
};
var defaultOptions = {
  strokeOpacity: 1,
  strokeWidth: 1,
  strokeDasharray: 0,
  strokeDataSeries: '> primary | seriesByType("number")',
  strokeDataPoint: "> strokeDataSeries | lastPoint()",
  fromArrow: false,
  toArrow: false
};
var defaultFrom = {
  x: 0,
  y: 0
};
var defaultTo = {
  x: 150,
  y: 0
};
var empty = {};
var ConnectedLine = ({
  from = defaultFrom,
  to = defaultTo,
  onLineSelect = dashboard_utils_exports.noop,
  options = defaultOptions,
  dataSources = empty,
  context = empty
}) => {
  const { colorScheme } = (0, themes_exports.useSplunkTheme)();
  const renderOptions = (0, react_exports.useMemo)(() => {
    try {
      return Options_exports.Options.evaluate(
        {
          context,
          options: __spreadValues(__spreadValues({}, defaultOptions), options)
        },
        dataSources
      );
    } catch (e) {
      if (e instanceof Error) {
        dashboard_utils_exports.console.error(
          `Unexpected error evaluating line options: ${e.message}`
        );
      }
    }
    return defaultOptions;
  }, [context, options, dataSources]);
  const themeStrokeColor = colorScheme === "light" ? "#000000" : "#ffffff";
  const { w, h, l } = computePathTriangle(from, to);
  const arrowL = 5 * renderOptions.strokeWidth;
  const { arrowW, arrowH } = computeArrowTriangle(w, h, l, arrowL);
  const squashedArrows = arrowL > l && renderOptions.fromArrow && renderOptions.toArrow;
  const fromArrowAngle = computeFromArrowAngle(from, to);
  const toArrowAngle = computeToArrowAngle(from, to);
  const fromArrowId = (0, dashboard_utils_exports.uniqueId)();
  const toArrowId = (0, dashboard_utils_exports.uniqueId)();
  const newW = w + arrowL;
  const newH = h + 2 * arrowL + 10;
  const { newFrom } = getNewFrom(
    from,
    to,
    arrowW,
    arrowH,
    renderOptions.fromArrow
  );
  const { newTo } = getNewTo(from, to, arrowW, arrowH, renderOptions.toArrow);
  return /* @__PURE__ */ react_default.createElement("svg", { width: newW, height: newH, style: rootSvgStyle }, /* @__PURE__ */ react_default.createElement(
    "path",
    {
      "data-test": "arrows",
      d: squashedArrows ? `M${newTo.x} ${newTo.y} L${newFrom.x} ${newFrom.y}` : `M${newFrom.x} ${newFrom.y} L${newTo.x} ${newTo.y}`,
      stroke: renderOptions.strokeColor || themeStrokeColor,
      strokeOpacity: `${renderOptions.strokeOpacity}`,
      strokeWidth: `${renderOptions.strokeWidth}`,
      strokeDasharray: `${renderOptions.strokeDasharray}`,
      fill: "none",
      markerStart: renderOptions.fromArrow ? `url(#${fromArrowId})` : "",
      markerEnd: renderOptions.toArrow ? `url(#${toArrowId})` : ""
    }
  ), /* @__PURE__ */ react_default.createElement(
    "path",
    {
      "data-test": "line",
      onMouseDown: onLineSelect,
      d: `M${from.x} ${from.y} L${to.x} ${to.y}`,
      strokeWidth: `${renderOptions.strokeWidth + 20}`,
      fill: "none",
      stroke: "white",
      strokeOpacity: 0,
      style: lineStyle
    }
  ), /* @__PURE__ */ react_default.createElement("defs", null, /* @__PURE__ */ react_default.createElement(
    "marker",
    {
      id: `${fromArrowId}`,
      markerWidth: "10",
      markerHeight: "10",
      refX: squashedArrows ? "5" : "0",
      refY: "2",
      orient: `${fromArrowAngle}deg`,
      markerUnits: "strokeWidth"
    },
    /* @__PURE__ */ react_default.createElement(
      "path",
      {
        d: "M0,0 L0,4 L5,2 z",
        fill: renderOptions.strokeColor || themeStrokeColor,
        fillOpacity: `${renderOptions.strokeOpacity}`
      }
    )
  ), /* @__PURE__ */ react_default.createElement(
    "marker",
    {
      id: `${toArrowId}`,
      markerWidth: "10",
      markerHeight: "10",
      refX: squashedArrows ? "5" : "0",
      refY: "2",
      orient: `${toArrowAngle}deg`,
      markerUnits: "strokeWidth"
    },
    /* @__PURE__ */ react_default.createElement(
      "path",
      {
        d: "M0,0 L0,4 L5,2 z",
        fill: renderOptions.strokeColor || themeStrokeColor,
        fillOpacity: `${renderOptions.strokeOpacity}`
      }
    )
  )));
};
ConnectedLine.config = vizConfig;
var ConnectedLine_default = ConnectedLine;

export {
  ConnectedLine_default
};
//# sourceMappingURL=chunk-33UDMVHW.js.map
