/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  BaseLayout_default
} from "./chunk-IJD6LVGH.js";
import {
  GridItem_default,
  computeMaxHeight,
  removeInvalidItems
} from "./chunk-NJ3ZVGVA.js";
import {
  styled_components_default,
  styled_components_exports
} from "./chunk-AZNCU5D5.js";
import {
  themes_exports
} from "./chunk-2OPMAP46.js";
import {
  dashboard_ui_exports
} from "./chunk-V6LCPHRJ.js";
import {
  react_default,
  react_exports
} from "./chunk-D2UIVRV7.js";
import {
  gridLayoutOptions
} from "./chunk-5YVOBQNO.js";
import {
  __spreadValues
} from "./chunk-W3Q7A2F6.js";

// src/utils/gridLayoutViewerUtils.ts
var getTemplateItems = ([currGridLine, nextGridLine, ...rest], axisLength, templateItems = []) => {
  templateItems.push(`[${currGridLine.names.join(" ")}]`);
  if (!nextGridLine) {
    return templateItems.join(" ");
  }
  const gridTrackWidth = Math.round(
    (nextGridLine.pos - currGridLine.pos) / axisLength * 100
  );
  templateItems.push(`minmax(0, ${gridTrackWidth}fr)`);
  return getTemplateItems([nextGridLine, ...rest], axisLength, templateItems);
};
var getTemplateItemsFromGridLines = (gridLines, axisLength) => {
  if (gridLines.length === 0) {
    return "";
  }
  return getTemplateItems(gridLines, axisLength);
};
var getCSSGridTemplate = ({
  layoutStructure,
  width,
  height
}) => {
  const xCoords = [];
  const yCoords = [];
  layoutStructure.forEach(({ position: { x, y, w, h }, item }) => {
    var _a, _b, _c, _d;
    xCoords[x] = (_a = xCoords[x]) != null ? _a : [];
    xCoords[x + w] = (_b = xCoords[x + w]) != null ? _b : [];
    yCoords[y] = (_c = yCoords[y]) != null ? _c : [];
    yCoords[y + h] = (_d = yCoords[y + h]) != null ? _d : [];
    xCoords[x].push(`${item}-start`);
    xCoords[x + w].push(`${item}-end`);
    yCoords[y].push(`${item}-start`);
    yCoords[y + h].push(`${item}-end`);
  });
  const xGridLines = Object.entries(xCoords).map(([pos, names]) => ({
    pos: Number(pos),
    names
  }));
  const yGridLines = Object.entries(yCoords).map(([pos, names]) => ({
    pos: Number(pos),
    names
  }));
  const gridRowTemplate = getTemplateItemsFromGridLines(yGridLines, height);
  const gridColTemplate = getTemplateItemsFromGridLines(xGridLines, width);
  return { gridRowTemplate, gridColTemplate };
};

// src/GridLayoutViewer.tsx
var GridCanvas = styled_components_default.div`
    ${themes_exports.mixins.reset("grid")};
    height: ${(props) => props.height}px;
    width: 100%;
    padding: ${(props) => props.gutterSize / 2}px;
    box-sizing: border-box;
    grid-template-columns: ${(props) => props.colTemplate};
    grid-template-rows: ${(props) => props.rowTemplate};
    gap: ${(props) => props.gutterSize}px;
    background: ${dashboard_ui_exports.customThemeVariables.dashboardBackgroundColor};
`;
GridCanvas.displayName = "GridCanvas";
var GridLayoutViewer = ({
  layoutStructure,
  renderLayoutItem,
  options
}) => {
  const { width, gutterSize } = __spreadValues(__spreadValues({}, gridLayoutOptions), options);
  const { height, gridItems, rowTemplate, colTemplate } = (0, react_exports.useMemo)(() => {
    const filteredStructure = removeInvalidItems(
      layoutStructure
    );
    const structureHeight = computeMaxHeight(filteredStructure);
    const { gridRowTemplate, gridColTemplate } = getCSSGridTemplate({
      layoutStructure: filteredStructure,
      width,
      height: structureHeight
    });
    const items = filteredStructure.map(({ item: itemId, type }) => /* @__PURE__ */ react_default.createElement(
      GridItem_default,
      {
        itemId,
        type,
        key: itemId,
        renderLayoutItem
      }
    ));
    return {
      height: structureHeight,
      gridItems: items,
      rowTemplate: gridRowTemplate,
      colTemplate: gridColTemplate
    };
  }, [layoutStructure, width, renderLayoutItem]);
  return /* @__PURE__ */ react_default.createElement(
    GridCanvas,
    {
      "data-test": "grid-layout-canvas",
      "data-layout-type": "grid",
      height,
      gutterSize,
      rowTemplate,
      colTemplate
    },
    gridItems
  );
};
GridLayoutViewer.propTypes = __spreadValues({}, BaseLayout_default.propTypes);
GridLayoutViewer.defaultProps = __spreadValues({}, BaseLayout_default.defaultProps);
var GridLayoutViewer_default = GridLayoutViewer;

export {
  GridLayoutViewer_default
};
//# sourceMappingURL=chunk-4PNGFGXC.js.map
