/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  gridLayoutOptionsSchema_default
} from "./chunk-XY7D6A2V.js";
import {
  GridLayoutApi_default,
  withLayoutShowHide
} from "./chunk-NRG3QARU.js";
import {
  Edge,
  GridCanvas_default,
  ItemDragPlaceholder,
  ItemDropTarget,
  Layer_default,
  PreviewPlaceholderItem,
  ResponsiveBlockItem_default,
  ResponsiveBlockOutline_default,
  applyVizPadding,
  computeEdges,
  computeMaxHeight,
  computeScaleToFit,
  dashboard_telemetry_exports,
  findEdgesInBoundary,
  findSnappableEdges,
  findTopBlockItemByPosition,
  getClientPosition,
  getHorizontalBoundaries,
  getNodes,
  getOffset,
  getVerticalBoundaries,
  lodash_exports,
  moveHorizontalEdge,
  moveVerticalEdge,
  scaleGridLayoutStructureByWidth,
  updateBlockItemPosition,
  updateBlockItemSize
} from "./chunk-NJ3ZVGVA.js";
import {
  dashboard_context_exports
} from "./chunk-2OPMAP46.js";
import {
  dashboard_ui_exports
} from "./chunk-V6LCPHRJ.js";
import {
  react_default,
  react_exports
} from "./chunk-D2UIVRV7.js";
import {
  gridLayoutOptions
} from "./chunk-5YVOBQNO.js";
import {
  dashboard_utils_exports
} from "./chunk-QCW3O7QD.js";
import {
  EDGE_PREVIEW_DELAY_MS,
  GRID_PADDING_PX,
  GRID_SIZE_PX,
  MIN_HEIGHT_PX,
  MIN_WIDTH_PX,
  PLACEHOLDER_SIZE_PX,
  SNAP_RANGE_PX,
  VIZ_PREVIEW_DELAY_MS
} from "./chunk-DLFGXT3M.js";
import {
  i18n_exports
} from "./chunk-PLOB2GBC.js";
import {
  __export,
  __reExport,
  __spreadProps,
  __spreadValues
} from "./chunk-W3Q7A2F6.js";

// esm-externals:memoize-one
var memoize_one_exports = {};
__export(memoize_one_exports, {
  default: () => memoize_one_default
});
__reExport(memoize_one_exports, memoize_one_star);
import * as defaultImport from "memoize-one";
import * as memoize_one_star from "memoize-one";
var memoize_one_default = "default" in defaultImport ? defaultImport.default : defaultImport;

// src/utils/gridLayoutUtils.ts
var resetLayoutAction = (payload) => ({
  type: "structure/reset",
  payload
});
var updateItemAction = (payload) => ({
  type: "structure/update",
  payload
});
var getBlockItems = (state) => Object.values(state);
var filterSelectedItems = (state, selectedItems = []) => {
  const itemKeys = state.map(({ item }) => item);
  return selectedItems.filter(({ id }) => itemKeys.indexOf(id) >= 0);
};
var getStructureItem = (state, itemId) => state.find(({ item }) => item === itemId);
var getItem = (state, itemId) => state[itemId];
var hasPreviewItem = (state, itemId = "preview-old-item") => !!state[itemId];
var getFilteredStructure = (state, layoutStructure) => {
  const result = [];
  layoutStructure.forEach((item) => {
    result.push(state[item.item] || item);
  });
  if (hasPreviewItem(state)) {
    result.push(state["preview-old-item"]);
  }
  return result;
};
var initializeGridReducer = (layoutStructure = []) => {
  const gridLayout = {};
  layoutStructure.forEach((item) => {
    gridLayout[item.item] = item;
  });
  return gridLayout;
};
var gridReducer = (state, action) => {
  switch (action.type) {
    // TODO: handle more actions (above), to further reduce size of grid code
    // TODO: expand set of reduced state, to limit amount of non-layout data tracked in layout component
    case "structure/reset":
      return initializeGridReducer(action.payload);
    case "structure/update":
      return __spreadProps(__spreadValues({}, state), {
        [action.payload.item]: action.payload
      });
    default:
      return state;
  }
};

// src/utils/gridOrderUtils.ts
var trackItemsByCoordinate = (layout) => {
  const result = {};
  layout.forEach((item) => {
    var _a, _b;
    (_b = result[_a = item.position.y]) != null ? _b : result[_a] = {};
    result[item.position.y][item.position.x] = item;
  });
  return result;
};
var getItemsInCoordinateRange = ({
  itemCoordinateMap,
  range
}) => {
  if (!range) {
    return Object.values(itemCoordinateMap).flatMap(Object.values);
  }
  const { x: fromX = 0, y: fromY = 0 } = range.from || {};
  const {
    x: toX = Number.POSITIVE_INFINITY,
    y: toY = Number.POSITIVE_INFINITY
  } = range.to || {};
  return Object.keys(itemCoordinateMap).map((key) => parseInt(key, 10)).filter((yPos) => yPos >= fromY && yPos < toY).reduce((acc, yPos) => {
    Object.keys(itemCoordinateMap[yPos]).forEach((key) => {
      const xPos = parseInt(key, 10);
      if (xPos >= fromX && xPos < toX) {
        acc.push(itemCoordinateMap[yPos][xPos]);
      }
    });
    return acc;
  }, []);
};
var getVizAtCoord = ({
  x,
  y,
  nodes
}) => nodes[x][y].find((viz) => viz.position.x === x && viz.position.y === y);
var isFullEdge = ({
  x,
  y,
  nodes,
  gridWidth,
  gridHeight,
  type
}) => {
  if (x === gridWidth || y === gridHeight) {
    return true;
  }
  const originViz = getVizAtCoord({ x, y, nodes });
  if (!originViz) {
    return false;
  }
  return isFullEdge({
    x: type === "horizontal" ? x + originViz.position.w : x,
    y: type === "vertical" ? y + originViz.position.h : y,
    nodes,
    gridWidth,
    gridHeight,
    type
  });
};
var splitGridByRows = ({
  nodes,
  layout,
  gridWidth,
  gridHeight,
  originViz,
  alwaysIncludeLastRow = false
}) => {
  const { x, y, h } = originViz.position;
  const itemCoordinateMap = trackItemsByCoordinate(layout);
  const newGrids = [];
  let lastHorizontalEdgeY = y;
  let nextViz = getVizAtCoord({ x, y: y + h, nodes });
  while (nextViz && nextViz.position.y < gridHeight) {
    if (isFullEdge({
      x,
      y: nextViz.position.y,
      nodes,
      gridWidth,
      gridHeight,
      type: "horizontal"
    })) {
      const rowItems = getItemsInCoordinateRange({
        itemCoordinateMap,
        range: {
          from: { y: lastHorizontalEdgeY },
          to: { y: nextViz.position.y }
        }
      });
      newGrids.push({
        nodes,
        layout,
        gridWidth,
        gridHeight: nextViz.position.y,
        origin: { x, y: lastHorizontalEdgeY },
        content: {
          // the width of the split is the full grid width less the origin of the split
          width: gridWidth - x,
          // the height of the split is the top of the next split less the origin of the current split
          height: nextViz.position.y - lastHorizontalEdgeY,
          items: rowItems
        }
      });
      lastHorizontalEdgeY = nextViz.position.y;
    }
    nextViz = getVizAtCoord({
      x,
      y: nextViz.position.y + nextViz.position.h,
      nodes
    });
  }
  if (newGrids.length === 0 && !alwaysIncludeLastRow) {
    return [];
  }
  const lastRowItems = getItemsInCoordinateRange({
    itemCoordinateMap,
    range: {
      from: { y: lastHorizontalEdgeY }
    }
  });
  newGrids.push({
    nodes,
    layout,
    gridWidth,
    gridHeight,
    origin: { x, y: lastHorizontalEdgeY },
    content: {
      width: gridWidth - x,
      height: gridHeight - lastHorizontalEdgeY,
      items: lastRowItems
    }
  });
  return newGrids;
};
var splitGridByColumns = ({
  nodes,
  layout,
  gridWidth,
  gridHeight,
  originViz
}) => {
  const { x, y, w } = originViz.position;
  const itemCoordinateMap = trackItemsByCoordinate(layout);
  const newGrids = [];
  let lastVerticalEdgeX = x;
  let nextViz = getVizAtCoord({ x: x + w, y, nodes });
  while (nextViz && nextViz.position.x < gridWidth) {
    if (isFullEdge({
      x: nextViz.position.x,
      y,
      nodes,
      gridWidth,
      gridHeight,
      type: "vertical"
    })) {
      const colItems = getItemsInCoordinateRange({
        itemCoordinateMap,
        range: {
          from: { x: lastVerticalEdgeX },
          to: { x: nextViz.position.x }
        }
      });
      newGrids.push({
        nodes,
        layout,
        gridWidth: nextViz.position.x,
        gridHeight,
        origin: { x: lastVerticalEdgeX, y },
        content: {
          // the width of the split is the left-edge of the next column less the origin of the current column
          width: nextViz.position.x - lastVerticalEdgeX,
          // the height of the split is the height of the grid less the origin of the column
          height: gridHeight - y,
          items: colItems
        }
      });
      lastVerticalEdgeX = nextViz.position.x;
    }
    nextViz = getVizAtCoord({
      x: nextViz.position.x + nextViz.position.w,
      y,
      nodes
    });
  }
  if (newGrids.length === 0) {
    return [];
  }
  const lastColumnItems = getItemsInCoordinateRange({
    itemCoordinateMap,
    range: {
      from: { x: lastVerticalEdgeX }
    }
  });
  newGrids.push({
    nodes,
    layout,
    gridWidth,
    gridHeight,
    origin: { x: lastVerticalEdgeX, y },
    content: {
      width: gridWidth - lastVerticalEdgeX,
      height: gridHeight - y,
      items: lastColumnItems
    }
  });
  return newGrids;
};
var splitGridByRowsAndColumns = ({
  origin,
  nodes,
  layout,
  newLayout,
  gridWidth,
  gridHeight
}) => {
  const originViz = getVizAtCoord({ x: origin.x, y: origin.y, nodes });
  if (!originViz) {
    throw new Error("Getting grid layout order failed");
  }
  if (originViz.position.x + originViz.position.w === gridWidth && originViz.position.y + originViz.position.h === gridHeight) {
    newLayout.push(originViz);
    return;
  }
  let newGrids = splitGridByRows({
    nodes,
    layout,
    originViz,
    gridHeight,
    gridWidth
  });
  newGrids.forEach(
    (gridInfo) => splitGridByRowsAndColumns(__spreadProps(__spreadValues({}, gridInfo), { newLayout }))
  );
  if (newGrids.length > 0) {
    return;
  }
  newGrids = splitGridByColumns({
    nodes,
    layout,
    originViz,
    gridHeight,
    gridWidth
  });
  newGrids.forEach(
    (gridInfo) => splitGridByRowsAndColumns(__spreadProps(__spreadValues({}, gridInfo), { newLayout }))
  );
  if (newGrids.length === 0) {
    const vizInCurrentGrid = [];
    layout.forEach((viz) => {
      const { x, y, w, h } = viz.position;
      if (x >= origin.x && x + w <= gridWidth && y >= origin.y && y + h <= gridHeight) {
        vizInCurrentGrid.push(viz);
      }
    });
    const sortedViz = vizInCurrentGrid.sort((viz1, viz2) => {
      if (viz1.position.y < viz2.position.y) {
        return -1;
      }
      if (viz1.position.y > viz2.position.y) {
        return 1;
      }
      if (viz1.position.x < viz2.position.x) {
        return -1;
      }
      if (viz1.position.x > viz2.position.x) {
        return 1;
      }
      return 0;
    });
    newLayout.push(...sortedViz);
  }
};
var getGridLayoutOrder = ({
  layout,
  canvasWidth,
  canvasHeight
}) => {
  const nodes = getNodes(layout);
  if (nodes === null || nodes[0] === void 0 || nodes[0][0] === void 0) {
    return layout;
  }
  const newLayout = [];
  try {
    splitGridByRowsAndColumns({
      origin: { x: 0, y: 0 },
      nodes,
      layout,
      newLayout,
      gridWidth: canvasWidth,
      gridHeight: canvasHeight
    });
  } catch (e) {
    return layout;
  }
  if (newLayout.length !== layout.length) {
    return layout;
  }
  return newLayout;
};

// src/utils/gridUtils.ts
var formatEdgeWrapper = ({
  edge,
  padding = 0
}) => {
  const wrapperStart = __spreadValues({}, edge.edgeStart);
  const wrapperEnd = __spreadValues({}, edge.edgeEnd);
  if (edge.orientation === "horizontal") {
    wrapperStart.x += padding;
    wrapperEnd.x -= padding;
    wrapperStart.y -= padding;
    wrapperEnd.y -= padding;
  } else {
    wrapperStart.y += padding;
    wrapperEnd.y -= padding;
    wrapperStart.x -= padding;
    wrapperEnd.x -= padding;
  }
  return __spreadProps(__spreadValues({}, edge), { edgeStart: wrapperStart, edgeEnd: wrapperEnd });
};
var getDimensions = ({
  edge,
  thickness
}) => {
  return {
    width: edge.orientation === "horizontal" ? edge.edgeEnd.x - edge.edgeStart.x : thickness,
    height: edge.orientation === "vertical" ? edge.edgeEnd.y - edge.edgeStart.y : thickness
  };
};
var isMouseOnEdge = ({
  edge,
  mousePosition,
  padding = 0
}) => {
  const formattedEdge = formatEdgeWrapper({ edge, padding });
  const { width, height } = getDimensions({
    edge: formattedEdge,
    thickness: 2 * padding
  });
  return mousePosition.x <= formattedEdge.edgeStart.x + width && mousePosition.x >= formattedEdge.edgeStart.x && mousePosition.y <= formattedEdge.edgeStart.y + height && mousePosition.y >= formattedEdge.edgeStart.y;
};
var computeNewVizPositionsGutterCase = ({
  edge,
  itemToMove,
  items
}) => {
  const updatedVisualizations = [];
  items.forEach((item) => {
    if (item.position.y >= edge.edgeStart.y) {
      const viz = item.item === itemToMove.item ? __spreadProps(__spreadValues({}, item), { item: "itemToRemove" }) : item;
      updatedVisualizations.push(__spreadProps(__spreadValues({}, viz), {
        position: __spreadProps(__spreadValues({}, viz.position), {
          y: viz.position.y + itemToMove.position.h
        })
      }));
    }
  });
  updatedVisualizations.push(__spreadProps(__spreadValues({}, itemToMove), {
    position: __spreadProps(__spreadValues({}, itemToMove.position), {
      // height is preserved
      y: edge.edgeStart.y,
      x: edge.edgeStart.x,
      w: edge.edgeEnd.x - edge.edgeStart.x
    })
  }));
  return updatedVisualizations;
};
var sizeToTakeFromViz = ({
  amountToTake,
  edge,
  minWidth,
  minHeight
}) => {
  const reduction = edge.visualizations.reduce(
    (acc, viz) => {
      if (acc === null) {
        return null;
      }
      if (edge.orientation === "vertical") {
        const widthToTake = Math.ceil(viz.position.w * amountToTake);
        if (viz.position.w - widthToTake < minWidth) {
          return null;
        }
        if (viz.position.x < edge.edgeStart.x) {
          return __spreadProps(__spreadValues({}, acc), {
            firstHalf: [...acc.firstHalf, widthToTake]
          });
        }
        return __spreadProps(__spreadValues({}, acc), {
          secondHalf: [...acc.secondHalf, widthToTake]
        });
      }
      const heightToTake = Math.ceil(viz.position.h * amountToTake);
      if (viz.position.h - heightToTake < minHeight) {
        return null;
      }
      if (viz.position.y < edge.edgeStart.y) {
        return __spreadProps(__spreadValues({}, acc), {
          firstHalf: [...acc.firstHalf, heightToTake]
        });
      }
      return __spreadProps(__spreadValues({}, acc), {
        secondHalf: [...acc.secondHalf, heightToTake]
      });
    },
    { firstHalf: [], secondHalf: [] }
  );
  if (reduction === null) {
    return {
      firstHalf: null,
      secondHalf: null
    };
  }
  return {
    firstHalf: reduction.firstHalf.length === 0 ? 0 : Math.min(...reduction.firstHalf),
    secondHalf: reduction.secondHalf.length === 0 ? 0 : Math.min(...reduction.secondHalf)
  };
};
var computeNewVizPositions = ({
  edge,
  itemToMove,
  firstHalf,
  secondHalf
}) => {
  const updatedVisualizations = [];
  edge.visualizations.forEach((edgeViz) => {
    const viz = edgeViz.item === itemToMove.item ? __spreadProps(__spreadValues({}, edgeViz), { item: "itemToRemove" }) : edgeViz;
    if (edge.orientation === "horizontal") {
      if (viz.position.y < edge.edgeStart.y) {
        updatedVisualizations.push(
          updateBlockItemSize({
            item: __spreadValues({}, viz),
            offset: { offsetY: -1 * firstHalf, offsetX: 0 },
            dir: "s"
          })
        );
      }
      if (viz.position.y >= edge.edgeStart.y) {
        updatedVisualizations.push(
          updateBlockItemSize({
            item: __spreadValues({}, viz),
            offset: { offsetY: secondHalf, offsetX: 0 },
            dir: "n"
          })
        );
      }
    } else {
      if (viz.position.x < edge.edgeStart.x) {
        updatedVisualizations.push(
          updateBlockItemSize({
            item: __spreadValues({}, viz),
            offset: { offsetX: -1 * firstHalf, offsetY: 0 },
            dir: "e"
          })
        );
      }
      if (viz.position.x >= edge.edgeStart.x) {
        updatedVisualizations.push(
          updateBlockItemSize({
            item: __spreadValues({}, viz),
            offset: { offsetX: secondHalf, offsetY: 0 },
            dir: "w"
          })
        );
      }
    }
  });
  updatedVisualizations.push(__spreadProps(__spreadValues({}, itemToMove), {
    position: {
      x: edge.edgeStart.x - (edge.orientation === "vertical" ? firstHalf : 0),
      y: edge.edgeStart.y - (edge.orientation === "horizontal" ? firstHalf : 0),
      w: edge.orientation === "vertical" ? firstHalf + secondHalf : edge.edgeEnd.x - edge.edgeStart.x,
      h: edge.orientation === "horizontal" ? firstHalf + secondHalf : edge.edgeEnd.y - edge.edgeStart.y
    }
  }));
  return updatedVisualizations;
};
var isResultingVizSizeGreaterThanMin = ({
  size,
  edgeOrientation
}) => {
  if (edgeOrientation === "vertical") {
    return size >= MIN_WIDTH_PX;
  }
  return size >= MIN_HEIGHT_PX;
};
var updateItems = ({
  updatedVisualizations,
  itemToMove,
  items,
  canvasWidth
}) => {
  const updatedItems = [
    ...items.filter(
      (item) => updatedVisualizations.find((viz) => viz.item === item.item) === void 0
    ),
    ...updatedVisualizations
  ];
  const updatedNeighbors = (0, dashboard_utils_exports.updateRemovedVizNeighbors)({
    itemToRemove: itemToMove,
    items: updatedItems,
    width: canvasWidth
  });
  const filteredUpdatedVisualizations = updatedVisualizations.filter(
    (viz) => updatedNeighbors.find((neighbor) => neighbor.item === viz.item) === void 0 && viz.item !== "itemToRemove"
  );
  const filteredItems = items.filter(
    (viz) => updatedNeighbors.find((neighbor) => neighbor.item === viz.item) === void 0 && filteredUpdatedVisualizations.find(
      (updatedViz) => updatedViz.item === viz.item
    ) === void 0 && viz.item !== itemToMove.item
  );
  return [
    ...filteredItems,
    ...filteredUpdatedVisualizations,
    ...updatedNeighbors
  ];
};
var isDropOnOwnEdge = ({
  edge,
  itemToMove
}) => edge.visualizations.find((viz) => viz.item === itemToMove.item) != null && (edge.orientation === "horizontal" && itemToMove.position.w === edge.edgeEnd.x - edge.edgeStart.x || edge.orientation === "vertical" && itemToMove.position.h === edge.edgeEnd.y - edge.edgeStart.y);
var updateDropOnEdge = ({
  edge,
  itemToMove,
  items,
  canvasWidth
}) => {
  if (isDropOnOwnEdge({ edge, itemToMove })) {
    return null;
  }
  const amountToTake = 1 / 3;
  let updatedVisualizations = [];
  if (edge.orientation === "horizontal" && edge.edgeEnd.x - edge.edgeStart.x === canvasWidth) {
    updatedVisualizations = computeNewVizPositionsGutterCase({
      edge,
      itemToMove,
      items
    });
  } else {
    const { firstHalf, secondHalf } = sizeToTakeFromViz({
      amountToTake,
      edge,
      minWidth: MIN_WIDTH_PX,
      minHeight: MIN_HEIGHT_PX
    });
    if (firstHalf === null || secondHalf === null || // Check if the moved viz resulting size will be greater than the minimum width/height
    // This particular case occurs when adding to the top and bottom canvas edge
    !isResultingVizSizeGreaterThanMin({
      size: firstHalf + secondHalf,
      edgeOrientation: edge.orientation
    })) {
      return null;
    }
    updatedVisualizations = computeNewVizPositions({
      edge,
      itemToMove,
      firstHalf,
      secondHalf
    });
  }
  const updatedItemToMove = updatedVisualizations.find((viz) => viz.item === "itemToRemove") || itemToMove;
  return { updatedVisualizations, updatedItemToMove };
};
var positionInItemBoundary = ({
  item,
  position
}) => {
  return position.x >= item.position.x && position.x <= item.position.x + item.position.w && position.y >= item.position.y && position.y <= item.position.y + item.position.h;
};
var findQuadrant = ({
  item,
  position
}) => {
  if (positionInItemBoundary({ item, position })) {
    let { x, y } = position;
    x -= item.position.x;
    y -= item.position.y;
    const slope = item.position.h / item.position.w;
    let quadrant;
    if (y > slope * x) {
      quadrant = ["w", "s"];
    } else {
      quadrant = ["n", "e"];
    }
    return y < -slope * x + item.position.h ? quadrant[0] : quadrant[1];
  }
  return null;
};
var updateDropOnViz = ({
  itemToMove,
  itemToDropOn,
  direction
}) => {
  if ((direction === "e" || direction === "w") && itemToDropOn.position.w < MIN_WIDTH_PX * 2 || (direction === "n" || direction === "s") && itemToDropOn.position.h < MIN_WIDTH_PX * 2) {
    return null;
  }
  let vizToMove = __spreadProps(__spreadValues({}, itemToMove), {
    position: itemToDropOn.position
  });
  let vizTarget = __spreadValues({}, itemToDropOn);
  switch (direction) {
    case "n":
      vizToMove = updateBlockItemSize({
        item: vizToMove,
        offset: {
          offsetY: -Math.floor(vizToMove.position.h / 2),
          offsetX: 0
        },
        dir: "s"
      });
      vizTarget = updateBlockItemSize({
        item: vizTarget,
        offset: {
          offsetY: Math.ceil(vizTarget.position.h / 2),
          offsetX: 0
        },
        dir: "n"
      });
      break;
    case "s":
      vizToMove = updateBlockItemSize({
        item: vizToMove,
        offset: {
          offsetY: Math.floor(vizToMove.position.h / 2),
          offsetX: 0
        },
        dir: "n"
      });
      vizTarget = updateBlockItemSize({
        item: vizTarget,
        offset: {
          offsetY: -Math.ceil(vizTarget.position.h / 2),
          offsetX: 0
        },
        dir: "s"
      });
      break;
    case "w":
      vizToMove = updateBlockItemSize({
        item: vizToMove,
        offset: {
          offsetX: -Math.floor(vizToMove.position.w / 2),
          offsetY: 0
        },
        dir: "e"
      });
      vizTarget = updateBlockItemSize({
        item: vizTarget,
        offset: {
          offsetX: Math.ceil(vizTarget.position.w / 2),
          offsetY: 0
        },
        dir: "w"
      });
      break;
    case "e":
      vizToMove = updateBlockItemSize({
        item: vizToMove,
        offset: {
          offsetX: Math.floor(vizToMove.position.w / 2),
          offsetY: 0
        },
        dir: "w"
      });
      vizTarget = updateBlockItemSize({
        item: vizTarget,
        offset: {
          offsetX: -Math.ceil(vizTarget.position.w / 2),
          offsetY: 0
        },
        dir: "e"
      });
      break;
    default:
      vizToMove = itemToMove;
      break;
  }
  return [vizToMove, vizTarget];
};
var previewDropOnEdge = ({
  edge,
  itemToMove,
  items,
  canvasWidth
}) => {
  const updatedItems = updateDropOnEdge({
    edge,
    itemToMove,
    items,
    canvasWidth
  });
  if (!updatedItems) {
    return null;
  }
  let { updatedVisualizations } = updatedItems;
  const { updatedItemToMove } = updatedItems;
  updatedVisualizations = updatedVisualizations.filter(
    (viz) => !(0, lodash_exports.isEqual)(viz, updatedItemToMove)
  );
  return [
    ...items.filter(
      (item) => updatedVisualizations.find((viz) => viz.item === item.item) === void 0
    ),
    ...updatedVisualizations,
    __spreadProps(__spreadValues({}, updatedItemToMove), {
      item: "preview-old-item"
    })
  ];
};
var previewDropOnViz = ({
  itemToDropOn,
  itemToMove,
  items,
  direction
}) => {
  const updatedItems = updateDropOnViz({
    itemToMove,
    itemToDropOn,
    direction
  });
  if (!updatedItems) {
    return null;
  }
  return [
    ...items.filter(
      (item) => updatedItems.find((viz) => viz.item === item.item) === void 0
    ),
    ...updatedItems,
    __spreadProps(__spreadValues({}, itemToMove), {
      item: "preview-old-item"
    })
  ];
};
var findItemsOutsideBoundary = ({
  items,
  boundary
}) => {
  const invalidItems = [];
  const message = (0, i18n_exports._)("is outside of canvas bounds");
  items.forEach((item) => {
    if (item.position.x < boundary.x || item.position.x + item.position.w > boundary.x + boundary.w || item.position.y < boundary.y || item.position.y + item.position.h > boundary.y + boundary.h) {
      invalidItems.push({
        itemId: item.item,
        messages: [`"${item.item}" ${message}`]
      });
    }
  });
  return invalidItems;
};
var getItemEdges = ({
  item,
  belongsTo = {}
}) => {
  const topEdge = {
    start: { x: item.x, y: item.y },
    end: { x: item.x + item.w, y: item.y },
    orientation: "horizontal",
    type: "above",
    belongsTo
  };
  const bottomEdge = {
    start: { x: item.x, y: item.y + item.h },
    end: { x: item.x + item.w, y: item.y + item.h },
    orientation: "horizontal",
    type: "below",
    belongsTo
  };
  const leftEdge = {
    start: { x: item.x, y: item.y },
    end: { x: item.x, y: item.y + item.h },
    orientation: "vertical",
    type: "left",
    belongsTo
  };
  const rightEdge = {
    start: { x: item.x + item.w, y: item.y },
    end: { x: item.x + item.w, y: item.y + item.h },
    orientation: "vertical",
    type: "right",
    belongsTo
  };
  return [topEdge, bottomEdge, leftEdge, rightEdge];
};
var createEdges = ({
  items,
  canvasBounds
}) => {
  const edges = [];
  items.forEach((item) => {
    edges.push(
      ...getItemEdges({ item: item.position, belongsTo: item.position })
    );
  });
  edges.push(...getItemEdges({ item: canvasBounds }));
  return edges;
};
var findInvalidItemEdges = ({
  item,
  edges
}) => {
  const invalidEdges = [];
  const curEdges = getItemEdges({
    item: item.position,
    belongsTo: item.position
  });
  curEdges.forEach((curEdge) => {
    const overlappingEdge = edges.find((edge) => {
      if (curEdge.orientation !== edge.orientation || (0, lodash_exports.isEqual)(curEdge.belongsTo, edge.belongsTo)) {
        return false;
      }
      if (curEdge.orientation === "horizontal" && curEdge.start.y === edge.start.y) {
        if (curEdge.start.x >= edge.end.x || edge.start.x >= curEdge.end.x) {
          return false;
        }
        return true;
      }
      if (curEdge.orientation === "vertical" && curEdge.start.x === edge.start.x) {
        if (curEdge.start.y >= edge.end.y || edge.start.y >= curEdge.end.y) {
          return false;
        }
        return true;
      }
      return false;
    });
    if (overlappingEdge === void 0) {
      const positionString = curEdge.orientation === "horizontal" ? `y=${curEdge.start.y}` : `x=${curEdge.start.x}`;
      const positionType = curEdge.type === "below" || curEdge.type === "above" ? curEdge.type : `to its ${curEdge.type}`;
      invalidEdges.push(
        (0, i18n_exports._)(
          `"${item.item}" expected a viz or canvas edge directly ${positionType} at ${positionString}`
        )
      );
    }
  });
  return invalidEdges;
};
var findInvalidItems = ({
  items,
  canvasBounds
}) => {
  const invalidItems = [];
  const edges = createEdges({ items, canvasBounds });
  items.forEach((item) => {
    const invalidEdges = findInvalidItemEdges({ item, edges });
    if (invalidEdges.length > 0) {
      invalidItems.push({
        itemId: item.item,
        messages: invalidEdges
      });
    }
  });
  return invalidItems;
};
var validateLayoutStructure = ({
  layout,
  canvasBounds
}) => {
  if (!layout) {
    return [];
  }
  const res = findItemsOutsideBoundary({
    items: layout,
    boundary: canvasBounds
  });
  if (!(0, lodash_exports.isEmpty)(res)) {
    return res;
  }
  return findInvalidItems({ items: layout, canvasBounds });
};
var isVerticallyAligned = (viz, another) => viz.position.y === another.position.y && viz.position.h === another.position.h;
var isHorizontallyAligned = (viz, another) => viz.position.x === another.position.x && viz.position.w === another.position.w;
var isInvalidAdjacentVizDrop = ({
  itemToMove,
  itemToDropOn,
  direction,
  visualizations
}) => {
  if (direction === "e" && (0, dashboard_utils_exports.isLeftNeighbor)(itemToMove, itemToDropOn) && isVerticallyAligned(itemToMove, itemToDropOn)) {
    return true;
  }
  if (direction === "w" && (0, dashboard_utils_exports.isRightNeighbor)(itemToMove, itemToDropOn) && isVerticallyAligned(itemToMove, itemToDropOn)) {
    return true;
  }
  if (direction === "e" || direction === "w") {
    return false;
  }
  const { leftNeighbors, rightNeighbors } = (0, dashboard_utils_exports.findHorizontalNeighbors)({
    item: itemToMove,
    visualizations
  });
  if (leftNeighbors.length > 0 || rightNeighbors.length > 0) {
    return false;
  }
  if (direction === "n" && (0, dashboard_utils_exports.isBottomNeighbor)(itemToMove, itemToDropOn) && isHorizontallyAligned(itemToMove, itemToDropOn)) {
    return true;
  }
  if (direction === "s" && (0, dashboard_utils_exports.isTopNeighbor)(itemToMove, itemToDropOn) && isHorizontallyAligned(itemToMove, itemToDropOn)) {
    return true;
  }
  return false;
};

// src/utils/splitGridStructure.ts
var DEFAULT_BLOCK_ITEM = {
  item: "",
  position: { x: -1, y: -1, w: -1, h: -1 }
};
var translateBlockItem = (item, translation) => __spreadProps(__spreadValues({}, item), {
  position: __spreadProps(__spreadValues({}, item.position), {
    x: item.position.x + translation.x,
    y: item.position.y + translation.y
  })
});
var translateBlockItems = (items, translation) => items.map((item) => translateBlockItem(item, translation));
var translateGridSplit = (gridSplit, translation) => __spreadProps(__spreadValues({}, gridSplit), {
  layout: translateBlockItems(gridSplit.layout, translation),
  content: __spreadProps(__spreadValues({}, gridSplit.content), {
    items: translateBlockItems(gridSplit.content.items, translation)
  }),
  origin: {
    x: gridSplit.origin.x + translation.x,
    y: gridSplit.origin.y + translation.y
  }
});
var splitGrid = ({
  layoutStructure,
  idToRemove,
  splitConfig = "both",
  alwaysIncludeLastRow = false
}) => {
  const gridInfo = {
    width: 0,
    height: 0,
    originItem: DEFAULT_BLOCK_ITEM,
    nodes: {},
    splitByRows: [],
    splitByColumns: [],
    itemToRemove: DEFAULT_BLOCK_ITEM
  };
  if (layoutStructure.length === 0) {
    return gridInfo;
  }
  let originX = Number.MAX_SAFE_INTEGER;
  let originY = Number.MAX_SAFE_INTEGER;
  layoutStructure.forEach((item) => {
    const { x, y, w, h } = item.position;
    gridInfo.width = Math.max(gridInfo.width, x + w);
    gridInfo.height = Math.max(gridInfo.height, y + h);
    if (x <= originX && y <= originY) {
      originX = x;
      originY = y;
      gridInfo.originItem = item;
    }
    [
      [x, y],
      [x + w, y],
      [x, y + h],
      [x + w, y + h]
    ].forEach(([cornerX, cornerY]) => {
      var _a, _b, _c, _d;
      (_b = (_a = gridInfo.nodes)[cornerX]) != null ? _b : _a[cornerX] = {};
      (_d = (_c = gridInfo.nodes[cornerX])[cornerY]) != null ? _d : _c[cornerY] = [];
      gridInfo.nodes[cornerX][cornerY].push(item);
    });
    if (idToRemove === item.item) {
      gridInfo.itemToRemove = item;
    }
  });
  gridInfo.width = Math.max(0, gridInfo.width - originX);
  gridInfo.height = Math.max(0, gridInfo.height - originY);
  if (splitConfig === "both" || splitConfig === "rows") {
    gridInfo.splitByRows = splitGridByRows({
      nodes: gridInfo.nodes,
      layout: layoutStructure,
      gridWidth: gridInfo.width,
      gridHeight: gridInfo.height,
      originViz: gridInfo.originItem,
      alwaysIncludeLastRow
    });
  }
  if (splitConfig === "both" || splitConfig === "columns") {
    gridInfo.splitByColumns = splitGridByColumns({
      nodes: gridInfo.nodes,
      layout: layoutStructure,
      gridWidth: gridInfo.width,
      gridHeight: gridInfo.height,
      originViz: gridInfo.originItem
    });
  }
  return gridInfo;
};

// src/utils/getContainingRowColumn.ts
var findContainingRow = ({ splits, itemPosition }) => {
  const edgeTop = itemPosition.y;
  const edgeBottom = itemPosition.y + itemPosition.h;
  return splits.find(
    ({ origin, content }) => (
      // Find the first row split which starts above the item in question and ends below it
      origin.y <= edgeTop && origin.y + content.height >= edgeBottom
    )
  );
};
var findContainingColumn = ({ splits, itemPosition }) => {
  const edgeLeft = itemPosition.x;
  const edgeRight = itemPosition.x + itemPosition.w;
  return splits.find(
    ({ origin, content }) => (
      // Find the first column split which starts before the item in question and ends after it
      origin.x <= edgeLeft && origin.x + content.width >= edgeRight
    )
  );
};
var getContainingRowColumnRecursive = ({
  findContainerFn,
  itemToContain,
  previousResult,
  structure
}) => {
  const splittingByRows = findContainerFn === findContainingRow;
  const splitGridInfo = splitGrid({
    layoutStructure: structure,
    idToRemove: itemToContain.item,
    splitConfig: splittingByRows ? "rows" : "columns",
    alwaysIncludeLastRow: true
  });
  const container = findContainerFn({
    splits: splittingByRows ? splitGridInfo.splitByRows : splitGridInfo.splitByColumns,
    itemPosition: splitGridInfo.itemToRemove.position
  });
  if (!container || container.content.items.length <= 1) {
    return previousResult;
  }
  const containerTranslation = {
    x: -container.origin.x,
    y: -container.origin.y
  };
  const translatedContainer = translateGridSplit(
    container,
    containerTranslation
  );
  const nextFindContainerFn = splittingByRows ? findContainingColumn : findContainingRow;
  return translateGridSplit(
    getContainingRowColumnRecursive({
      structure: translatedContainer.content.items,
      itemToContain,
      findContainerFn: nextFindContainerFn,
      previousResult: translatedContainer
    }),
    // NOSONAR
    container.origin
  );
};
var getContainingRowColumn = ({
  structure,
  itemToContain
}) => getContainingRowColumnRecursive({
  structure,
  itemToContain,
  findContainerFn: findContainingRow
});

// src/utils/gridLayoutAutoResize.ts
var recursivelyApplyHorizontalResize = ({
  container: {
    content: { items, width: totalWidth },
    origin
  },
  newSize,
  position,
  translation
}) => {
  const changes = {};
  const { splitByRows, splitByColumns } = splitGrid({
    layoutStructure: translateBlockItems(items, {
      x: -origin.x,
      y: -origin.y
    })
  });
  splitByRows.forEach((container) => {
    Object.assign(
      changes,
      recursivelyApplyHorizontalResize({
        container,
        newSize,
        position,
        translation: {
          x: translation.x + origin.x,
          y: translation.y + origin.y
        }
      })
    );
  });
  let colPosition = position;
  splitByColumns.forEach((container, idx) => {
    let newColWidth = Math.floor(
      newSize * (container.content.width / totalWidth)
    );
    if (
      // If resizing the last column
      idx === splitByColumns.length - 1 && // and there's a fractional correction needed
      colPosition + newColWidth !== position + newSize
    ) {
      newColWidth = position + newSize - colPosition;
    }
    Object.assign(
      changes,
      recursivelyApplyHorizontalResize({
        container,
        newSize: newColWidth,
        position: colPosition,
        translation: {
          x: translation.x + origin.x,
          y: translation.y + origin.y
        }
      })
    );
    colPosition += newColWidth;
  });
  if (splitByColumns.length === 0 && splitByRows.length === 0) {
    items.forEach((item) => {
      const { position: itemPos } = translateBlockItem(item, translation);
      changes[item.item] = __spreadProps(__spreadValues({}, item), {
        position: __spreadProps(__spreadValues({}, itemPos), {
          x: position,
          w: newSize
        })
      });
    });
  }
  return changes;
};
var recursivelyApplyVerticalResize = ({
  container: {
    content: { items, height: totalHeight },
    origin
  },
  newSize,
  position,
  translation
}) => {
  const changes = {};
  const { splitByRows, splitByColumns } = splitGrid({
    layoutStructure: translateBlockItems(items, {
      x: -origin.x,
      y: -origin.y
    })
  });
  splitByColumns.forEach((container) => {
    Object.assign(
      changes,
      recursivelyApplyVerticalResize({
        container,
        newSize,
        position,
        translation: {
          x: translation.x + origin.x,
          y: translation.y + origin.y
        }
      })
    );
  });
  let rowPosition = position;
  splitByRows.forEach((container, idx) => {
    let newRowHeight = Math.floor(
      newSize * (container.content.height / totalHeight)
    );
    if (
      // If resizing the last row
      idx === splitByRows.length - 1 && // and there's a fractional correction needed
      rowPosition + newRowHeight !== position + newSize
    ) {
      newRowHeight = position + newSize - rowPosition;
    }
    Object.assign(
      changes,
      recursivelyApplyVerticalResize({
        container,
        newSize: newRowHeight,
        position: rowPosition,
        translation: {
          x: translation.x + origin.x,
          y: translation.y + origin.y
        }
      })
    );
    rowPosition += newRowHeight;
  });
  if (splitByColumns.length === 0 && splitByRows.length === 0) {
    items.forEach((item) => {
      const { position: itemPos } = translateBlockItem(item, translation);
      changes[item.item] = __spreadProps(__spreadValues({}, item), {
        position: __spreadProps(__spreadValues({}, itemPos), {
          y: position,
          h: newSize
        })
      });
    });
  }
  return changes;
};
var HorizontalResizeConfig = {
  dimension: "width",
  blockSizeDimension: "w",
  blockPosDimension: "x",
  splitGridConfig: "columns",
  splitGridKey: "splitByColumns",
  recursiveResizeFn: recursivelyApplyHorizontalResize
};
var VerticalResizeConfig = {
  dimension: "height",
  blockSizeDimension: "h",
  blockPosDimension: "y",
  splitGridConfig: "rows",
  splitGridKey: "splitByRows",
  recursiveResizeFn: recursivelyApplyVerticalResize
};
var handleResizing = ({
  container,
  itemToRemove,
  config: {
    dimension,
    blockSizeDimension,
    blockPosDimension,
    splitGridConfig,
    splitGridKey,
    recursiveResizeFn
  }
}) => {
  if (!container) {
    return {};
  }
  const {
    origin,
    content: {
      // Items in the container which are being resized
      items: impactedItems,
      // The important container dimension (width or height) being recalculated
      [dimension]: containerSize
    }
  } = container;
  const negatedOrigin = { x: -origin.x, y: -origin.y };
  if (impactedItems.length < 2) {
    return {};
  }
  if (impactedItems.length === 2) {
    const itemToUpdate = impactedItems[0].item !== itemToRemove.item ? impactedItems[0] : impactedItems[1];
    return {
      [itemToUpdate.item]: __spreadProps(__spreadValues({}, itemToUpdate), {
        position: __spreadProps(__spreadValues({}, itemToUpdate.position), {
          [blockPosDimension]: origin[blockPosDimension],
          [blockSizeDimension]: containerSize
        })
      })
    };
  }
  const pendingChanges = {};
  const {
    [splitGridKey]: splitByResizeDimension,
    itemToRemove: {
      position: { [blockSizeDimension]: itemToRemoveSize }
    }
  } = splitGrid({
    layoutStructure: translateBlockItems(impactedItems, negatedOrigin),
    splitConfig: splitGridConfig,
    idToRemove: itemToRemove.item
  });
  const pctToDistribute = itemToRemoveSize / (splitByResizeDimension.length - 1) / containerSize;
  let itemPosition = origin[blockPosDimension];
  const containerEdge = origin[blockPosDimension] + containerSize;
  let lastFRCIdx = splitByResizeDimension.length - 2;
  splitByResizeDimension.forEach((split, idx) => {
    if (split.content.items[0].item === itemToRemove.item) {
      lastFRCIdx += 1;
      return;
    }
    const splitSize = split.content[dimension] / containerSize;
    let newSize = Math.floor(containerSize * (pctToDistribute + splitSize));
    if (idx === lastFRCIdx && itemPosition + newSize !== containerEdge) {
      newSize = containerEdge - itemPosition;
    }
    Object.values(
      recursiveResizeFn({
        container: split,
        newSize,
        position: itemPosition,
        translation: origin
      })
    ).forEach((change) => {
      pendingChanges[change.item] = change;
    });
    itemPosition += newSize;
  });
  return pendingChanges;
};
var handleHorizontalResizing = (args) => handleResizing(__spreadProps(__spreadValues({}, args), { config: HorizontalResizeConfig }));
var handleVerticalResizing = (args) => handleResizing(__spreadProps(__spreadValues({}, args), { config: VerticalResizeConfig }));

// src/utils/gridLayoutShowHide.ts
var cloneLayoutStructure = (source) => {
  if (!source.length) {
    return [];
  }
  return (0, lodash_exports.cloneDeep)(source);
};
var createStructureMap = (structure) => Object.fromEntries(structure.map((item) => [item.item, item]));
var getWidthFromStructure = (structure) => {
  const [min, max] = structure.reduce(
    ([minX, maxX], { position: { x, w } }) => [
      // minX = either block item left or current minX (initial: MAX_SAFE_INTEGER)
      Math.min(minX, x),
      // maxX = either block item right or current maxX (initial: 0)
      Math.max(maxX, x + w)
    ],
    [Number.MAX_SAFE_INTEGER, 0]
  );
  return Math.max(max - min, 0);
};
var popTopLeftBlock = ({
  itemsToRemove,
  itemMap
}) => {
  let result = itemMap[itemsToRemove[0].item];
  let resultIdx = 0;
  itemsToRemove.forEach(({ item }, idx) => {
    const itemMapItem = itemMap[item];
    if (result.position.y > itemMapItem.position.y) {
      result = itemMapItem;
      resultIdx = idx;
      return;
    }
    if (result.position.y === itemMapItem.position.y && result.position.x > itemMapItem.position.x) {
      result = itemMapItem;
      resultIdx = idx;
    }
  });
  itemsToRemove.splice(resultIdx, 1);
  return result;
};
var applyUpdatesToStructure = ({
  structure,
  itemMap,
  itemToRemove,
  updatesMap
}) => {
  let idxToRemove = -1;
  structure.forEach((item, idx) => {
    if (item.item === itemToRemove.item) {
      idxToRemove = idx;
    } else if (item.item in updatesMap) {
      structure[idx] = updatesMap[item.item];
      itemMap[item.item] = updatesMap[item.item];
    }
  });
  if (idxToRemove >= 0) {
    structure.splice(idxToRemove, 1);
  }
  delete itemMap[itemToRemove.item];
};
var baseCaseRemoval = ({
  structure,
  itemMap,
  itemToRemove,
  width
}) => {
  const updates = (0, dashboard_utils_exports.updateRemovedVizNeighbors)({
    items: structure,
    itemToRemove,
    width
  });
  const updatesMap = createStructureMap(updates);
  applyUpdatesToStructure({
    structure,
    itemMap,
    itemToRemove,
    updatesMap
  });
};
var handleRecursiveItemRemoval = ({
  structure,
  itemsToRemove,
  itemMap
}) => {
  var _a, _b;
  if (itemsToRemove.length === 0) {
    return;
  }
  const itemToRemove = popTopLeftBlock({ itemsToRemove, itemMap });
  const container = getContainingRowColumn({
    structure,
    itemToContain: itemToRemove
  });
  if (!container || container.content.items.length <= 1) {
    baseCaseRemoval({
      structure,
      itemMap,
      width: (_a = container == null ? void 0 : container.gridWidth) != null ? _a : getWidthFromStructure(structure),
      itemToRemove
    });
  } else {
    const { width, height } = container.content;
    let updatesMap;
    if (itemToRemove.position.w === width) {
      updatesMap = handleVerticalResizing({ container, itemToRemove });
    } else if (itemToRemove.position.h === height) {
      updatesMap = handleHorizontalResizing({ container, itemToRemove });
    } else {
      baseCaseRemoval({
        structure,
        width: (_b = container == null ? void 0 : container.gridWidth) != null ? _b : getWidthFromStructure(structure),
        itemToRemove,
        itemMap
      });
    }
    if (updatesMap) {
      applyUpdatesToStructure({
        structure,
        itemMap,
        itemToRemove,
        updatesMap
      });
    }
  }
  handleRecursiveItemRemoval({ structure, itemsToRemove, itemMap });
};
var gridLayoutShowHideReflow = (initialStructure, itemsToRemove) => {
  if (!itemsToRemove.length) {
    return initialStructure;
  }
  const newStructure = cloneLayoutStructure(
    initialStructure
  );
  const clonedItemsToRemove = cloneLayoutStructure(
    itemsToRemove
  );
  const itemMap = createStructureMap(newStructure);
  handleRecursiveItemRemoval({
    structure: newStructure,
    itemsToRemove: clonedItemsToRemove,
    itemMap
  });
  return newStructure;
};

// src/GridLayout.tsx
var HANDLE_DIRECTIONS = [];
var MemoizedBlockItem = react_default.memo(
  ResponsiveBlockItem_default,
  (prevProps, nextProps) => {
    if (nextProps.y + nextProps.h === nextProps.canvasHeight) {
      return (0, lodash_exports.isEqual)(prevProps, nextProps);
    }
    const prevPropsWithoutCanvasHeight = (0, lodash_exports.omit)(prevProps, "canvasHeight");
    const nextPropsWithoutCanvasHeight = (0, lodash_exports.omit)(nextProps, "canvasHeight");
    return (0, lodash_exports.isEqual)(
      prevPropsWithoutCanvasHeight,
      nextPropsWithoutCanvasHeight
    );
  }
);
var MemoizedBlockOutline = react_default.memo(ResponsiveBlockOutline_default);
var MemoizedEdge = react_default.memo(Edge);
var renderBlockItems = memoize_one_default(
  ({
    layoutStructure,
    renderLayoutItem,
    handleItemSelected,
    isBlockItemMoving,
    selectedItem,
    mode,
    errors,
    padding,
    canvasHeight
  }) => {
    return layoutStructure.map((item) => {
      const key = item.item;
      if (key === "preview-old-item") {
        return null;
      }
      const isSelected = (selectedItem == null ? void 0 : selectedItem.id) === key;
      let appearance = mode === "view" || !selectedItem || isSelected ? "highlighted" : "visible";
      if (isSelected && isBlockItemMoving) {
        appearance = "hidden";
      }
      let errorMessages;
      if (errors) {
        const vizErrors = errors.find(({ itemId }) => itemId === key);
        errorMessages = vizErrors == null ? void 0 : vizErrors.messages;
      }
      return /* @__PURE__ */ react_default.createElement(
        MemoizedBlockItem,
        {
          key,
          itemId: key,
          type: item.type,
          x: item.position.x,
          y: item.position.y,
          h: item.position.h,
          w: item.position.w,
          padding,
          canvasHeight,
          renderLayoutItem,
          onItemSelected: handleItemSelected,
          appearance,
          errorMessages
        }
      );
    });
  },
  lodash_exports.isEqual
);
var BOTTOM_EDGE_MIN_SPEED = 2;
var BOTTOM_EDGE_MAX_SPEED = 4;
var BOTTOM_RESIZING_SCROLL_SPEED = 50;
var CANVAS_BOTTOM_PADDING = 8;
var BaseGridLayout = ({
  layoutApiRef,
  onLayoutItemsSelect,
  selectedItems: selectedLayoutItems,
  layoutStructure,
  onLayoutStructureChange,
  options = {},
  renderLayoutItem,
  mode,
  showGrid,
  scale,
  setCanvasHeight,
  canvasWidth
}) => {
  var _a;
  const canvasRef = (0, react_exports.useRef)(null);
  const canvasContext = (0, dashboard_context_exports.useCanvas)();
  const keyboardListener = (0, dashboard_context_exports.useKeyboardListener)();
  const userMessage = (_a = (0, dashboard_context_exports.useUserMessageAPI)()) != null ? _a : dashboard_utils_exports.noop;
  const telemetryAPI = (0, dashboard_telemetry_exports.useTelemetryApi)();
  const { enableGridLayoutErrors, enableGridLayoutCssScaling } = (0, dashboard_context_exports.useFeatureFlags)();
  const { backgroundColor, gutterSize = gridLayoutOptions.gutterSize } = options;
  const panelPadding = gutterSize / 2;
  const [gridState, dispatch] = (0, react_exports.useReducer)(
    gridReducer,
    layoutStructure,
    initializeGridReducer
  );
  const maxHeight = computeMaxHeight(getBlockItems(gridState));
  (0, react_exports.useLayoutEffect)(() => {
    setCanvasHeight(maxHeight);
  }, [setCanvasHeight, maxHeight]);
  const [isBlockItemMoving, setIsBlockItemMoving] = (0, react_exports.useState)(false);
  const [edges, setEdges] = (0, react_exports.useState)(
    computeEdges({
      layout: layoutStructure,
      canvasWidth,
      canvasHeight: computeMaxHeight(layoutStructure)
      // need to recompute height on layout change
    })
  );
  const [snappableEdges, setSnappableEdges] = (0, react_exports.useState)([]);
  const [scrollToBottom, setScrollToBottom] = (0, react_exports.useState)(false);
  const [mousePosition, setMousePosition] = (0, react_exports.useState)(null);
  const [isItemAdded, setIsItemAdded] = (0, react_exports.useState)(false);
  const [edgesBeforeMove, setEdgesBeforeMove] = (0, react_exports.useState)(
    null
  );
  const [invalidEdgeId, setInvalidEdgeId] = (0, react_exports.useState)(null);
  const [hoveredEdge, setHoveredEdge] = (0, react_exports.useState)(null);
  const [initialItemToMove, setInitialItemToMove] = (0, react_exports.useState)(null);
  const [hoveredBlock, setHoveredBlock] = (0, react_exports.useState)(
    null
  );
  const [hoveredQuadrant, setHoveredQuadrant] = (0, react_exports.useState)(
    null
  );
  const [, setForceUpdate] = (0, react_exports.useState)(0);
  const [selectedItemsForEdge, setSelectedItemsForEdge] = (0, react_exports.useState)(
    []
  );
  const [isInvalidVizDrop, setIsInvalidVizDrop] = (0, react_exports.useState)(false);
  const [showPreviewPlaceholder, setShowPreviewPlaceholder] = (0, react_exports.useState)(true);
  const [layoutErrors, setLayoutErrors] = (0, react_exports.useState)([]);
  const isDraggingEdge = (0, react_exports.useRef)(false);
  const delayPreviewEdge = (0, react_exports.useRef)(null);
  const delayPreviewViz = (0, react_exports.useRef)(null);
  const mouseDownEdge = (0, react_exports.useRef)(null);
  const edgeMouseDownPosition = (0, react_exports.useRef)(null);
  const edgeBoundaries = (0, react_exports.useRef)(null);
  const edgesInBoundary = (0, react_exports.useRef)(null);
  const cumulativeScrollIncrease = (0, react_exports.useRef)(0);
  const edgeVelocity = (0, react_exports.useRef)(0);
  const containsPreviewItem = hasPreviewItem(gridState);
  const sendTelemetry = (0, react_exports.useCallback)(
    (eventData) => {
      telemetryAPI.collect(eventData);
    },
    [telemetryAPI]
  );
  const onItemAdded = (0, react_exports.useCallback)(() => {
    setIsItemAdded(true);
  }, [setIsItemAdded]);
  const layoutApi = (0, react_exports.useRef)(null);
  const layoutStructureRef = (0, react_exports.useRef)(layoutStructure);
  const firstSelectedItemStructure = (0, react_exports.useMemo)(
    () => (selectedLayoutItems == null ? void 0 : selectedLayoutItems.length) ? getItem(gridState, selectedLayoutItems[0].id) : null,
    [gridState, selectedLayoutItems]
  );
  const initializeLayoutApi = (0, react_exports.useCallback)(() => {
    const getCanvasDomElement = () => {
      return canvasRef.current;
    };
    if (layoutApi.current === null) {
      layoutApi.current = new GridLayoutApi_default({
        layoutStructureRef,
        options,
        userMessage,
        onItemAdded,
        getCanvasDomElement,
        telemetry: telemetryAPI
      });
    }
  }, [telemetryAPI, userMessage, onItemAdded, options]);
  const prevLayoutStructure = (0, dashboard_ui_exports.usePrevious)(layoutStructure);
  (0, react_exports.useEffect)(() => {
    if ((0, lodash_exports.isEqual)(prevLayoutStructure, layoutStructure)) {
      return;
    }
    dispatch(resetLayoutAction(layoutStructure));
    layoutStructureRef.current = enableGridLayoutCssScaling ? layoutStructure : scaleGridLayoutStructureByWidth({
      scale: 1 / scale,
      layout: layoutStructure
    });
  }, [
    enableGridLayoutCssScaling,
    layoutStructure,
    prevLayoutStructure,
    scale
  ]);
  initializeLayoutApi();
  (0, react_exports.useEffect)(() => {
    if (mode === "edit" && enableGridLayoutErrors) {
      const errors = validateLayoutStructure({
        layout: layoutStructure,
        canvasBounds: { x: 0, y: 0, w: canvasWidth, h: maxHeight }
      });
      setLayoutErrors(errors);
    } else if (mode === "view") {
      setLayoutErrors([]);
    }
  }, [layoutStructure, mode, enableGridLayoutErrors]);
  (0, react_exports.useLayoutEffect)(() => {
    layoutApiRef(layoutApi.current);
    return () => {
      layoutApiRef(null);
    };
  }, [layoutApiRef]);
  (0, react_exports.useEffect)(() => {
    if ((0, lodash_exports.isEqual)(prevLayoutStructure, layoutStructure)) {
      return;
    }
    dispatch(resetLayoutAction(layoutStructure));
    const canvasHeight = computeMaxHeight(layoutStructure);
    const prevCanvasHeight = computeMaxHeight(prevLayoutStructure || []);
    setEdges(
      computeEdges({
        layout: layoutStructure,
        canvasWidth,
        canvasHeight
        // need to recompute height on layout change
      })
    );
    if (prevLayoutStructure && canvasHeight > prevCanvasHeight && isItemAdded) {
      setScrollToBottom(true);
      setIsItemAdded(false);
    }
  }, [isItemAdded, layoutStructure, canvasWidth, prevLayoutStructure]);
  (0, react_exports.useEffect)(() => {
    if (scrollToBottom) {
      const layoutContainer = canvasContext.current;
      if (layoutContainer) {
        layoutContainer.scrollTop = layoutContainer.scrollHeight;
      }
      setScrollToBottom(false);
    }
  }, [canvasContext, layoutStructure, scrollToBottom]);
  const handleLayoutStructureChange = (0, react_exports.useCallback)(() => {
    const layout = getBlockItems(gridState);
    onLayoutStructureChange(layout);
  }, [onLayoutStructureChange, gridState]);
  const getSelectedItems = (0, react_exports.useMemo)(() => {
    let itemsCache;
    return () => {
      const filteredItems = filterSelectedItems(
        layoutStructure,
        selectedLayoutItems
      );
      if (!(0, lodash_exports.isEqual)(itemsCache, filteredItems)) {
        itemsCache = filteredItems;
      }
      return itemsCache;
    };
  }, [layoutStructure, selectedLayoutItems]);
  const handleItemSelected = (0, react_exports.useCallback)(
    (_e, selectedItems = []) => {
      const selectedItem = selectedItems.length > 0 ? [selectedItems[selectedItems.length - 1]] : [];
      onLayoutItemsSelect(selectedItem);
    },
    [onLayoutItemsSelect]
  );
  const mouseScale = enableGridLayoutCssScaling ? scale : 1;
  (0, react_exports.useEffect)(() => {
    let delayed;
    const layoutContainer = canvasContext.current;
    if (!isBlockItemMoving || !mousePosition || !layoutContainer) {
      return () => void 0;
    }
    const visibleCanvasHeight = layoutContainer.clientHeight / mouseScale;
    const hasScrolledToBottom = layoutContainer.scrollTop / mouseScale + visibleCanvasHeight >= maxHeight;
    const isInBottomScrollArea = layoutContainer.scrollTop / mouseScale + visibleCanvasHeight - mousePosition.y < MIN_HEIGHT_PX;
    const hasScrolledToTop = layoutContainer.scrollTop <= 0;
    const isInTopScrollArea = mousePosition.y - layoutContainer.scrollTop / mouseScale < MIN_HEIGHT_PX;
    if (!hasScrolledToBottom && isInBottomScrollArea) {
      const increment = Math.min(
        Math.abs(
          maxHeight - (layoutContainer.scrollTop / mouseScale + visibleCanvasHeight)
        ),
        10
      );
      delayed = setTimeout(() => {
        layoutContainer.scrollTop += increment;
        setMousePosition(
          (pos) => pos ? __spreadProps(__spreadValues({}, pos), {
            y: pos.y + increment / mouseScale
          }) : null
        );
      }, 32);
    } else if (!hasScrolledToTop && isInTopScrollArea) {
      const decrement = Math.min(layoutContainer.scrollTop, 10);
      delayed = setTimeout(() => {
        layoutContainer.scrollTop -= decrement;
        setMousePosition(
          (pos) => pos ? __spreadProps(__spreadValues({}, pos), {
            y: pos.y - decrement / mouseScale
          }) : null
        );
      }, 32);
    }
    return () => {
      clearTimeout(delayed);
    };
  }, [
    isBlockItemMoving,
    mousePosition,
    canvasContext,
    mouseScale,
    maxHeight
  ]);
  const resetEdgePreviewState = (0, react_exports.useCallback)(() => {
    if (edgesBeforeMove) {
      if (delayPreviewEdge.current) {
        clearTimeout(delayPreviewEdge.current);
      }
      setEdges(edgesBeforeMove);
      dispatch(resetLayoutAction(layoutStructure));
    }
  }, [edgesBeforeMove, layoutStructure]);
  const showEdgeDropPreview = (0, react_exports.useCallback)(
    ({ mousePos }) => {
      if (edgesBeforeMove === null) {
        setEdgesBeforeMove(edges);
      }
      const validEdges = edgesBeforeMove || edges;
      const edge = validEdges.find((edgeToFind) => {
        return isMouseOnEdge({
          edge: edgeToFind,
          mousePosition: mousePos,
          padding: panelPadding
        });
      });
      if (!edge) {
        if (hoveredEdge) {
          setHoveredEdge(null);
          setInvalidEdgeId(null);
          resetEdgePreviewState();
        }
        return;
      }
      if (!hoveredEdge || hoveredEdge.item !== edge.item) {
        let itemToMove = firstSelectedItemStructure;
        if (!initialItemToMove) {
          setInitialItemToMove(itemToMove);
        } else {
          itemToMove = initialItemToMove;
        }
        if (isDropOnOwnEdge({
          edge,
          itemToMove
        })) {
          return;
        }
        const updatedItems = previewDropOnEdge({
          edge,
          itemToMove,
          items: layoutStructure,
          canvasWidth
        });
        if (updatedItems) {
          setInvalidEdgeId(null);
          if (delayPreviewEdge.current) {
            clearTimeout(delayPreviewEdge.current);
          }
          setShowPreviewPlaceholder(false);
          delayPreviewEdge.current = setTimeout(() => {
            dispatch(resetLayoutAction(updatedItems));
            setEdges(
              computeEdges({
                layout: updatedItems,
                canvasWidth,
                canvasHeight: computeMaxHeight(updatedItems)
              })
            );
            setShowPreviewPlaceholder(true);
          }, EDGE_PREVIEW_DELAY_MS);
        } else {
          setShowPreviewPlaceholder(true);
          setInvalidEdgeId(edge.item);
          resetEdgePreviewState();
        }
      }
      setHoveredEdge(edge);
    },
    [
      firstSelectedItemStructure,
      edgesBeforeMove,
      edges,
      hoveredEdge,
      panelPadding,
      resetEdgePreviewState,
      initialItemToMove,
      layoutStructure,
      canvasWidth
    ]
  );
  const showVizDropPreview = (0, react_exports.useCallback)(
    ({ mousePos }) => {
      const block = findTopBlockItemByPosition(
        layoutStructure,
        mousePos,
        panelPadding
      );
      let itemToMove = firstSelectedItemStructure;
      if (!initialItemToMove) {
        setInitialItemToMove(itemToMove);
      } else {
        itemToMove = initialItemToMove;
      }
      if (block === void 0 || block.item === (itemToMove == null ? void 0 : itemToMove.item)) {
        if (delayPreviewViz.current) {
          clearTimeout(delayPreviewViz.current);
        }
        setHoveredQuadrant(null);
        setHoveredBlock(null);
        setIsInvalidVizDrop(false);
        if (hoveredBlock) {
          dispatch(resetLayoutAction(layoutStructure));
        }
        return;
      }
      const currentQuadrant = findQuadrant({
        item: applyVizPadding({ item: block, padding: panelPadding }),
        position: mousePos
      });
      if (!hoveredBlock && !hoveredQuadrant || hoveredQuadrant !== currentQuadrant || (hoveredBlock == null ? void 0 : hoveredBlock.item) !== block.item) {
        if (isInvalidAdjacentVizDrop({
          itemToMove,
          itemToDropOn: block,
          visualizations: layoutStructure,
          direction: currentQuadrant
        })) {
          return;
        }
        const updatedItems = previewDropOnViz({
          itemToDropOn: block,
          itemToMove,
          items: layoutStructure,
          direction: currentQuadrant
        });
        if (delayPreviewViz.current) {
          clearTimeout(delayPreviewViz.current);
        }
        if (updatedItems) {
          setIsInvalidVizDrop(false);
          setHoveredQuadrant(currentQuadrant);
          setHoveredBlock(block);
          setShowPreviewPlaceholder(false);
          delayPreviewViz.current = setTimeout(() => {
            dispatch(resetLayoutAction(updatedItems));
            setForceUpdate(
              (prevForceUpdate) => prevForceUpdate + 1
            );
            setShowPreviewPlaceholder(true);
          }, VIZ_PREVIEW_DELAY_MS);
        } else {
          setHoveredQuadrant(null);
          setHoveredBlock(null);
          setIsInvalidVizDrop(true);
          if (hoveredBlock) {
            dispatch(resetLayoutAction(layoutStructure));
          }
        }
      }
    },
    [
      firstSelectedItemStructure,
      panelPadding,
      hoveredBlock,
      hoveredQuadrant,
      initialItemToMove,
      layoutStructure
    ]
  );
  const renderItemDropTarget = memoize_one_default(
    ({
      position,
      direction
    }) => {
      return /* @__PURE__ */ react_default.createElement(ItemDropTarget, { position, direction });
    }
  );
  const handleRenderItemDropTarget = (0, react_exports.useCallback)(() => {
    let block = findTopBlockItemByPosition(
      layoutStructure,
      mousePosition,
      panelPadding
    );
    const selectedItems = getSelectedItems();
    if (block !== void 0 && selectedItems.length > 0 && block.item !== selectedItems[0].id && // disables drop targets when showing preview
    !hoveredQuadrant) {
      block = applyVizPadding({ item: block, padding: panelPadding });
      return renderItemDropTarget({
        position: block.position,
        direction: findQuadrant({
          item: block,
          position: mousePosition
        })
      });
    }
    return null;
  }, [
    layoutStructure,
    panelPadding,
    getSelectedItems,
    hoveredQuadrant,
    mousePosition,
    renderItemDropTarget
  ]);
  const handleShowPreviewPlaceholder = (0, react_exports.useCallback)(
    ({ mousePos }) => {
      const selectedItem = getSelectedItems()[0];
      if (!selectedItem) {
        setShowPreviewPlaceholder(false);
        return;
      }
      const hoveredViz = findTopBlockItemByPosition(
        layoutStructure,
        mousePos,
        panelPadding
      );
      if (hoveredViz && hoveredViz.item === selectedItem.id) {
        setShowPreviewPlaceholder(true);
        return;
      }
      if (invalidEdgeId !== null || isInvalidVizDrop) {
        setShowPreviewPlaceholder(true);
        return;
      }
      if (containsPreviewItem) {
        setShowPreviewPlaceholder(true);
        return;
      }
      setShowPreviewPlaceholder(false);
    },
    [
      containsPreviewItem,
      panelPadding,
      getSelectedItems,
      invalidEdgeId,
      isInvalidVizDrop,
      layoutStructure
    ]
  );
  const handleBlockItemMove = (0, react_exports.useCallback)(
    (_e, { currentPosition }) => {
      if (getSelectedItems().length > 0) {
        handleShowPreviewPlaceholder({ mousePos: currentPosition });
        setIsBlockItemMoving(true);
        setMousePosition(currentPosition);
        showEdgeDropPreview({ mousePos: currentPosition });
        showVizDropPreview({ mousePos: currentPosition });
      }
    },
    [
      getSelectedItems,
      showEdgeDropPreview,
      showVizDropPreview,
      handleShowPreviewPlaceholder
    ]
  );
  const handleVizDrop = (0, react_exports.useCallback)(() => {
    const validEdges = edgesBeforeMove || edges;
    const selectedItems = getSelectedItems();
    if (selectedItems.length === 0) {
      return;
    }
    const edge = validEdges.find((e) => {
      return isMouseOnEdge({
        edge: e,
        mousePosition,
        padding: panelPadding
      });
    });
    const block = findTopBlockItemByPosition(
      layoutStructure,
      mousePosition,
      panelPadding
    );
    const itemToMove = layoutStructure.find(
      ({ item }) => item === selectedItems[0].id
    );
    if (block === itemToMove) {
      sendTelemetry({
        source: "canvas",
        event: "viz_drop_on_self_successful"
      });
      return;
    }
    const itemsSelected = layoutStructure;
    let updatedItems = null;
    if (edge !== void 0) {
      updatedItems = updateDropOnEdge({
        edge,
        itemToMove,
        items: itemsSelected,
        canvasWidth
      });
      if (updatedItems === null) {
        sendTelemetry({
          source: "canvas",
          event: "viz_drop_on_edge_unsuccessful"
        });
        return;
      }
      updatedItems = updateItems({
        updatedVisualizations: updatedItems.updatedVisualizations,
        itemToMove: updatedItems.updatedItemToMove,
        items: itemsSelected,
        canvasWidth
      });
      sendTelemetry({
        source: "canvas",
        event: "viz_drop_on_edge_successful"
      });
    } else if (block !== void 0 && block.item !== selectedItems[0].id) {
      const direction = findQuadrant({
        item: applyVizPadding({ item: block, padding: panelPadding }),
        position: mousePosition
      });
      if (isInvalidAdjacentVizDrop({
        itemToMove,
        itemToDropOn: block,
        direction,
        visualizations: layoutStructure
      })) {
        return;
      }
      const updatedVisualizations = updateDropOnViz({
        itemToMove,
        itemToDropOn: block,
        direction
      });
      if (updatedVisualizations === null) {
        sendTelemetry({
          source: "canvas",
          event: "viz_drop_on_viz_unsuccessful"
        });
        return;
      }
      updatedItems = updateItems({
        updatedVisualizations,
        itemToMove,
        items: itemsSelected,
        canvasWidth
      });
      sendTelemetry({
        source: "canvas",
        event: "viz_drop_on_viz_successful"
      });
    }
    if (updatedItems != null) {
      onLayoutStructureChange(updatedItems);
    }
  }, [
    layoutStructure,
    edgesBeforeMove,
    edges,
    getSelectedItems,
    mousePosition,
    panelPadding,
    sendTelemetry,
    canvasWidth,
    onLayoutStructureChange
  ]);
  const cleanupAfterPreview = (0, react_exports.useCallback)(() => {
    if (delayPreviewViz.current) {
      clearTimeout(delayPreviewViz.current);
    }
    if (delayPreviewEdge.current) {
      clearTimeout(delayPreviewEdge.current);
    }
    setInvalidEdgeId(null);
    setIsInvalidVizDrop(false);
    setInitialItemToMove(null);
    setHoveredBlock(null);
    setHoveredQuadrant(null);
    if (edgesBeforeMove) {
      setEdges(edgesBeforeMove);
      setEdgesBeforeMove(null);
      dispatch(resetLayoutAction(layoutStructure));
    }
    if (hoveredBlock) {
      dispatch(resetLayoutAction(layoutStructure));
    }
  }, [edgesBeforeMove, hoveredBlock, layoutStructure]);
  const handleBlockItemMoved = (0, react_exports.useCallback)(() => {
    cleanupAfterPreview();
    if (mousePosition !== null) {
      handleVizDrop();
    }
    setIsBlockItemMoving(false);
    setMousePosition(null);
  }, [cleanupAfterPreview, mousePosition, handleVizDrop]);
  const handleBlockItemPositionUpdate = (0, react_exports.useCallback)(
    ({ item, offset }) => {
      const newItem = updateBlockItemPosition(item, offset);
      dispatch(updateItemAction(newItem));
    },
    []
  );
  const handleBlockItemResize = (0, react_exports.useCallback)(
    (_e, {
      itemId,
      offset,
      dir
    }) => {
      const item = getStructureItem(layoutStructure, itemId);
      const newItem = updateBlockItemSize({
        item,
        offset,
        dir,
        options: {
          minHeight: MIN_HEIGHT_PX,
          minWidth: MIN_WIDTH_PX
        }
      });
      dispatch(updateItemAction(newItem));
    },
    [layoutStructure]
  );
  const handleBlockItemResized = (0, react_exports.useCallback)(() => {
    handleLayoutStructureChange();
    sendTelemetry({
      source: "canvas",
      event: "edge_resize"
    });
  }, [handleLayoutStructureChange, sendTelemetry]);
  const handleMouseDownOnEdge = (0, react_exports.useCallback)(
    (e, { id }) => {
      isDraggingEdge.current = true;
      const edge = edges.find(({ item }) => item === id);
      if (!edge) {
        return;
      }
      const visualizationIds = edge.visualizations.map((viz) => viz.item);
      const newEdgeBoundaries = edge.orientation === "horizontal" ? getVerticalBoundaries({
        visualizations: edge.visualizations,
        y: edge.edgeStart.y,
        minHeight: MIN_HEIGHT_PX,
        isFullWidthEdge: edge.edgeEnd.x - edge.edgeStart.x === canvasWidth
      }) : getHorizontalBoundaries({
        visualizations: edge.visualizations,
        x: edge.edgeStart.x,
        minWidth: MIN_WIDTH_PX
      });
      edgeBoundaries.current = newEdgeBoundaries;
      edgesInBoundary.current = findEdgesInBoundary({
        edge,
        edges,
        edgeBoundaries: newEdgeBoundaries
      });
      edgeMouseDownPosition.current = getClientPosition(e, mouseScale);
      mouseDownEdge.current = __spreadValues({}, edge);
      setSelectedItemsForEdge(visualizationIds);
    },
    [edges, mouseScale, canvasWidth]
  );
  const handleEdgeMove = (0, dashboard_ui_exports.useEventCallback)((e) => {
    var _a2, _b, _c, _d;
    e.preventDefault();
    e.stopPropagation();
    if (!isDraggingEdge.current || !mouseDownEdge.current) {
      return;
    }
    const currentPosition = getClientPosition(e, mouseScale);
    const offset = getOffset(
      currentPosition,
      edgeMouseDownPosition.current
    );
    offset.offsetY += cumulativeScrollIncrease.current;
    let updatedEdge = ((_a2 = mouseDownEdge == null ? void 0 : mouseDownEdge.current) == null ? void 0 : _a2.orientation) === "vertical" ? moveVerticalEdge({
      edge: mouseDownEdge.current,
      offset: offset.offsetX,
      edgeBoundaries: edgeBoundaries.current
    }) : moveHorizontalEdge({
      edge: mouseDownEdge.current,
      offset: offset.offsetY,
      edgeBoundaries: edgeBoundaries.current
    });
    let updatedSnappableEdges = [];
    if (!e.shiftKey) {
      const { updatedEdge: ue, snappableEdges: se } = findSnappableEdges({
        edge: updatedEdge,
        edges: (_b = edgesInBoundary.current) != null ? _b : [],
        snapRange: SNAP_RANGE_PX
      });
      updatedEdge = ue;
      updatedSnappableEdges = se;
    }
    setEdges(
      (prevEdges) => prevEdges.map((anEdge) => {
        if (anEdge.item === updatedEdge.item) {
          return updatedEdge;
        }
        return anEdge;
      })
    );
    const vizOffset = {
      offsetX: updatedEdge.edgeStart.x - mouseDownEdge.current.edgeStart.x,
      offsetY: updatedEdge.edgeStart.y - mouseDownEdge.current.edgeStart.y
    };
    const isFullWidthEdge = mouseDownEdge.current.edgeEnd.x - mouseDownEdge.current.edgeStart.x === canvasWidth;
    if (isFullWidthEdge) {
      layoutStructure.forEach((item) => {
        if (mouseDownEdge.current && item.position.y >= mouseDownEdge.current.edgeStart.y) {
          handleBlockItemPositionUpdate({
            item,
            offset: vizOffset
          });
        }
      });
    }
    if (selectedItemsForEdge.length > 0) {
      selectedItemsForEdge.forEach((itemId) => {
        var _a3;
        const item = getStructureItem(layoutStructure, itemId);
        if (!mouseDownEdge.current || isFullWidthEdge && item.position.y >= mouseDownEdge.current.edgeStart.y) {
          return;
        }
        let resizeDir;
        if (((_a3 = mouseDownEdge.current) == null ? void 0 : _a3.orientation) === "horizontal") {
          resizeDir = item.position.y >= mouseDownEdge.current.edgeStart.y ? "n" : "s";
        } else {
          resizeDir = item.position.x >= mouseDownEdge.current.edgeStart.x ? "w" : "e";
        }
        handleBlockItemResize(e, {
          itemId,
          offset: vizOffset,
          dir: resizeDir
        });
      });
    }
    setSnappableEdges(updatedSnappableEdges);
    if (((_c = mouseDownEdge == null ? void 0 : mouseDownEdge.current) == null ? void 0 : _c.orientation) === "horizontal" && ((_d = mouseDownEdge == null ? void 0 : mouseDownEdge.current) == null ? void 0 : _d.isCanvasEdge)) {
      const scrollVelocity = e.clientY - window.innerHeight + CANVAS_BOTTOM_PADDING * 2;
      edgeVelocity.current = scrollVelocity;
      setMousePosition(getClientPosition(e, mouseScale));
    }
  });
  (0, dashboard_ui_exports.useMouseMoveHandler)({
    onMouseMove: handleEdgeMove,
    isEnabled: mode === "edit",
    throttledMs: 32
  });
  (0, react_exports.useEffect)(() => {
    if (mode !== "edit" || edgeVelocity.current <= 0 || !mouseDownEdge.current || !edgeMouseDownPosition.current || !mousePosition) {
      return void 0;
    }
    const currEdge = mouseDownEdge.current;
    const isFullWidthEdge = currEdge.edgeEnd.x - currEdge.edgeStart.x === canvasWidth;
    if (!isFullWidthEdge) {
      return void 0;
    }
    const offset = edgeVelocity.current > 8 ? BOTTOM_EDGE_MAX_SPEED : BOTTOM_EDGE_MIN_SPEED;
    const edgeMousePos = edgeMouseDownPosition.current;
    const interval = setInterval(() => {
      cumulativeScrollIncrease.current += offset;
      const mouseOffset = getOffset(mousePosition, edgeMousePos);
      const updatedEdge = moveHorizontalEdge({
        edge: currEdge,
        offset: mouseOffset.offsetY + cumulativeScrollIncrease.current,
        edgeBoundaries: edgeBoundaries.current
      });
      setEdges(
        (prevEdges) => prevEdges.map((anEdge) => {
          if (anEdge.item === currEdge.item) {
            return __spreadProps(__spreadValues({}, anEdge), {
              edgeStart: {
                x: anEdge.edgeStart.x,
                y: updatedEdge.edgeStart.y
              },
              edgeEnd: {
                x: anEdge.edgeEnd.x,
                y: updatedEdge.edgeStart.y
              }
            });
          }
          return anEdge;
        })
      );
      const vizOffset = {
        offsetX: 0,
        offsetY: updatedEdge.edgeStart.y - currEdge.edgeStart.y
      };
      if (selectedItemsForEdge.length > 0) {
        selectedItemsForEdge.forEach((itemId) => {
          const resizeDir = "s";
          handleBlockItemResize(mousePosition, {
            itemId,
            offset: vizOffset,
            dir: resizeDir
          });
        });
      }
      if (canvasContext.current) {
        canvasContext.current.scrollTop += BOTTOM_RESIZING_SCROLL_SPEED;
      }
    }, 32);
    return () => {
      clearInterval(interval);
    };
  }, [
    mode,
    edgeVelocity,
    mousePosition,
    cumulativeScrollIncrease,
    canvasContext,
    canvasWidth,
    handleBlockItemResize,
    layoutStructure,
    selectedItemsForEdge
  ]);
  const handleEdgeMoved = (0, dashboard_ui_exports.useEventCallback)((e) => {
    e.preventDefault();
    mouseDownEdge.current = null;
    edgeMouseDownPosition.current = null;
    edgeBoundaries.current = null;
    edgesInBoundary.current = null;
    cumulativeScrollIncrease.current = 0;
    edgeVelocity.current = 0;
    if (isDraggingEdge.current) {
      isDraggingEdge.current = false;
      setSelectedItemsForEdge([]);
      handleBlockItemResized();
    }
    if (snappableEdges.length > 0) {
      setSnappableEdges([]);
    }
  });
  (0, react_exports.useEffect)(() => {
    if (mode !== "edit") {
      return void 0;
    }
    document.addEventListener("mouseup", handleEdgeMoved);
    return () => {
      document.removeEventListener("mouseup", handleEdgeMoved);
    };
  }, [handleEdgeMoved, mode]);
  (0, react_exports.useEffect)(() => {
    if (!keyboardListener) {
      return () => void 0;
    }
    const unsubscribeKeyboardListener = keyboardListener.subscribe(
      "cancel",
      () => {
        if (isBlockItemMoving) {
          cleanupAfterPreview();
          setIsBlockItemMoving(false);
          setMousePosition(null);
        } else {
          onLayoutItemsSelect([]);
        }
      }
    );
    return () => {
      unsubscribeKeyboardListener();
    };
  }, [
    cleanupAfterPreview,
    isBlockItemMoving,
    onLayoutItemsSelect,
    keyboardListener
  ]);
  const renderOutline = () => {
    const selectedItems = selectedItemsForEdge.length > 0 ? selectedItemsForEdge : getSelectedItems().map(({ id }) => id);
    const blockItems = getBlockItems(gridState);
    return blockItems.map((item) => {
      const displayOutline = selectedItems.includes(item.item) && (!isBlockItemMoving || showPreviewPlaceholder);
      if (displayOutline) {
        const paddedItem = applyVizPadding({
          item,
          padding: panelPadding
        });
        return /* @__PURE__ */ react_default.createElement(
          MemoizedBlockOutline,
          {
            handleDirections: HANDLE_DIRECTIONS,
            key: item.item,
            itemId: item.item,
            scale: mouseScale,
            x: paddedItem.position.x,
            y: paddedItem.position.y,
            w: paddedItem.position.w,
            h: paddedItem.position.h
          }
        );
      }
      return null;
    });
  };
  const getEdgeAppearance = (0, react_exports.useCallback)(
    ({ edge }) => {
      var _a2;
      const renderAllEdges = !mouseDownEdge.current;
      let appearance = renderAllEdges || edge.item === ((_a2 = mouseDownEdge.current) == null ? void 0 : _a2.item) || snappableEdges.find((e) => e.item === edge.item) ? "normal" : "hidden";
      if (isBlockItemMoving) {
        const itemToMove = layoutStructure.find(
          ({ item }) => {
            var _a3;
            return item === ((_a3 = getSelectedItems()[0]) == null ? void 0 : _a3.id);
          }
        );
        if (itemToMove && isDropOnOwnEdge({ edge, itemToMove })) {
          appearance = "hidden";
        } else {
          appearance = "dropTarget";
        }
      }
      if (invalidEdgeId === edge.item) {
        appearance = "invalid";
      }
      return appearance;
    },
    [
      invalidEdgeId,
      isBlockItemMoving,
      mouseDownEdge,
      snappableEdges,
      getSelectedItems,
      layoutStructure
    ]
  );
  const edgeItems = (0, react_exports.useMemo)(() => {
    if (mode !== "edit") {
      return null;
    }
    return edges.map((edge) => {
      const formattedEdge = formatEdgeWrapper({
        edge,
        padding: panelPadding
      });
      const { width, height } = getDimensions({
        edge: formattedEdge,
        thickness: 2 * panelPadding
      });
      const { x, y } = formattedEdge.edgeStart;
      return /* @__PURE__ */ react_default.createElement(
        MemoizedEdge,
        {
          key: edge.item,
          itemId: edge.item,
          x,
          y,
          w: width,
          h: height,
          padding: panelPadding,
          orientation: edge.orientation,
          isCanvasEdge: edge.isCanvasEdge,
          appearance: getEdgeAppearance({ edge }),
          onMouseDown: handleMouseDownOnEdge,
          "data-test-edge-position": `${edge.edgeStart.x},${edge.edgeStart.y}-${edge.edgeEnd.x},${edge.edgeEnd.y}`
        }
      );
    });
  }, [mode, edges, getEdgeAppearance, panelPadding, handleMouseDownOnEdge]);
  const handleRenderPreviewPlaceholderItem = (0, react_exports.useCallback)(() => {
    if (!firstSelectedItemStructure || !showPreviewPlaceholder) {
      return null;
    }
    const item = applyVizPadding({
      item: firstSelectedItemStructure,
      padding: panelPadding
    });
    return /* @__PURE__ */ react_default.createElement(
      PreviewPlaceholderItem,
      {
        key: `${firstSelectedItemStructure.item}-preview`,
        position: item.position
      }
    );
  }, [firstSelectedItemStructure, panelPadding, showPreviewPlaceholder]);
  const filteredState = (0, react_exports.useMemo)(
    () => getFilteredStructure(gridState, layoutStructure),
    [gridState, layoutStructure]
  );
  const items = getSelectedItems();
  const commonProps = {
    width: canvasWidth,
    height: maxHeight,
    scale,
    selectable: true,
    selectedLayoutItems: items,
    onItemSelected: handleItemSelected,
    panelPadding,
    ref: canvasRef
  };
  const modeSpecificProps = mode === "edit" ? {
    movable: true,
    showOverflowContent: false,
    showGrid,
    showBorder: true,
    gridWidth: GRID_SIZE_PX - 1,
    gridHeight: GRID_SIZE_PX - 1,
    gridLineWidth: 1,
    gridPadding: GRID_PADDING_PX,
    onItemMove: handleBlockItemMove,
    onItemMoved: handleBlockItemMoved,
    userSelect: false
  } : {
    movable: false,
    showOverflowContent: false,
    showGrid: false,
    showBorder: false,
    userSelect: true
  };
  return /* @__PURE__ */ react_default.createElement(
    GridCanvas_default,
    __spreadValues(__spreadValues({
      "data-test": "grid-layout",
      blockItems: filteredState,
      "data-test-scale": scale,
      backgroundColor
    }, commonProps), modeSpecificProps),
    /* @__PURE__ */ react_default.createElement(Layer_default, { key: "block-item-layer", "data-test": "block-item-layer" }, renderBlockItems({
      layoutStructure: filteredState,
      renderLayoutItem,
      handleItemSelected,
      isBlockItemMoving,
      selectedItem: getSelectedItems()[0],
      mode,
      errors: layoutErrors,
      padding: panelPadding,
      canvasHeight: maxHeight
    })),
    edgeItems,
    /* @__PURE__ */ react_default.createElement(Layer_default, { key: "outline-layer", "data-test": "outline-layer" }, renderOutline()),
    mousePosition && isBlockItemMoving && handleRenderItemDropTarget(),
    mousePosition && isBlockItemMoving && handleRenderPreviewPlaceholderItem(),
    mousePosition && isBlockItemMoving && /* @__PURE__ */ react_default.createElement(
      ItemDragPlaceholder,
      {
        position: mousePosition,
        size: PLACEHOLDER_SIZE_PX
      }
    )
  );
};
var GridLayout = (props) => {
  const {
    containerWidth,
    containerHeight,
    options: { width: canvasWidth = gridLayoutOptions.width } = {},
    onLayoutStructureChange,
    layoutStructure
  } = props;
  const { enableGridLayoutCssScaling } = (0, dashboard_context_exports.useFeatureFlags)();
  const scrollbarWidth = (0, dashboard_utils_exports.getScrollbarWidth)();
  const [canvasHeight, setCanvasHeight] = (0, react_exports.useState)(
    () => computeMaxHeight(layoutStructure)
  );
  const scale = (0, react_exports.useMemo)(
    () => computeScaleToFit({
      canvasWidth,
      canvasHeight,
      containerWidth,
      containerHeight,
      scrollbarWidth,
      enableGridLayoutCssScaling
    }),
    [
      enableGridLayoutCssScaling,
      canvasWidth,
      canvasHeight,
      containerWidth,
      containerHeight,
      scrollbarWidth
    ]
  );
  const handleLayoutStructureChange = (0, react_exports.useCallback)(
    (layout) => {
      onLayoutStructureChange(
        scaleGridLayoutStructureByWidth({
          layout,
          scale: 1 / scale
        })
      );
    },
    [scale, onLayoutStructureChange]
  );
  const scaledLayoutStructure = (0, react_exports.useMemo)(
    () => !enableGridLayoutCssScaling ? scaleGridLayoutStructureByWidth({
      layout: layoutStructure,
      scale
    }) : layoutStructure,
    [enableGridLayoutCssScaling, layoutStructure, scale]
  );
  const sortedLayoutStructure = (0, react_exports.useMemo)(
    () => getGridLayoutOrder({
      layout: scaledLayoutStructure,
      canvasWidth,
      canvasHeight: computeMaxHeight(scaledLayoutStructure)
    }),
    [canvasWidth, scaledLayoutStructure]
  );
  return /* @__PURE__ */ react_default.createElement(
    BaseGridLayout,
    __spreadValues(__spreadProps(__spreadValues({}, props), {
      scale,
      canvasWidth,
      layoutStructure: sortedLayoutStructure,
      setCanvasHeight
    }), !enableGridLayoutCssScaling ? {
      canvasWidth: containerWidth - scrollbarWidth,
      onLayoutStructureChange: handleLayoutStructureChange
    } : null)
  );
};
var GridLayout_default = withLayoutShowHide(GridLayout, {
  schema: gridLayoutOptionsSchema_default,
  reflowFn: gridLayoutShowHideReflow
});

export {
  GridLayout_default
};
//# sourceMappingURL=chunk-KPUER3CF.js.map
