/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  styled_components_default,
  styled_components_exports
} from "./chunk-AZNCU5D5.js";
import {
  dashboard_context_exports,
  themes_exports
} from "./chunk-2OPMAP46.js";
import {
  dashboard_ui_exports
} from "./chunk-V6LCPHRJ.js";
import {
  GRID_SIZE,
  ZOOM_STEP_SIZE
} from "./chunk-WSRZLQBK.js";
import {
  prop_types_default,
  prop_types_exports
} from "./chunk-RVNUZD6H.js";
import {
  react_default,
  react_exports
} from "./chunk-D2UIVRV7.js";
import {
  dashboard_utils_exports
} from "./chunk-QCW3O7QD.js";
import {
  ITEM_DROP_TARGET_PX
} from "./chunk-DLFGXT3M.js";
import {
  i18n_exports
} from "./chunk-PLOB2GBC.js";
import {
  __async,
  __export,
  __objRest,
  __reExport,
  __spreadProps,
  __spreadValues
} from "./chunk-W3Q7A2F6.js";

// esm-externals:lodash
var lodash_exports = {};
__export(lodash_exports, {
  default: () => lodash_default
});
__reExport(lodash_exports, lodash_star);
import * as defaultImport from "lodash";
import * as lodash_star from "lodash";
var lodash_default = "default" in defaultImport ? defaultImport.default : defaultImport;

// esm-externals:@splunk/dashboard-telemetry
var dashboard_telemetry_exports = {};
__export(dashboard_telemetry_exports, {
  default: () => dashboard_telemetry_default
});
__reExport(dashboard_telemetry_exports, dashboard_telemetry_star);
import * as defaultImport2 from "@splunk/dashboard-telemetry";
import * as dashboard_telemetry_star from "@splunk/dashboard-telemetry";
var dashboard_telemetry_default = "default" in defaultImport2 ? defaultImport2.default : defaultImport2;

// src/utils/edgeUtils.ts
var nextEdgeId = () => `edge_${(0, dashboard_utils_exports.uniqueId)()}`;
var getNextEdgeId = () => {
  return nextEdgeId();
};
var getNodes = (layoutStructure) => {
  if (layoutStructure.length === 0) {
    return null;
  }
  const nodes = {};
  layoutStructure.forEach((viz) => {
    const { x, y, w, h } = viz.position;
    const corners = [
      { x, y },
      { x: x + w, y },
      { x, y: y + h },
      { x: x + w, y: y + h }
    ];
    corners.forEach((corner) => {
      if (nodes[corner.x] === void 0) {
        nodes[corner.x] = {};
      }
      if (nodes[corner.x][corner.y] === void 0) {
        nodes[corner.x][corner.y] = [];
      }
      nodes[corner.x][corner.y].push(viz);
    });
  });
  return nodes;
};
var isTopEdge = (edge) => edge.orientation === "horizontal" && edge.edgeStart.y === 0;
var isBottomEdge = (edge, canvasHeight) => edge.orientation === "horizontal" && edge.edgeStart.y === canvasHeight;
var isLeftEdge = (edge) => edge.orientation === "vertical" && edge.edgeStart.x === 0;
var isRightEdge = (edge, canvasWidth) => edge.orientation === "vertical" && edge.edgeStart.x === canvasWidth;
var findVizAlongVerticalEdge = ({
  x,
  yStart,
  yEnd,
  yCurrent = yStart,
  nodes,
  visualizations = /* @__PURE__ */ new Set()
}) => {
  nodes[x][yCurrent].forEach((viz) => {
    if (viz.position.y >= yStart && viz.position.y < yEnd) {
      visualizations.add(viz);
    }
    if (viz.position.y >= yCurrent && yCurrent + viz.position.h <= yEnd) {
      findVizAlongVerticalEdge({
        x,
        yStart,
        yEnd,
        yCurrent: yCurrent + viz.position.h,
        nodes,
        visualizations
      });
    }
  });
  return visualizations;
};
var findVizAlongHorizontalEdge = ({
  y,
  xStart,
  xEnd,
  xCurrent = xStart,
  nodes,
  visualizations = /* @__PURE__ */ new Set()
}) => {
  var _a, _b;
  (_b = (_a = nodes[xCurrent]) == null ? void 0 : _a[y]) == null ? void 0 : _b.forEach((viz) => {
    if (viz.position.x >= xStart && viz.position.x < xEnd) {
      visualizations.add(viz);
    }
    if (viz.position.x >= xCurrent && xCurrent + viz.position.w <= xEnd) {
      findVizAlongHorizontalEdge({
        y,
        xStart,
        xEnd,
        xCurrent: xCurrent + viz.position.w,
        nodes,
        visualizations
      });
    }
  });
  return visualizations;
};
var addEdge = ({
  edges,
  edgeStart,
  edgeEnd,
  nodes
}) => {
  const orientation = edgeStart.y === edgeEnd.y ? "horizontal" : "vertical";
  const visualizations = orientation === "vertical" ? findVizAlongVerticalEdge({
    x: edgeStart.x,
    yStart: edgeStart.y,
    yEnd: edgeEnd.y,
    nodes
  }) : findVizAlongHorizontalEdge({
    y: edgeStart.y,
    xStart: edgeStart.x,
    xEnd: edgeEnd.x,
    nodes
  });
  edges.push({
    item: getNextEdgeId(),
    edgeStart,
    edgeEnd,
    visualizations: Array.from(visualizations),
    orientation
  });
};
var shouldTraverseDown = (y, visualizations) => {
  return visualizations.every((viz) => viz.position.y >= y);
};
var shouldTraverseRight = (x, visualizations) => {
  return visualizations.every((viz) => viz.position.x >= x);
};
var findOffset = ({
  x,
  y,
  visualizations,
  type
}) => {
  const vizXY = visualizations.find(
    ({ position }) => position.y === y && position.x === x
  );
  if (vizXY) {
    return vizXY.position[type];
  }
  const vizWH = visualizations.find(
    ({ position }) => (
      // horizontal canvas edge
      type === "w" && position.x === x || // vertical canvas edge
      type === "h" && position.y === y
    )
  );
  if (vizWH) {
    return vizWH.position[type];
  }
  return 0;
};
function traverseRight({
  x,
  y,
  nodes,
  edges,
  edgeStart,
  canTraverseDown = true
}) {
  if (canTraverseDown && shouldTraverseDown(y, nodes[x][y])) {
    traverseDown({
      x,
      y,
      nodes,
      edges,
      edgeStart: { x, y },
      // Start new edge when changing directions
      canTraverseRight: false
    });
  }
  const offset = findOffset({ x, y, visualizations: nodes[x][y], type: "w" });
  if (!offset) {
    addEdge({ edges, edgeStart, edgeEnd: { x, y }, nodes });
    return;
  }
  const isWindow = nodes[x][y].length === 4;
  if (isWindow) {
    addEdge({ edges, edgeStart, edgeEnd: { x, y }, nodes });
    traverseRight({
      x: x + offset,
      y,
      nodes,
      edges,
      edgeStart: { x, y }
    });
    return;
  }
  traverseRight({
    x: x + offset,
    y,
    nodes,
    edges,
    edgeStart
  });
}
function traverseDown({
  x,
  y,
  nodes,
  edges,
  edgeStart,
  canTraverseRight = true
}) {
  if (canTraverseRight && shouldTraverseRight(x, nodes[x][y])) {
    traverseRight({
      x,
      y,
      nodes,
      edges,
      edgeStart: { x, y },
      // Start new edge when changing directions
      canTraverseDown: false
    });
  }
  const offset = findOffset({ x, y, visualizations: nodes[x][y], type: "h" });
  if (!offset) {
    addEdge({ edges, edgeStart, edgeEnd: { x, y }, nodes });
    return;
  }
  const isWindow = nodes[x][y].length === 4;
  if (isWindow) {
    addEdge({ edges, edgeStart, edgeEnd: { x, y }, nodes });
    traverseDown({
      x,
      y: y + offset,
      nodes,
      edges,
      edgeStart: { x, y }
    });
    return;
  }
  traverseDown({
    x,
    y: y + offset,
    nodes,
    edges,
    edgeStart
  });
}
var computeVerticalCanvasEdges = ({
  edges,
  canvasWidth,
  layout
}) => {
  const nodes = getNodes(layout);
  if (nodes == null) {
    return;
  }
  const sortedRowEdges = edges.filter(
    (edge) => edge.orientation === "horizontal" && edge.edgeStart.x === 0 && edge.edgeEnd.x === canvasWidth
  ).sort((a, b) => a.edgeStart.y - b.edgeStart.y);
  for (let i = 0; i < sortedRowEdges.length - 1; i += 1) {
    const { edgeStart: edgeStartA, edgeEnd: edgeEndA } = sortedRowEdges[i];
    const { edgeStart: edgeStartB, edgeEnd: edgeEndB } = sortedRowEdges[i + 1];
    addEdge({
      edges,
      edgeStart: edgeStartA,
      edgeEnd: edgeStartB,
      nodes
    });
    addEdge({
      edges,
      edgeStart: edgeEndA,
      edgeEnd: edgeEndB,
      nodes
    });
  }
};
var getAllEdges = (layout) => {
  const edges = [];
  const nodes = getNodes(layout);
  if (nodes === null || nodes[0] === void 0 || nodes[0][0] === void 0) {
    return [];
  }
  traverseRight({
    x: 0,
    y: 0,
    nodes,
    edges,
    edgeStart: { x: 0, y: 0 }
  });
  return edges;
};
var computeEdges = ({
  layout,
  canvasHeight,
  canvasWidth
}) => {
  if (layout === void 0) {
    return [];
  }
  let edges = getAllEdges(layout);
  edges = edges.filter((edge) => {
    return !(isLeftEdge(edge) || isRightEdge(edge, canvasWidth));
  });
  computeVerticalCanvasEdges({ edges, layout, canvasWidth });
  edges = edges.map((edge) => {
    let isCanvasEdge = false;
    if (isTopEdge(edge) || isRightEdge(edge, canvasWidth) || isBottomEdge(edge, canvasHeight) || isLeftEdge(edge)) {
      isCanvasEdge = true;
    }
    return __spreadProps(__spreadValues({}, edge), { isCanvasEdge });
  });
  return edges;
};
var applyVizPadding = ({
  item,
  padding = 0
}) => {
  const { x, y, w, h } = item.position;
  return __spreadProps(__spreadValues({}, item), {
    position: {
      x: x + padding,
      y: y + padding,
      w: w - 2 * padding,
      h: h - 2 * padding
    }
  });
};
var getVerticalBoundaries = ({
  visualizations,
  y,
  minHeight,
  isFullWidthEdge = false
}) => {
  const boundaries = {
    upperBoundary: Number.NEGATIVE_INFINITY,
    lowerBoundary: Number.POSITIVE_INFINITY
  };
  const comparePosition = (position) => {
    if (position.y >= y && !isFullWidthEdge) {
      boundaries.lowerBoundary = Math.min(
        boundaries.lowerBoundary,
        position.y + position.h - minHeight
      );
    }
    if (position.y < y) {
      boundaries.upperBoundary = Math.max(
        boundaries.upperBoundary,
        position.y + minHeight
      );
    }
  };
  visualizations.forEach((viz) => comparePosition(viz.position));
  return boundaries;
};
var getHorizontalBoundaries = ({
  visualizations,
  x,
  minWidth
}) => {
  const boundaries = {
    rightBoundary: Number.POSITIVE_INFINITY,
    leftBoundary: Number.NEGATIVE_INFINITY
  };
  const comparePosition = (position) => {
    if (position.x >= x) {
      boundaries.rightBoundary = Math.min(
        boundaries.rightBoundary,
        position.x + position.w - minWidth
      );
    }
    if (position.x < x) {
      boundaries.leftBoundary = Math.max(
        boundaries.leftBoundary,
        position.x + minWidth
      );
    }
  };
  visualizations.forEach((viz) => comparePosition(viz.position));
  return boundaries;
};
var moveHorizontalEdge = ({
  edge,
  offset,
  edgeBoundaries: { upperBoundary, lowerBoundary }
}) => {
  const updatedY = offset < 0 ? Math.max(upperBoundary, edge.edgeStart.y + offset) : Math.min(lowerBoundary, edge.edgeStart.y + offset);
  return __spreadProps(__spreadValues({}, edge), {
    edgeStart: {
      x: edge.edgeStart.x,
      y: updatedY
    },
    edgeEnd: {
      x: edge.edgeEnd.x,
      y: updatedY
    }
  });
};
var moveVerticalEdge = ({
  edge,
  offset,
  edgeBoundaries: { leftBoundary, rightBoundary }
}) => {
  const updatedX = offset < 0 ? Math.max(leftBoundary, edge.edgeStart.x + offset) : Math.min(rightBoundary, edge.edgeStart.x + offset);
  return __spreadProps(__spreadValues({}, edge), {
    edgeStart: {
      x: updatedX,
      y: edge.edgeStart.y
    },
    edgeEnd: {
      x: updatedX,
      y: edge.edgeEnd.y
    }
  });
};
var findSnappableEdges = ({
  edge,
  edges,
  snapRange
}) => {
  const coordinate = edge.orientation === "horizontal" ? "y" : "x";
  const snappableEdges = edges.reduce(
    (currentSnappableEdges, nextEdge) => {
      const nextEdgeDistance = Math.abs(
        nextEdge.edgeStart[coordinate] - edge.edgeStart[coordinate]
      );
      if (nextEdgeDistance <= snapRange) {
        if (currentSnappableEdges.length === 0) {
          return [nextEdge];
        }
        const lastEdge = currentSnappableEdges[currentSnappableEdges.length - 1];
        const lastEdgeDistance = Math.abs(
          lastEdge.edgeStart[coordinate] - edge.edgeStart[coordinate]
        );
        if (nextEdgeDistance < lastEdgeDistance) {
          return [nextEdge];
        }
        if (nextEdgeDistance === lastEdgeDistance) {
          return [...currentSnappableEdges, nextEdge];
        }
      }
      return [...currentSnappableEdges];
    },
    []
  );
  const firstSnappableEdge = snappableEdges.length > 0 ? snappableEdges[0] : edge;
  return {
    updatedEdge: __spreadProps(__spreadValues({}, edge), {
      edgeStart: __spreadProps(__spreadValues({}, edge.edgeStart), {
        [coordinate]: firstSnappableEdge.edgeStart[coordinate]
      }),
      edgeEnd: __spreadProps(__spreadValues({}, edge.edgeEnd), {
        [coordinate]: firstSnappableEdge.edgeEnd[coordinate]
      })
    }),
    snappableEdges
  };
};
var findEdgesInBoundary = ({
  edge,
  edges,
  edgeBoundaries
}) => {
  const { orientation } = edge;
  if (orientation === "horizontal") {
    return edges.filter(
      (e) => e.orientation === "horizontal" && e.edgeStart.y >= edgeBoundaries.upperBoundary && e.edgeStart.y <= edgeBoundaries.lowerBoundary && e !== edge
    );
  }
  if (orientation === "vertical") {
    return edges.filter(
      (e) => e.orientation === "vertical" && e.edgeStart.x >= edgeBoundaries.leftBoundary && e.edgeStart.x <= edgeBoundaries.rightBoundary && e !== edge
    );
  }
  return [];
};

// src/utils/layoutUtils.ts
var computeMaxHeight = dashboard_utils_exports.computeMaxHeight;
var isBlockPositionValid = (position) => !!(position && Number.isInteger(position.x) && Number.isInteger(position.y) && Number.isInteger(position.w) && Number.isInteger(position.h));
var isValidConnection = (connection) => {
  if ("x" in connection && "y" in connection && Number.isInteger(connection.x) && Number.isInteger(connection.y)) {
    return true;
  }
  if ("item" in connection && "port" in connection && connection.item != null && connection.port != null) {
    return true;
  }
  return false;
};
var isLinePositionValid = (position) => {
  if (position.from != null && position.to != null) {
    return isValidConnection(position.from) && isValidConnection(position.to);
  }
  return false;
};
var removeInvalidItems = (layoutStructure = []) => layoutStructure.filter((structure) => {
  if (structure.type === "line") {
    return isLinePositionValid(structure.position);
  }
  return isBlockPositionValid(structure.position);
});
var positionToStyle = (pos) => ({
  width: `${pos.w}px`,
  height: `${pos.h}px`,
  transform: `translate(${pos.x}px, ${pos.y}px)`
});
var getClientPosition = (e, scaleFactor = 1) => ({
  x: Math.round(e.clientX / scaleFactor),
  y: Math.round(e.clientY / scaleFactor)
});
var getOffset = (currentPosition, startPosition) => ({
  offsetX: currentPosition.x - startPosition.x,
  offsetY: currentPosition.y - startPosition.y
});
var considerMoved = ({
  offsetX,
  offsetY
}) => offsetX !== 0 || offsetY !== 0;
var positionsToBoundary = (startPos, endPos) => ({
  x: Math.min(startPos.x, endPos.x),
  y: Math.min(startPos.y, endPos.y),
  w: Math.abs(startPos.x - endPos.x),
  h: Math.abs(startPos.y - endPos.y)
});
var filterBlockItemsByBoundary = (items, boundary) => items.filter((item) => {
  const itemPosition = item.position;
  return !(boundary.x > itemPosition.x + itemPosition.w || boundary.x + boundary.w < itemPosition.x || boundary.y > itemPosition.y + itemPosition.h || boundary.y + boundary.h < itemPosition.y);
});
var findTopBlockItemByPosition = (items, pos, padding = 0) => (0, lodash_exports.findLast)(items, (item) => {
  const itemPosition = padding ? applyVizPadding({ item, padding }).position : item.position;
  return itemPosition.x <= pos.x && itemPosition.x + itemPosition.w >= pos.x && itemPosition.y <= pos.y && itemPosition.y + itemPosition.h >= pos.y;
});
var snapOffset = (current, offset, snapTo) => {
  const remainder = (current + offset) % snapTo;
  const newOffset = remainder > snapTo / 2 ? offset + (snapTo - remainder) : offset - remainder;
  return newOffset;
};
var snapOffsetToXY = ({
  position,
  offset,
  gridWidth,
  gridHeight,
  spacing = 0,
  padding = 0
}) => {
  let { offsetX, offsetY } = offset;
  offsetX = snapOffset(position.x, offsetX, gridWidth + spacing) + padding;
  offsetY = snapOffset(position.y, offsetY, gridHeight + spacing) + padding;
  return {
    offsetX,
    offsetY
  };
};
var snapOffsetToWH = ({
  position,
  offset,
  gridWidth,
  gridHeight,
  spacing = 0,
  padding = 0
}) => {
  let { offsetX, offsetY } = offset;
  offsetX = snapOffset(position.x + position.w, offsetX, gridWidth + spacing) + (padding - spacing);
  offsetY = snapOffset(position.y + position.h, offsetY, gridHeight + spacing) + (padding - spacing);
  return {
    offsetX,
    offsetY
  };
};
var updateBlockItemSize = ({
  item,
  offset,
  dir,
  options: { minWidth = 0, minHeight = 0 } = {}
}) => {
  const { w, h } = item.position;
  const { offsetX, offsetY } = offset;
  const updatedPosition = __spreadValues({}, item.position);
  if (["n", "ne", "nw"].includes(dir)) {
    updatedPosition.y += Math.min(h - minHeight, offsetY);
    updatedPosition.h -= offsetY;
  }
  if (["s", "se", "sw"].includes(dir)) {
    updatedPosition.h += offsetY;
  }
  if (["e", "ne", "se"].includes(dir)) {
    updatedPosition.w += offsetX;
  }
  if (["w", "nw", "sw"].includes(dir)) {
    updatedPosition.x += Math.min(w - minWidth, offsetX);
    updatedPosition.w -= offsetX;
  }
  updatedPosition.w = Math.max(minWidth, updatedPosition.w);
  updatedPosition.h = Math.max(minHeight, updatedPosition.h);
  return __spreadProps(__spreadValues({}, item), {
    position: updatedPosition
  });
};
var updateBlockItemPosition = (item, offset) => {
  const { offsetX, offsetY } = offset;
  return __spreadProps(__spreadValues({}, item), {
    position: __spreadProps(__spreadValues({}, item.position), {
      x: item.position.x + offsetX,
      y: item.position.y + offsetY
    })
  });
};
var createOffset = (dir, x, y) => {
  switch (dir) {
    case "n":
      return {
        offsetX: 0,
        offsetY: -y
      };
    case "s":
      return {
        offsetX: 0,
        offsetY: y
      };
    case "w":
      return {
        offsetX: -x,
        offsetY: 0
      };
    case "e":
      return {
        offsetX: x,
        offsetY: 0
      };
    default:
      return {
        offsetX: 0,
        offsetY: 0
      };
  }
};
var computeScaleToFit = ({
  canvasWidth,
  canvasHeight,
  containerWidth,
  containerHeight,
  scrollbarWidth,
  max = Infinity,
  enableGridLayoutCssScaling = true
}) => {
  if (!((0, lodash_exports.isNumber)(containerWidth) && containerWidth > 0 && (0, lodash_exports.isNumber)(canvasWidth) && canvasWidth > 0)) {
    dashboard_utils_exports.console.warn(
      `Failed to calculate layout scale: canvasWidth=${canvasWidth}, containerWidth=${containerWidth}; falling back to scale=1`
    );
    return 1;
  }
  const scale = Math.min(containerWidth / canvasWidth, max);
  const scrollbarScale = Math.min(
    (containerWidth - scrollbarWidth) / canvasWidth,
    max
  );
  if (!enableGridLayoutCssScaling) {
    return scrollbarScale;
  }
  if (canvasHeight * scale > containerHeight) {
    return scrollbarScale;
  }
  return scale;
};
var moveLayoutItem = (items, from, to) => {
  const structure = [...items];
  const removed = (0, lodash_exports.pullAt)(structure, [from]);
  structure.splice(to, 0, removed[0]);
  return structure;
};
var isLineConnected = ({ line, dir }) => "item" in line.position[dir];
var disconnectLine = ({
  line,
  dir,
  absPos
}) => {
  const updatePosition = __spreadProps(__spreadValues({}, line.position), {
    [dir]: __spreadValues({}, absPos)
  });
  return __spreadProps(__spreadValues({}, line), {
    position: updatePosition
  });
};
var connectLine = ({
  line,
  dir,
  itemId,
  port
}) => {
  const { position } = line;
  const updatedPosition = __spreadProps(__spreadValues({}, position), {
    [dir]: {
      item: itemId,
      port
    }
  });
  return __spreadProps(__spreadValues({}, line), {
    position: updatedPosition
  });
};
var updateLineAbsPosition = ({
  line,
  dir,
  offset
}) => {
  const { offsetX, offsetY } = offset;
  const { position } = line;
  if (!("x" in position[dir])) {
    throw Error(
      `line item ${line.item} does not have x value for its direction ${dir}`
    );
  }
  if (!("y" in position[dir])) {
    throw Error(
      `line item ${line.item} does not have y value for its direction ${dir}`
    );
  }
  const { x, y } = position[dir];
  return __spreadProps(__spreadValues({}, line), {
    position: __spreadProps(__spreadValues({}, position), {
      [dir]: {
        x: x + offsetX,
        y: y + offsetY
      }
    })
  });
};
var computeLineBoxPosition = (from, to) => ({
  x: Math.min(from.x, to.x),
  y: Math.min(from.y, to.y)
});
var computeLineRelativePosition = (from, to, box) => ({
  from: {
    x: from.x - box.x,
    y: from.y - box.y
  },
  to: {
    x: to.x - box.x,
    y: to.y - box.y
  }
});
var cloneBlockItem = ({
  id,
  item,
  offsetMultiplier
}) => {
  const copiedPosition = item.position;
  return __spreadProps(__spreadValues({}, item), {
    item: id,
    position: {
      x: copiedPosition.x + 20 * offsetMultiplier,
      y: copiedPosition.y + 20 * offsetMultiplier,
      w: copiedPosition.w,
      h: copiedPosition.h
    }
  });
};
var cloneLine = ({
  id,
  item,
  offsetMultiplier
}) => {
  const copiedPosition = item.position;
  return __spreadProps(__spreadValues({}, item), {
    item: id,
    position: {
      from: {
        x: copiedPosition.from.x + 20 * offsetMultiplier,
        y: copiedPosition.from.y + 20 * offsetMultiplier
      },
      to: {
        x: copiedPosition.to.x + 20 * offsetMultiplier,
        y: copiedPosition.to.y + 20 * offsetMultiplier
      }
    }
  });
};
var shiftViewportOnZoom = ({
  scrollLeft,
  scrollTop,
  offsetWidth,
  offsetHeight,
  scaleRatio
}) => {
  const middleOffsetWidth = offsetWidth / 2;
  const middleOffsetHeight = offsetHeight / 2;
  return {
    scrollLeft: (scrollLeft + middleOffsetWidth) * scaleRatio - middleOffsetWidth,
    scrollTop: (scrollTop + middleOffsetHeight) * scaleRatio - middleOffsetHeight
  };
};
var computeRelativePosition = (e, canvasRef, scale = 1) => {
  var _a;
  const pos = getClientPosition(e, scale);
  const canvasDomNode = canvasRef == null ? void 0 : canvasRef.current;
  const rect = (_a = canvasDomNode == null ? void 0 : canvasDomNode.getBoundingClientRect) == null ? void 0 : _a.call(canvasDomNode);
  if (pos && rect) {
    return {
      x: pos.x - rect.left / scale,
      y: pos.y - rect.top / scale
    };
  }
  return pos;
};
var scaleGridLayoutStructureByWidth = ({
  layout,
  scale = 1
}) => {
  if (scale === 1) {
    return layout;
  }
  const edges = getAllEdges(layout);
  const verticalEdges = (0, lodash_exports.sortBy)(
    edges.filter((edge) => edge.orientation === "vertical"),
    ["edgeStart.x"]
  );
  const newLayout = [];
  verticalEdges.forEach((edge) => {
    const {
      visualizations,
      edgeStart: { x }
    } = edge;
    const scaledEdgeX = Math.round(x * scale);
    const left = visualizations.filter((viz) => viz.position.x < x);
    const right = visualizations.filter((viz) => viz.position.x >= x);
    left.forEach((layoutItem) => {
      const newLayoutItem = newLayout.find(
        ({ item: id }) => layoutItem.item === id
      );
      if (newLayoutItem) {
        newLayoutItem.position.w = scaledEdgeX - newLayoutItem.position.x;
      }
    });
    right.forEach((layoutItem) => {
      newLayout.push(__spreadProps(__spreadValues({}, layoutItem), {
        position: __spreadProps(__spreadValues({}, layoutItem.position), {
          x: scaledEdgeX
        })
      }));
    });
  });
  return newLayout;
};

// src/utils/imageUtils.ts
var isFromImageRegistry = (src = "") => {
  const [type] = src.split("://");
  return src.indexOf("://") > -1 && type !== "http" && type !== "https" && type !== "ftp" && type !== "file";
};
var validImagePos = ({
  x,
  y,
  canvasWidth,
  canvasHeight
}) => !!(x != null && Number.isInteger(x) && y != null && Number.isInteger(y) && x <= canvasWidth && y <= canvasHeight && x >= 0 && y >= 0);
var validImageWidthHeight = (imageWidth = 1, imageHeight = 1) => !!(Number.isInteger(imageWidth) && Number.isInteger(imageHeight) && imageWidth > 0 && imageHeight > 0);
var sizeTypes = ["cover", "contain", "auto", void 0];
var invalidImageSizeType = (sizeType) => !sizeTypes.includes(sizeType);
var getImageDimension = (0, lodash_exports.memoize)(
  (imgSrc) => {
    const img = new Image();
    try {
      img.src = imgSrc;
      img.style.position = "absolute";
      img.style.left = "-9999";
      img.style.visibility = "hidden";
      document.body.appendChild(img);
      const height = img.naturalHeight;
      const width = img.naturalWidth;
      return { width, height };
    } catch (e) {
      return { width: 0, height: 0 };
    } finally {
      if (document.body.contains(img)) {
        document.body.removeChild(img);
      }
    }
  }
);
var validateOpacity = (opacity) => {
  if (typeof opacity !== "number" || opacity > 1) {
    return 1;
  }
  if (opacity < 0) {
    return 0;
  }
  return opacity;
};
var validateImageXY = ({
  canvasWidth,
  canvasHeight,
  x,
  y
}) => {
  if (!validImagePos({ x, y, canvasWidth, canvasHeight })) {
    dashboard_utils_exports.console.warn(
      `Invalid background image position, x: ${x} y: ${y}. Defaulting to x: 0, y: 0`
    );
    return [0, 0];
  }
  return [x, y];
};
var validateSizeType = ({
  w,
  h,
  sizeType
}) => {
  if (typeof w !== "number" && typeof h !== "number" && (typeof sizeType !== "string" || invalidImageSizeType(sizeType))) {
    dashboard_utils_exports.console.warn(
      `Invalid background image sizeType, ${sizeType}. Defaulting to contain`
    );
    return "contain";
  }
  return sizeType;
};
var validateImageWH = ({ w, h }) => {
  if (validImageWidthHeight(w, h)) {
    return [w, h];
  }
  dashboard_utils_exports.console.warn(
    `Invalid background image width and height, w: ${w} h: ${h}. Defaulting to image original size`
  );
  return [w != null ? w : 0, h != null ? h : 0];
};
var validateBackgroundImage = ({
  backgroundImage = {},
  canvasWidth,
  canvasHeight
}) => {
  var _a;
  if ((0, lodash_exports.isEmpty)(backgroundImage) || !backgroundImage.src) {
    return {
      src: backgroundImage.src,
      x: backgroundImage.x,
      y: backgroundImage.y,
      w: backgroundImage.w,
      h: backgroundImage.h,
      sizeType: backgroundImage.sizeType,
      opacity: (_a = backgroundImage.opacity) != null ? _a : 1
    };
  }
  const opacity = validateOpacity(backgroundImage.opacity);
  const [x, y] = validateImageXY({
    canvasWidth,
    canvasHeight,
    x: backgroundImage.x,
    y: backgroundImage.y
  });
  const sizeType = validateSizeType({
    w: backgroundImage.w,
    h: backgroundImage.h,
    sizeType: backgroundImage.sizeType
  });
  const [w, h] = validateImageWH({
    w: backgroundImage.w,
    h: backgroundImage.h
  });
  return { src: backgroundImage.src, x, y, w, h, sizeType, opacity };
};

// src/hooks/useBackgroundImage.ts
var useBackgroundImage = (backgroundImageSrc) => {
  const [imageSrc, setImageSrc] = (0, react_exports.useState)("");
  const imageRegistry = (0, dashboard_context_exports.useImageRegistry)();
  const fetchBackgroundImage = (registry, bgImageSrc) => __async(void 0, null, function* () {
    try {
      if (registry) {
        if (registry.isResourceURL(bgImageSrc)) {
          const imageObject = yield registry.getByURL(bgImageSrc);
          setImageSrc(imageObject.dataURI);
        } else {
          setImageSrc(bgImageSrc);
        }
      } else {
        setImageSrc(bgImageSrc);
      }
    } catch (error) {
      if (error instanceof Error) {
        dashboard_utils_exports.console.error(error.message);
      } else {
        dashboard_utils_exports.console.error("An unknown error occurred");
      }
    }
  });
  (0, react_exports.useEffect)(() => {
    if (backgroundImageSrc) {
      fetchBackgroundImage(
        imageRegistry,
        backgroundImageSrc
      );
    } else {
      setImageSrc("");
    }
  }, [imageRegistry, backgroundImageSrc]);
  return imageSrc;
};

// esm-externals:rxjs
var rxjs_exports = {};
__export(rxjs_exports, {
  default: () => rxjs_default
});
__reExport(rxjs_exports, rxjs_star);
import * as defaultImport3 from "rxjs";
import * as rxjs_star from "rxjs";
var rxjs_default = "default" in defaultImport3 ? defaultImport3.default : defaultImport3;

// src/hooks/useZoom.ts
var useZoom = ({
  canvasRef,
  width,
  height,
  containerWidth,
  containerHeight
}) => {
  const [internalScale, setInternalScale] = (0, react_exports.useState)(1);
  const [fitToWidth, setFitToWidth] = (0, react_exports.useState)(false);
  const scrollbarWidth = (0, react_exports.useMemo)(() => (0, dashboard_utils_exports.getScrollbarWidth)(), []);
  const keyboardListener = (0, dashboard_context_exports.useKeyboardListener)();
  const zoomObserver = (0, react_exports.useRef)(null);
  const currentStepRef = (0, react_exports.useRef)(1);
  currentStepRef.current = Math.round(internalScale / ZOOM_STEP_SIZE) * ZOOM_STEP_SIZE;
  const getZoomObserver = (0, react_exports.useCallback)(() => {
    if (zoomObserver.current == null) {
      zoomObserver.current = new rxjs_exports.BehaviorSubject(1);
    }
    return zoomObserver.current;
  }, []);
  (0, react_exports.useEffect)(() => {
    if (!fitToWidth) {
      return;
    }
    const newScale = computeScaleToFit({
      canvasWidth: width,
      canvasHeight: height,
      containerWidth,
      containerHeight,
      max: fitToWidth,
      scrollbarWidth
    });
    getZoomObserver().next(newScale);
    setInternalScale(newScale);
  }, [
    width,
    height,
    containerWidth,
    containerHeight,
    fitToWidth,
    scrollbarWidth,
    getZoomObserver
  ]);
  const setScale = (0, react_exports.useCallback)(
    (s) => {
      getZoomObserver().next(s);
      setInternalScale(s);
      setFitToWidth(false);
    },
    [getZoomObserver]
  );
  const scrollToTopLeft = (0, react_exports.useCallback)(() => {
    var _a;
    const layoutEl = (_a = canvasRef.current) == null ? void 0 : _a.parentElement;
    if (layoutEl) {
      layoutEl.scrollTop = 0;
      layoutEl.scrollLeft = 0;
    }
  }, [canvasRef]);
  const setFitToWidthAndScrollToTopLeft = (0, react_exports.useCallback)(
    (maxScale = Infinity) => {
      setFitToWidth(maxScale);
      scrollToTopLeft();
    },
    [scrollToTopLeft]
  );
  const zoomIn = (0, react_exports.useCallback)(() => {
    setScale(currentStepRef.current + ZOOM_STEP_SIZE);
  }, [setScale]);
  const zoomOut = (0, react_exports.useCallback)(() => {
    const nextStep = currentStepRef.current - ZOOM_STEP_SIZE;
    if (nextStep >= ZOOM_STEP_SIZE) {
      setScale(nextStep);
    }
  }, [setScale]);
  const handleKeyboardZoom = (0, react_exports.useCallback)(
    ({
      dir,
      preventDefault
    }) => {
      switch (dir) {
        case "in":
          zoomIn();
          break;
        case "out":
          zoomOut();
          break;
        case "reset":
          setFitToWidthAndScrollToTopLeft();
          break;
        default:
          break;
      }
      preventDefault();
    },
    [setFitToWidthAndScrollToTopLeft, zoomIn, zoomOut]
  );
  (0, react_exports.useEffect)(() => {
    if (!keyboardListener) {
      return dashboard_utils_exports.noop;
    }
    const unsubscribeZoom = keyboardListener.subscribe(
      "zoom",
      handleKeyboardZoom
    );
    return () => {
      unsubscribeZoom();
    };
  }, [handleKeyboardZoom, keyboardListener]);
  return {
    scale: internalScale,
    fitToWidth,
    getZoomObserver,
    setScale,
    zoomIn,
    zoomOut,
    setFitToWidthAndScrollToTopLeft
  };
};

// esm-externals:@splunk/dashboard-state
var dashboard_state_exports = {};
__export(dashboard_state_exports, {
  default: () => dashboard_state_default
});
__reExport(dashboard_state_exports, dashboard_state_star);
import * as defaultImport4 from "@splunk/dashboard-state";
import * as dashboard_state_star from "@splunk/dashboard-state";
var dashboard_state_default = "default" in defaultImport4 ? defaultImport4.default : defaultImport4;

// esm-externals:@splunk/datasource-utils
var datasource_utils_exports = {};
__export(datasource_utils_exports, {
  default: () => datasource_utils_default
});
__reExport(datasource_utils_exports, datasource_utils_star);
import * as defaultImport5 from "@splunk/datasource-utils";
import * as datasource_utils_star from "@splunk/datasource-utils";
var datasource_utils_default = "default" in defaultImport5 ? defaultImport5.default : defaultImport5;

// src/utils/showHideUtils.ts
var hasNoResults = (searchData) => {
  var _a, _b, _c;
  if (typeof searchData === "undefined") {
    return true;
  }
  return ((_b = (_a = searchData.meta) == null ? void 0 : _a.totalCount) != null ? _b : 0) === 0 && ((_c = searchData == null ? void 0 : searchData.meta) == null ? void 0 : _c.status) !== datasource_utils_exports.DS_STATUS.REFRESHING;
};
var shouldHideItem = ({
  item,
  definition,
  searchData,
  conditionResults
}) => {
  var _a, _b, _c, _d;
  if (!definition) {
    return false;
  }
  if (item.type === "line") {
    return false;
  }
  const {
    hideWhenNoData,
    conditions = [],
    hideWhenConditions
  } = (_b = (_a = definition.containerOptions) == null ? void 0 : _a.visibility) != null ? _b : {};
  if (!!(hideWhenNoData != null ? hideWhenNoData : definition.hideWhenNoData) && hasNoResults(searchData)) {
    return true;
  }
  if (((_c = definition.containerOptions) == null ? void 0 : _c.visibility) && "hideInViewMode" in definition.containerOptions.visibility && !!((_d = definition.containerOptions.visibility) == null ? void 0 : _d.hideInViewMode)) {
    return true;
  }
  if (conditions.length !== 0) {
    return hideWhenConditions === "all-true" ? conditions.every((condId) => conditionResults[condId]) : conditions.some((condId) => conditionResults[condId]);
  }
  return false;
};

// src/hooks/useLayoutShowHide.ts
var getCondIds = (allItems) => allItems === void 0 ? [] : Object.values(allItems).flatMap(
  (itemDef) => {
    var _a, _b, _c;
    return (_c = (_b = (_a = itemDef.containerOptions) == null ? void 0 : _a.visibility) == null ? void 0 : _b.conditions) != null ? _c : [];
  }
);
var useLayoutShowHide = (layoutStructure, handleReflow) => {
  const allVizs = (0, dashboard_state_exports.useSelector)(dashboard_state_exports.selectVisualizations);
  const allInputs = (0, dashboard_state_exports.useSelector)(dashboard_state_exports.selectInputs);
  const mode = (0, dashboard_state_exports.useSelector)(dashboard_state_exports.selectMode);
  const allConditionIds = (0, react_exports.useMemo)(
    () => /* @__PURE__ */ new Set([...getCondIds(allVizs), ...getCondIds(allInputs)]),
    [allInputs, allVizs]
  );
  const conditionResults = (0, dashboard_state_exports.useSelector)(
    (state) => (0, dashboard_state_exports.selectConditionResults)(state, Array.from(allConditionIds))
  );
  const subscriptions = (0, dashboard_ui_exports.useBatchSubscribeToSearches)({
    consumerId: "<layout>"
  });
  const { enableShowHide } = (0, dashboard_context_exports.useFeatureFlags)();
  return (0, react_exports.useMemo)(() => {
    if (!enableShowHide || mode === "edit") {
      return layoutStructure;
    }
    const itemsToRemove = [];
    const filteredStructure = layoutStructure.filter((item) => {
      var _a;
      if (shouldHideItem({
        item,
        definition: (_a = allVizs[item.item]) != null ? _a : allInputs == null ? void 0 : allInputs[item.item],
        searchData: subscriptions[item.item],
        conditionResults
      })) {
        itemsToRemove.push(item);
        return false;
      }
      return true;
    });
    return itemsToRemove.length && handleReflow ? handleReflow(layoutStructure, itemsToRemove) : filteredStructure;
  }, [
    enableShowHide,
    mode,
    layoutStructure,
    handleReflow,
    allVizs,
    allInputs,
    subscriptions,
    conditionResults
  ]);
};

// src/hooks/useMouseDownWithHandleEventListeners.ts
var useMouseDownWithHandleEventListeners = ({
  handleMouseDownOnItem,
  canvasRef,
  scale,
  setStartPosition,
  movable,
  setIsMoving,
  isMovingRef
}) => {
  const handleMouseDownOnVizWithHandle = (0, dashboard_ui_exports.useEventCallback)(
    (e) => {
      const { vizId, initialEvent, type } = e.detail;
      const pos = computeRelativePosition(initialEvent, canvasRef, scale);
      setStartPosition(pos);
      const block = {
        id: vizId,
        type
      };
      handleMouseDownOnItem(initialEvent, block);
    }
  );
  const handleMouseDownOnHandle = (0, dashboard_ui_exports.useEventCallback)((e) => {
    const { vizId, initialEvent, type } = e.detail;
    const pos = computeRelativePosition(initialEvent, canvasRef, scale);
    setStartPosition(pos);
    const block = {
      id: vizId,
      type
    };
    handleMouseDownOnItem(initialEvent, block);
    if (movable) {
      setIsMoving(true);
      if (isMovingRef) {
        isMovingRef.current = true;
      }
    }
  });
  (0, dashboard_ui_exports.useEventListener)({
    eventName: dashboard_ui_exports.EVENT_MOUSE_DOWN_ON_VIZ_WITH_HANDLE,
    target: document,
    listener: handleMouseDownOnVizWithHandle
  });
  (0, dashboard_ui_exports.useEventListener)({
    eventName: dashboard_ui_exports.EVENT_MOUSE_DOWN_ON_HANDLE,
    target: document,
    listener: handleMouseDownOnHandle
  });
};

// src/components/Canvas/BackgroundImage.tsx
var toBackgroundImageSize = ({
  src,
  imageHeight,
  imageWidth,
  sizeType
}) => {
  const { width: w, height: h } = getImageDimension(src != null ? src : "");
  if (imageWidth && imageHeight) {
    return `${(0, dashboard_utils_exports.toPx)(imageWidth)} ${(0, dashboard_utils_exports.toPx)(imageHeight)}`;
  }
  if (imageWidth || imageHeight) {
    return `${(0, dashboard_utils_exports.toPx)(imageWidth != null ? imageWidth : w)} ${(0, dashboard_utils_exports.toPx)(imageHeight != null ? imageHeight : h)}`;
  }
  return sizeType != null ? sizeType : "contain";
};
var BackgroundImageDiv = styled_components_default.div.attrs(() => ({
  "data-test": "background-image"
}))`
    ${themes_exports.mixins.reset("inline-block")};
    width: 100%;
    height: 100%;
    position: relative;
    background-repeat: no-repeat;
    background-image: ${(props) => `url("${props.src}")`};
    background-size: ${(props) => toBackgroundImageSize(props)};
    background-position: ${(props) => `${(0, dashboard_utils_exports.toPx)(props.imageX)} ${(0, dashboard_utils_exports.toPx)(props.imageY)}`};
    opacity: ${(prop) => validateOpacity(prop.opacity)};
`;
var BackgroundImage = (_a) => {
  var _b = _a, { src } = _b, rest = __objRest(_b, ["src"]);
  const imageSrc = useBackgroundImage(src);
  if (!imageSrc) {
    return null;
  }
  return /* @__PURE__ */ react_default.createElement(BackgroundImageDiv, __spreadProps(__spreadValues({}, rest), { src: imageSrc }));
};

// src/components/Canvas/CanvasBorder.tsx
var Border = styled_components_default.div.attrs(() => ({
  "data-test": "canvas-border"
}))`
    position: absolute;
    box-sizing: border-box;
    border: 2px dashed
        ${(0, themes_exports.pick)({
  enterprise: {
    light: themes_exports.variables.gray80,
    dark: themes_exports.variables.gray30
  },
  prisma: themes_exports.variables.interactiveColorBorder
})};
    left: 0px;
    top: 0px;
    right: 0px;
    bottom: 0px;
    width: 100%;
    height: 100%;
`;
var CanvasBorder = ({
  showBorder = false
}) => {
  if (!showBorder) {
    return null;
  }
  return /* @__PURE__ */ react_default.createElement(Border, null);
};

// src/components/Canvas/CanvasGridlines.tsx
var GRID_LINE_OPACITY = 0.1;
var getGridLineColor = (props) => {
  var _a;
  return (_a = props.gridLineColor) != null ? _a : (0, themes_exports.pick)({
    light: "#D8D8D8",
    dark: "#9B9B9B"
  });
};
var GridLines = styled_components_default.div`
    position: absolute;
    box-sizing: border-box;
    left: 0px;
    top: 0px;
    right: 0px;
    bottom: 0px;
    width: 100%;
    height: 100%;
    opacity: ${(prop) => {
  var _a;
  return (_a = prop.gridLineOpacity) != null ? _a : GRID_LINE_OPACITY;
}};
    background-size: ${(prop) => prop.gridWidth + prop.gridLineWidth}px
        ${(prop) => prop.gridHeight + prop.gridLineWidth}px;
    background-image: repeating-linear-gradient(
            0deg,
            ${getGridLineColor},
            ${getGridLineColor} ${(prop) => (0, dashboard_utils_exports.toPx)(prop.gridLineWidth)},
            transparent ${(prop) => prop.gridLineWidth}px,
            transparent ${(prop) => prop.gridHeight + prop.gridLineWidth}px
        ),
        repeating-linear-gradient(
            -90deg,
            ${getGridLineColor},
            ${getGridLineColor} ${(prop) => (0, dashboard_utils_exports.toPx)(prop.gridLineWidth)},
            transparent ${(prop) => prop.gridLineWidth}px,
            transparent ${(prop) => prop.gridWidth + prop.gridLineWidth}px
        );
`;
var CanvasGridlines = (_a) => {
  var _b = _a, { showGrid } = _b, rest = __objRest(_b, ["showGrid"]);
  if (!showGrid) {
    return null;
  }
  return /* @__PURE__ */ react_default.createElement(GridLines, __spreadProps(__spreadValues({}, rest), { "data-test": "gridlines" }));
};

// src/components/Canvas/Background.tsx
var toScale = (scale) => scale ? {
  transform: `scale(${scale})`,
  transformOrigin: "0 0",
  width: `${100 / scale}%`
} : {
  width: "100%"
};
var Background = styled_components_default.div.attrs((props) => ({
  style: __spreadValues({
    height: props.height
  }, toScale(props.scale))
}))`
    ${themes_exports.mixins.reset("inline-block")};
    position: relative;
    background: ${(prop) => prop.bgColor || // NOTE: this needs to match packages/dashboard-editors/src/layouts/AbsoluteLayoutEditor.jsx
dashboard_ui_exports.customThemeVariables.dashboardBackgroundColor};
`;

// src/components/Canvas/Canvas.tsx
var CanvasContainer = styled_components_default.div.attrs((props) => ({
  style: { width: props.width, height: props.height }
}))`
    ${themes_exports.mixins.reset("block")};
    overflow: ${(prop) => prop.showOverflowContent ? "visible" : "hidden"};
    user-select: ${(prop) => prop.userSelect ? "text" : "none"};
    position: relative;
`;
var Canvas = (_a, canvasRef) => {
  var _b = _a, {
    width,
    height,
    scale,
    backgroundColor: unsanitizedBgColor = "",
    backgroundImageSrc,
    backgroundImageSizeType,
    backgroundImageWidth,
    backgroundImageHeight,
    backgroundImagePositionX,
    backgroundImagePositionY,
    backgroundImageOpacity,
    gridLineOpacity,
    children,
    showOverflowContent = false,
    userSelect = false,
    showGrid = false,
    gridPadding = 0,
    gridLineWidth = 1,
    gridLineColor,
    gridWidth = 9,
    gridHeight = 9,
    showBorder = false,
    cssScaling = true
  } = _b, others = __objRest(_b, [
    "width",
    "height",
    "scale",
    "backgroundColor",
    "backgroundImageSrc",
    "backgroundImageSizeType",
    "backgroundImageWidth",
    "backgroundImageHeight",
    "backgroundImagePositionX",
    "backgroundImagePositionY",
    "backgroundImageOpacity",
    "gridLineOpacity",
    "children",
    "showOverflowContent",
    "userSelect",
    "showGrid",
    "gridPadding",
    "gridLineWidth",
    "gridLineColor",
    "gridWidth",
    "gridHeight",
    "showBorder",
    "cssScaling"
  ]);
  const containerWidth = scale != null ? width * scale : width;
  const containerHeight = scale != null && cssScaling ? height * scale : height;
  const backgroundColor = (0, dashboard_ui_exports.sanitizeColor)(unsanitizedBgColor);
  return /* @__PURE__ */ react_default.createElement(
    CanvasContainer,
    __spreadValues({
      "data-test": "canvas-container",
      "data-width": containerWidth,
      "data-height": containerHeight,
      width: cssScaling ? containerWidth : width,
      height: containerHeight,
      showOverflowContent,
      userSelect,
      ref: canvasRef
    }, others),
    /* @__PURE__ */ react_default.createElement(
      Background,
      {
        "data-test": "canvas",
        "data-width": width,
        "data-height": height,
        "data-scale": scale,
        height,
        scale: cssScaling ? scale : void 0,
        bgColor: backgroundColor
      },
      /* @__PURE__ */ react_default.createElement(
        BackgroundImage,
        {
          src: backgroundImageSrc,
          sizeType: backgroundImageSizeType,
          imageWidth: backgroundImageWidth,
          imageHeight: backgroundImageHeight,
          imageX: backgroundImagePositionX,
          imageY: backgroundImagePositionY,
          opacity: backgroundImageOpacity
        }
      ),
      /* @__PURE__ */ react_default.createElement(CanvasBorder, { showBorder }),
      /* @__PURE__ */ react_default.createElement(
        CanvasGridlines,
        {
          showGrid,
          gridLineOpacity,
          gridPadding,
          gridLineWidth,
          gridLineColor,
          gridWidth,
          gridHeight
        }
      ),
      children
    )
  );
};
var Canvas_default = (0, react_exports.forwardRef)(Canvas);

// src/components/SelectBox.jsx
var SelectDiv = styled_components_default.div.attrs(({ x, y, w, h }) => ({
  style: {
    width: w,
    height: h,
    transform: `translate(${x}px, ${y}px)`
  }
}))`
    ${themes_exports.mixins.reset("block")};
    position: absolute;
    background: transparent;
    // the next line is important to avoid blocking the drilldowns, and it allows to select text from visualizations
    pointer-events: none;
    border: 1px solid
        ${(0, themes_exports.pick)({
  enterprise: themes_exports.variables.accentColorL10,
  prisma: themes_exports.variables.interactiveColorPrimary
})};
    z-index: 1000;
`;
var Wrapper = styled_components_default.div`
    ${themes_exports.mixins.reset("block")};
    background: transparent;
    position: absolute;
    top: 0;
    left: 0;
    width: 100%;
    height: 100%;
`;
var SelectBox = ({ start, end }) => /* @__PURE__ */ react_default.createElement(Wrapper, null, /* @__PURE__ */ react_default.createElement(
  SelectDiv,
  {
    w: Math.abs(start.x - end.x),
    h: Math.abs(start.y - end.y),
    x: Math.min(start.x, end.x),
    y: Math.min(start.y, end.y)
  }
));
SelectBox.propTypes = {
  /**
   * starting position
   */
  start: prop_types_default.shape({
    x: prop_types_default.number,
    y: prop_types_default.number
  }),
  /**
   * end position
   */
  end: prop_types_default.shape({
    x: prop_types_default.number,
    y: prop_types_default.number
  })
};
var SelectBox_default = SelectBox;

// src/utils/findStatusIcon.ts
var findStatusIcon = ({
  canvasRef,
  id
}) => {
  const canvas = canvasRef == null ? void 0 : canvasRef.current;
  if (!canvas) {
    return null;
  }
  return canvas.querySelector(
    `[data-id="${id}"] [data-test="simple-status-icon-container"]`
  );
};

// src/components/AbsoluteCanvas.tsx
var defaultState = {
  startPosition: null,
  currentPosition: null,
  isMoving: false,
  isSelecting: false
};
var DEFAULT_SELECTED_LAYOUT_ITEMS = [];
var AbsoluteCanvas = (props) => {
  const _a = props, {
    scale,
    children,
    canvasRef,
    blockItems,
    onItemMove = dashboard_utils_exports.noop,
    onItemMoved = dashboard_utils_exports.noop,
    onItemSelected,
    movable = false,
    allowMultiselect,
    selectedLayoutItems = DEFAULT_SELECTED_LAYOUT_ITEMS,
    initialState = defaultState,
    onMouseDownCapture,
    onMouseUpCapture
  } = _a, canvasProps = __objRest(_a, [
    "scale",
    "children",
    "canvasRef",
    "blockItems",
    "onItemMove",
    "onItemMoved",
    "onItemSelected",
    "movable",
    "allowMultiselect",
    "selectedLayoutItems",
    "initialState",
    "onMouseDownCapture",
    "onMouseUpCapture"
  ]);
  const [startPosition, setStartPosition] = (0, react_exports.useState)(
    initialState.startPosition
  );
  const [currentPosition, setCurrentPosition] = (0, react_exports.useState)(
    initialState.currentPosition
  );
  const [isMoving, setIsMoving] = (0, react_exports.useState)(initialState.isMoving);
  const [isSelecting, setIsSelecting] = (0, react_exports.useState)(initialState.isSelecting);
  const isMovingRef = (0, react_exports.useRef)(false);
  const { isMouseDownOnItemWithHandle, setIsMouseDownOnItemWithHandle } = (0, dashboard_context_exports.useMouseEventsContext)();
  const isMetaKeyUsed = (0, react_exports.useRef)(false);
  const isSelectedWithMetaKey = (0, react_exports.useRef)(false);
  const isMacOS = (0, dashboard_utils_exports.isMac)();
  const profiler = (0, dashboard_telemetry_exports.useDashboardProfiler)();
  const resetState = (0, react_exports.useCallback)(() => {
    setStartPosition(defaultState.startPosition);
    setCurrentPosition(defaultState.currentPosition);
    setIsMoving(defaultState.isMoving);
    setIsSelecting(defaultState.isSelecting);
    isMovingRef.current = false;
    isMetaKeyUsed.current = false;
    isSelectedWithMetaKey.current = false;
  }, []);
  (0, react_exports.useEffect)(() => {
    if (isMovingRef.current && blockItems && (profiler == null ? void 0 : profiler.hasPartialMeasurement(dashboard_telemetry_exports.VIZ_MOVE_EVENT))) {
      profiler == null ? void 0 : profiler.endMeasurement(dashboard_telemetry_exports.VIZ_MOVE_EVENT);
    }
  }, [blockItems, profiler]);
  (0, react_exports.useEffect)(() => {
    if (selectedLayoutItems.length) {
      profiler == null ? void 0 : profiler.emitAndClearTimer({
        timerName: dashboard_telemetry_exports.VIZ_SELECT_EVENT,
        metadata: {
          numOfSelectedItems: selectedLayoutItems.length
        }
      });
    }
  }, [profiler, selectedLayoutItems]);
  const handleItemSelected = (0, react_exports.useCallback)(
    (e, items = []) => {
      if (items.some(
        ({ id, type }) => {
          var _a2;
          return type === "input" && ((_a2 = findStatusIcon({ canvasRef, id })) == null ? void 0 : _a2.contains(
            e.target
          ));
        }
      )) {
        return;
      }
      profiler == null ? void 0 : profiler.startTimer({
        timerName: dashboard_telemetry_exports.VIZ_SELECT_EVENT
      });
      if (isMacOS && e.metaKey || !isMacOS && e.ctrlKey) {
        const selectedItems = selectedLayoutItems.filter(
          ({ type }) => type === "block" || type === "input"
        );
        isSelectedWithMetaKey.current = true;
        onItemSelected(e, (0, lodash_exports.uniqBy)([...selectedItems, ...items], "id"));
      } else {
        onItemSelected(e, items);
      }
    },
    [canvasRef, profiler, selectedLayoutItems, onItemSelected, isMacOS]
  );
  const handleMouseDownOnItem = (0, react_exports.useCallback)(
    (event, blockItem) => {
      if (isMacOS && event.metaKey || !isMacOS && event.ctrlKey) {
        isMetaKeyUsed.current = true;
      }
      const { id, type = "block" } = blockItem;
      if (selectedLayoutItems.find(
        ({ id: selectedId }) => selectedId === id
      )) {
        return;
      }
      handleItemSelected(event, [{ id, type }]);
    },
    [handleItemSelected, selectedLayoutItems, isMacOS]
  );
  const handleMouseMove = (0, dashboard_ui_exports.useEventCallback)((e) => {
    if (!startPosition) {
      return;
    }
    const newPosition = computeRelativePosition(e, canvasRef, scale);
    const offset = getOffset(newPosition, startPosition);
    if (isMoving && considerMoved(offset)) {
      e.stopImmediatePropagation();
      e.preventDefault();
      onItemMove(e, offset);
    }
    setCurrentPosition(newPosition);
  });
  const handleMouseUpWithMetaKey = (event, item) => {
    var _a2;
    if (!item) {
      return;
    }
    const { item: itemId } = item;
    if (itemId && isMetaKeyUsed.current && !isSelectedWithMetaKey.current) {
      const selectedItems = selectedLayoutItems.filter(
        ({ type }) => type === "block" || type === "input"
      );
      const alreadySelected = selectedItems.filter(
        (element) => element.id === itemId
      );
      if (alreadySelected.length) {
        const removedAlreadySelected = selectedItems.filter(
          (element) => element.id !== itemId
        );
        onItemSelected(
          event,
          (0, lodash_exports.uniqBy)([...removedAlreadySelected], "id")
        );
      } else {
        handleItemSelected(event, [
          { id: itemId, type: (_a2 = item == null ? void 0 : item.type) != null ? _a2 : "block" }
        ]);
      }
    }
  };
  const handleMouseUp = (0, dashboard_ui_exports.useEventCallback)((e) => {
    if (!startPosition) {
      return;
    }
    e.preventDefault();
    const curPosition = computeRelativePosition(e, canvasRef, scale);
    const offset = getOffset(curPosition, startPosition);
    const isMoved = considerMoved(offset);
    if (isMoving && isMoved) {
      isMovingRef.current = false;
      onItemMoved(e, offset);
    } else if (isSelecting) {
      const boundary = positionsToBoundary(startPosition, curPosition);
      const selectedBlocks = filterBlockItemsByBoundary(
        blockItems,
        boundary
      ).map(
        ({ item, type = "block" }) => ({
          id: item,
          type
        })
      );
      let selectedBlockItems = [];
      if (!isMoved && selectedBlocks.length) {
        selectedBlockItems.push((0, lodash_exports.last)(selectedBlocks));
      } else {
        selectedBlockItems = selectedBlocks;
      }
      handleItemSelected(e, selectedBlockItems);
    } else {
      const item = findTopBlockItemByPosition(blockItems, curPosition);
      const { item: itemId, type = "block" } = item != null ? item : {};
      if (isMetaKeyUsed.current) {
        handleMouseUpWithMetaKey(e, item);
      } else if (itemId && !(selectedLayoutItems.length === 1 && selectedLayoutItems[0].id === itemId) && !isMoved && !isMetaKeyUsed.current) {
        handleItemSelected(e, [{ id: itemId, type }]);
      } else if (!itemId && !findTopBlockItemByPosition(blockItems, startPosition)) {
        handleItemSelected(e, []);
      }
    }
    resetState();
  });
  const handleMouseDown = (0, dashboard_ui_exports.useEventCallback)(
    (e, selectedBlockItem) => {
      if (isMouseDownOnItemWithHandle == null ? void 0 : isMouseDownOnItemWithHandle.current) {
        setIsMouseDownOnItemWithHandle(false);
        return;
      }
      const pos = computeRelativePosition(e, canvasRef, scale);
      setStartPosition(pos);
      const block = selectedBlockItem != null ? selectedBlockItem : findTopBlockItemByPosition(blockItems, pos);
      if (!block) {
        if (allowMultiselect) {
          setIsSelecting(true);
        }
        return;
      }
      const { item, type } = block;
      handleMouseDownOnItem(e, {
        id: item,
        type
      });
      if (movable) {
        setIsMoving(true);
        isMovingRef.current = true;
      }
    }
  );
  const handleMouseDownCapture = (0, dashboard_ui_exports.useEventCallback)((e) => {
    onMouseDownCapture == null ? void 0 : onMouseDownCapture(e);
    const pos = computeRelativePosition(e, canvasRef, scale);
    const selectedBlockItems = blockItems.filter(
      (item) => selectedLayoutItems.some(
        (selectedItem) => selectedItem.id === item.item
      )
    );
    const topSelectedBlock = findTopBlockItemByPosition(
      selectedBlockItems,
      pos
    );
    const topBlock = findTopBlockItemByPosition(blockItems, pos);
    if (topSelectedBlock && topSelectedBlock !== topBlock) {
      handleMouseDown(e, topSelectedBlock);
      e.stopPropagation();
      e.preventDefault();
    }
  });
  (0, react_exports.useEffect)(() => {
    document.addEventListener("mousemove", handleMouseMove, true);
    return () => {
      document.removeEventListener("mousemove", handleMouseMove, true);
    };
  }, [handleMouseMove]);
  (0, react_exports.useEffect)(() => {
    document.addEventListener("mouseup", handleMouseUp);
    return () => {
      document.removeEventListener("mouseup", handleMouseUp);
    };
  }, [handleMouseUp]);
  useMouseDownWithHandleEventListeners({
    handleMouseDownOnItem,
    canvasRef,
    scale,
    setStartPosition,
    movable,
    setIsMoving,
    isMovingRef
  });
  const selectBox = (0, react_exports.useMemo)(() => {
    if (allowMultiselect && isSelecting && !isMoving && startPosition && currentPosition) {
      return /* @__PURE__ */ react_default.createElement(SelectBox_default, { start: startPosition, end: currentPosition });
    }
    return null;
  }, [
    allowMultiselect,
    isSelecting,
    isMoving,
    startPosition,
    currentPosition
  ]);
  return /* @__PURE__ */ react_default.createElement(
    Canvas_default,
    __spreadProps(__spreadValues({
      ref: canvasRef
    }, canvasProps), {
      scale,
      onMouseDown: handleMouseDown,
      onContextMenu: resetState,
      onMouseDownCapture: handleMouseDownCapture,
      onMouseUpCapture
    }),
    selectBox,
    children
  );
};
var AbsoluteCanvas_default = (0, react_exports.forwardRef)((props, ref) => {
  return /* @__PURE__ */ react_default.createElement(AbsoluteCanvas, __spreadValues({ canvasRef: ref }, props));
});

// src/components/AbsoluteItem.jsx
var AbsoluteDiv = styled_components_default.div.attrs(({ x, y, w, h }) => ({
  style: {
    width: w,
    height: h,
    top: `${y}px`,
    left: `${x}px`
  }
}))`
    display: block;
    position: absolute;
    background: transparent;
`;
var AbsoluteItem = ({
  itemId,
  type = "block",
  x,
  y,
  w,
  h,
  renderLayoutItem
}) => /* @__PURE__ */ react_default.createElement(
  AbsoluteDiv,
  {
    x,
    y,
    w,
    h,
    "data-test": "absolute-item",
    "data-x": x,
    "data-y": y,
    "data-w": w,
    "data-h": h
  },
  renderLayoutItem(
    itemId,
    {
      width: w,
      height: h
    },
    type
  )
);
AbsoluteItem.propTypes = {
  itemId: prop_types_default.string.isRequired,
  type: prop_types_default.string,
  x: prop_types_default.number.isRequired,
  y: prop_types_default.number.isRequired,
  w: prop_types_default.number.isRequired,
  h: prop_types_default.number.isRequired,
  renderLayoutItem: prop_types_default.func.isRequired
};
var AbsoluteItem_default = AbsoluteItem;

// src/components/Edge.tsx
var EdgeWrapper = styled_components_default.div.attrs(
  ({ x, y, w, h }) => {
    return {
      style: {
        width: w,
        height: h,
        transform: `translate(${x}px, ${y}px)`
      }
    };
  }
)`
    display: block;
    position: absolute;
    background: transparent;
    box-sizing: border-box;

    &:hover {
        cursor: ${(props) => props.orientation === "vertical" ? "col-resize" : "row-resize"};
        background: ${(0, themes_exports.pickVariant)("appearance", {
  invalid: dashboard_ui_exports.customThemeVariables.invalidDropTargetBackgroundColor,
  dropTarget: dashboard_ui_exports.customThemeVariables.validDropTargetBackgroundColor,
  normal: "transparent",
  hidden: "transparent"
})};
        border: ${(0, themes_exports.pickVariant)("appearance", {
  invalid: `1px solid ${dashboard_ui_exports.customThemeVariables.invalidDropTargetBorderColor}`,
  dropTarget: `1px solid ${dashboard_ui_exports.customThemeVariables.validDropTargetBorderColor}`,
  normal: "none",
  hidden: "none"
})};
        transition: background 0.2s ease-out;
    }
`;
var EdgeComponent = styled_components_default.div.attrs(
  ({ x, y, w, h }) => {
    return {
      style: {
        width: w,
        height: h,
        transform: `translate(${x}px, ${y}px)`
      }
    };
  }
)`
    display: block;
    position: absolute;
    opacity: 0.35;
    pointer-events: none;
    background-color: ${(props) => props.appearance === "dropTarget" ? dashboard_ui_exports.customThemeVariables.validDropTargetBorderColor : (0, themes_exports.pick)({
  enterprise: themes_exports.variables.accentColorL10,
  prisma: themes_exports.variables.interactiveColorPrimary
})};
    ${EdgeWrapper}:hover + & {
        opacity: 0.7;
    }
`;
var Edge = ({
  itemId,
  x,
  y,
  h,
  w,
  orientation,
  isCanvasEdge,
  padding = 0,
  appearance = "normal",
  onMouseDown,
  "data-test-edge-position": edgePositionTestHook
}) => {
  const handleMouseDown = (0, react_exports.useCallback)(
    (e) => {
      e.preventDefault();
      e.stopPropagation();
      onMouseDown(e, { id: itemId });
    },
    [onMouseDown, itemId]
  );
  if (appearance === "hidden" || isCanvasEdge && appearance !== "dropTarget" && (orientation === "vertical" || y + padding === 0)) {
    return null;
  }
  const coord = { x, y, w, h };
  const shift = padding / 2;
  if (orientation === "horizontal") {
    coord.y += shift;
    coord.h -= padding;
  }
  if (orientation === "vertical") {
    coord.x += shift;
    coord.w -= padding;
  }
  return /* @__PURE__ */ react_default.createElement(react_default.Fragment, null, /* @__PURE__ */ react_default.createElement(
    EdgeWrapper,
    {
      x,
      y,
      w,
      h,
      orientation,
      appearance,
      onMouseDown: handleMouseDown,
      "data-test": "edge",
      "data-test-edge-position": edgePositionTestHook
    }
  ), /* @__PURE__ */ react_default.createElement(
    EdgeComponent,
    {
      x: coord.x,
      y: coord.y,
      w: coord.w,
      h: coord.h,
      appearance
    }
  ));
};

// src/components/GridCanvas.tsx
var defaultState2 = {
  startPosition: null,
  isMoving: false
};
var DEFAULT_SELECTED_LAYOUT_ITEMS2 = [];
var noop3 = () => void 0;
var GridCanvas = (props) => {
  const _a = props, {
    movable = false,
    selectable = false,
    selectedLayoutItems = DEFAULT_SELECTED_LAYOUT_ITEMS2,
    panelPadding = 0,
    scale = 1,
    onItemMove = noop3,
    onItemMoved = noop3,
    onItemSelected,
    children,
    canvasRef,
    blockItems
  } = _a, canvasProps = __objRest(_a, [
    "movable",
    "selectable",
    "selectedLayoutItems",
    "panelPadding",
    "scale",
    "onItemMove",
    "onItemMoved",
    "onItemSelected",
    "children",
    "canvasRef",
    "blockItems"
  ]);
  const { enableGridLayoutCssScaling } = (0, dashboard_context_exports.useFeatureFlags)();
  const [startPosition, setStartPosition] = (0, react_exports.useState)(
    defaultState2.startPosition
  );
  const [isMoving, setIsMoving] = (0, react_exports.useState)(defaultState2.isMoving);
  const { isMouseDownOnItemWithHandle, setIsMouseDownOnItemWithHandle } = (0, dashboard_context_exports.useMouseEventsContext)();
  const resetState = (0, react_exports.useCallback)(() => {
    setIsMoving(defaultState2.isMoving);
    setStartPosition(defaultState2.startPosition);
  }, []);
  const mouseScale = enableGridLayoutCssScaling ? scale : 1;
  const handleItemSelected = (0, react_exports.useCallback)(
    (e, itemIds = []) => {
      if (itemIds.some(
        ({ id, type }) => {
          var _a2;
          return type === "input" && ((_a2 = findStatusIcon({ canvasRef, id })) == null ? void 0 : _a2.contains(
            e.target
          ));
        }
      )) {
        return;
      }
      if (e.metaKey) {
        const selectedItems = selectedLayoutItems.filter(
          ({ type }) => type === "block"
        );
        onItemSelected(e, (0, lodash_exports.uniqBy)([...selectedItems, ...itemIds], "id"));
      } else {
        onItemSelected(e, itemIds);
      }
    },
    [canvasRef, selectedLayoutItems, onItemSelected]
  );
  const handleMouseDownOnItem = (0, react_exports.useCallback)(
    (event, blockItem) => {
      const { id, type = "block" } = blockItem;
      if (!selectedLayoutItems.find(
        ({ id: selectedId }) => selectedId === id
      )) {
        handleItemSelected(event, [{ id, type }]);
      }
    },
    [handleItemSelected, selectedLayoutItems]
  );
  const handleMouseDown = (0, react_exports.useCallback)(
    (e) => {
      if (isMouseDownOnItemWithHandle == null ? void 0 : isMouseDownOnItemWithHandle.current) {
        setIsMouseDownOnItemWithHandle(false);
        return;
      }
      const pos = computeRelativePosition(e, canvasRef, mouseScale);
      const block = findTopBlockItemByPosition(
        blockItems,
        pos,
        panelPadding
      );
      if (selectable) {
        setStartPosition(pos);
      }
      if (block) {
        handleMouseDownOnItem(e, {
          id: block.item,
          type: block.type
        });
        if (movable) {
          setIsMoving(true);
        }
      }
    },
    [
      isMouseDownOnItemWithHandle,
      canvasRef,
      mouseScale,
      blockItems,
      panelPadding,
      selectable,
      setIsMouseDownOnItemWithHandle,
      handleMouseDownOnItem,
      movable
    ]
  );
  const handleMouseMove = (0, dashboard_ui_exports.useEventCallback)((e) => {
    if (startPosition) {
      const currentPosition = computeRelativePosition(
        e,
        canvasRef,
        mouseScale
      );
      if (currentPosition) {
        const offset = getOffset(currentPosition, startPosition);
        if (isMoving && considerMoved(offset)) {
          e.preventDefault();
          onItemMove(e, { currentPosition });
        }
      }
    }
  });
  const handleMouseUp = (0, dashboard_ui_exports.useEventCallback)((e) => {
    if (startPosition) {
      const currentPosition = computeRelativePosition(
        e,
        canvasRef,
        mouseScale
      );
      if (currentPosition) {
        const offset = getOffset(currentPosition, startPosition);
        if (isMoving && considerMoved(offset)) {
          onItemMoved(e, offset);
        } else {
          let blocks = blockItems;
          if (panelPadding) {
            blocks = blocks.map(
              (block) => applyVizPadding({
                item: block,
                padding: panelPadding
              })
            );
          }
          const boundary = positionsToBoundary(
            startPosition,
            currentPosition
          );
          const selectedBlocks = filterBlockItemsByBoundary(
            blocks,
            boundary
          ).map(({ item, type = "block" }) => ({
            id: item,
            type
          }));
          const topItem = selectedBlocks[selectedBlocks.length - 1];
          handleItemSelected(e, topItem ? [topItem] : []);
        }
        resetState();
      }
    }
  });
  useMouseDownWithHandleEventListeners({
    handleMouseDownOnItem,
    canvasRef,
    scale: mouseScale,
    setStartPosition,
    movable,
    setIsMoving
  });
  (0, react_exports.useEffect)(() => {
    document.addEventListener("mouseup", handleMouseUp);
    return () => {
      document.removeEventListener("mouseup", handleMouseUp);
    };
  }, [handleMouseUp]);
  (0, dashboard_ui_exports.useMouseMoveHandler)({ onMouseMove: handleMouseMove, isEnabled: movable });
  return /* @__PURE__ */ react_default.createElement(
    Canvas_default,
    __spreadProps(__spreadValues({
      ref: canvasRef
    }, canvasProps), {
      scale,
      onMouseDown: handleMouseDown,
      onContextMenu: resetState,
      cssScaling: enableGridLayoutCssScaling,
      showGrid: false
    }),
    children
  );
};
var GridCanvas_default = (0, react_exports.forwardRef)(
  (props, ref) => /* @__PURE__ */ react_default.createElement(GridCanvas, __spreadValues({ canvasRef: ref }, props))
);

// src/components/GridItem.tsx
var GridDiv = styled_components_default.div`
    height: 100%;
    width: 100%;
    grid-row: ${(props) => props.id}-start / ${(props) => props.id}-end;
    grid-column: ${(props) => props.id}-start / ${(props) => props.id}-end;
    background: transparent;
`;
var GridItem = ({
  itemId,
  type = "block",
  renderLayoutItem
}) => /* @__PURE__ */ react_default.createElement(GridDiv, { id: itemId, "data-test": "grid-item" }, renderLayoutItem(itemId, { width: "100%", height: "100%" }, type));
GridItem.propTypes = {
  itemId: prop_types_default.string.isRequired,
  type: prop_types_default.string,
  renderLayoutItem: prop_types_default.func.isRequired
};
var GridItem_default = GridItem;

// src/components/ItemDragPlaceholder.tsx
var Placeholder = styled_components_default.div.attrs(
  ({ position }) => ({
    style: positionToStyle(position)
  })
)`
    background-color: ${(0, themes_exports.pick)({
  enterprise: {
    light: themes_exports.variables.backgroundColorHover,
    dark: themes_exports.variables.neutral300
  },
  prisma: themes_exports.variables.interactiveColorOverlayDrag
})};
    opacity: 0.75;
    user-select: none;
    position: absolute;
    flex-direction: column;
    top: 0;
    left: 0;
    cursor: move;
`;
var ItemDragPlaceholder = ({
  position,
  size = 128
}) => {
  const placeHolderPosition = (0, react_exports.useMemo)(() => {
    return {
      // update x and y to be the center of the placeholder
      x: position.x - Math.round(size / 2),
      y: position.y - Math.round(size / 2),
      w: size,
      h: size
    };
  }, [position, size]);
  return /* @__PURE__ */ react_default.createElement(
    Placeholder,
    {
      "data-test": "item-drag-placeholder",
      position: placeHolderPosition
    }
  );
};

// src/components/ItemDropTarget.jsx
var DropTarget = styled_components_default.div.attrs(({ position }) => ({
  style: positionToStyle(position)
}))`
    background-color: ${(0, themes_exports.pickVariant)("validDropTarget", {
  true: dashboard_ui_exports.customThemeVariables.validDropTargetBackgroundColor,
  false: dashboard_ui_exports.customThemeVariables.invalidDropTargetBackgroundColor
})};
    border-width: '1px';
    border-style: 'solid';
    border-color: ${(0, themes_exports.pickVariant)("validDropTarget", {
  true: dashboard_ui_exports.customThemeVariables.validDropTargetBorderColor,
  false: dashboard_ui_exports.customThemeVariables.invalidDropTargetBorderColor
})};
    user-select: none;
    position: absolute;
    pointer-events: none;
`;
var ItemDropTarget = ({ position, direction }) => {
  const validDropTarget = (direction === "e" || direction === "w") && position.w >= 128 || (direction === "n" || direction === "s") && position.h >= 128;
  const dropTargetPosition = (0, react_exports.useMemo)(() => {
    let updatedPosition = null;
    switch (direction) {
      case "n":
        updatedPosition = __spreadProps(__spreadValues({}, position), {
          h: ITEM_DROP_TARGET_PX
        });
        break;
      case "s":
        updatedPosition = __spreadProps(__spreadValues({}, position), {
          y: position.y + position.h - ITEM_DROP_TARGET_PX,
          h: ITEM_DROP_TARGET_PX
        });
        break;
      case "w":
        updatedPosition = __spreadProps(__spreadValues({}, position), {
          w: ITEM_DROP_TARGET_PX
        });
        break;
      case "e":
        updatedPosition = __spreadProps(__spreadValues({}, position), {
          x: position.x + position.w - ITEM_DROP_TARGET_PX,
          w: ITEM_DROP_TARGET_PX
        });
        break;
      default:
        updatedPosition = position;
        break;
    }
    return updatedPosition;
  }, [position, direction]);
  if (validDropTarget) {
    return null;
  }
  return /* @__PURE__ */ react_default.createElement(
    DropTarget,
    {
      "data-test": "item-drop-target",
      position: dropTargetPosition,
      validDropTarget
    }
  );
};
ItemDropTarget.propTypes = {
  position: prop_types_default.shape({
    x: prop_types_default.number.isRequired,
    y: prop_types_default.number.isRequired,
    w: prop_types_default.number.isRequired,
    h: prop_types_default.number.isRequired
  }).isRequired,
  direction: prop_types_default.oneOf(["n", "s", "w", "e"]).isRequired
};

// src/components/Layer.tsx
var Layer = styled_components_default.div`
    position: absolute;
    top: 0px;
    left: 0px;
    right: 0px;
    bottom: 0px;
    pointer-events: none;
    z-index: ${(props) => props.zIndex};
`;
var Layer_default = Layer;

// src/components/PreviewPlaceholderItem.jsx
var PlaceholderComponent = styled_components_default.div.attrs(({ position }) => ({
  style: {
    width: position.w,
    height: position.h,
    transform: `translate(${position.x}px, ${position.y}px)`
  }
}))`
    display: block;
    position: absolute;
    pointer-events: none;
    box-sizing: border-box;
    border: 1px solid ${dashboard_ui_exports.customThemeVariables.validDropTargetBorderColor};
    background-color: ${dashboard_ui_exports.customThemeVariables.validDropTargetBackgroundColor};
`;
var PreviewPlaceholderItem = ({ position }) => {
  return /* @__PURE__ */ react_default.createElement(
    PlaceholderComponent,
    {
      position,
      "data-test": "preview-placeholder",
      "data-test-preview-position": `${position.x},${position.y}-${position.w},${position.h}`
    }
  );
};
PreviewPlaceholderItem.propTypes = {
  position: prop_types_default.shape({
    x: prop_types_default.number.isRequired,
    y: prop_types_default.number.isRequired,
    w: prop_types_default.number.isRequired,
    h: prop_types_default.number.isRequired
  }).isRequired
};

// src/components/ResponsiveBox.tsx
var ResponsiveBox = styled_components_default.div.attrs(
  ({ itemId, x, y, w, h, "data-test": dataTest, zIndex }) => {
    const attributes = {
      "data-test": dataTest != null ? dataTest : "responsive-box",
      "data-id": itemId,
      "data-test-viz-item-position": `${x},${y},${w},${h}`,
      style: {
        width: `${w}px`,
        height: `${h}px`,
        top: `${y}px`,
        left: `${x}px`,
        zIndex
      }
    };
    if (!zIndex) {
      delete attributes.style.zIndex;
    }
    return attributes;
  }
)`
    display: block;
    position: absolute;
    background: transparent;
    pointer-events: none;
    box-sizing: border-box;
    visibility: ${(props) => {
  return props.appearance === "hidden" ? "hidden" : "visible";
}};
    opacity: ${(props) => props.appearance === "highlighted" ? 1 : 0.5};
`;
ResponsiveBox.defaultProps = {
  appearance: "highlighted"
};
var ResponsiveBox_default = ResponsiveBox;

// src/components/ErrorMessageOverlay.tsx
var ErrorOverlay = styled_components_default.div`
    position: absolute;
    box-sizing: border-box;
    top: 0;
    left: 0;
    height: 100%;
    width: 100%;
    pointer-events: none;
    background-color: ${(0, themes_exports.pick)({
  light: "#a81916cc",
  dark: "#a81916b3"
})};
    color: white;
    overflow: hidden;
    padding: 8px;
    z-index: 2;
`;
var MessageContainer = styled_components_default.div`
    padding-top: 8px;
`;
var OverlayTitle = styled_components_default.div`
    font-weight: bold;
`;
var ErrorMessageOverlay = ({ messages }) => {
  if (!Array.isArray(messages) || messages.length === 0) {
    return null;
  }
  return /* @__PURE__ */ react_default.createElement(ErrorOverlay, { "data-test": "error-overlay" }, /* @__PURE__ */ react_default.createElement(OverlayTitle, null, (0, i18n_exports._)("Layout Errors:")), messages.map((message) => /* @__PURE__ */ react_default.createElement(
    MessageContainer,
    {
      "data-test": "error-overlay-message",
      key: message
    },
    message
  )));
};

// src/components/ActionMenuPortal.ts
var ActionMenuPortal = styled_components_default.div.attrs((props) => ({
  "data-test": "action-menu-portal",
  style: {
    width: props.w,
    top: props.y + props.h,
    left: props.x
  }
}))`
    display: block;
    position: absolute;
    background: transparent;
    pointer-events: all;
    box-sizing: border-box;
    height: 0px;
    z-index: ${(props) => props.zIndex};
`;
var ActionMenuPortal_default = ActionMenuPortal;

// src/components/DragHandlePortal.tsx
var DragHandlePortal = styled_components_default.div.attrs(
  (props) => ({
    "data-test": "drag-handle-portal",
    style: {
      width: props.w,
      top: props.y,
      left: props.x
    }
  })
)`
    display: block;
    position: absolute;
    background: transparent;
    pointer-events: all;
    box-sizing: border-box;
    height: 0px;
    z-index: ${(props) => props.zIndex};
    visibility: ${({ isHidden }) => isHidden ? "hidden" : "visible"};
`;

// src/components/ResponsiveBlockItem.tsx
var defaultPortalState = Object.freeze({
  current: null
});
var ResponsiveBlockItem = ({
  x: originalX,
  y: originalY,
  w: originalW,
  h: originalH,
  canvasHeight,
  padding,
  itemId,
  type = "block",
  errorMessages,
  onItemSelected,
  renderLayoutItem,
  appearance = "highlighted",
  isMouseDownEventRef
}) => {
  var _a;
  const selectIsSelected = (0, react_exports.useMemo)(dashboard_state_exports.makeSelectIsSelected, []);
  const mode = (0, dashboard_state_exports.useSelector)(dashboard_state_exports.selectMode);
  const isSelected = (0, dashboard_state_exports.useSelector)((state) => selectIsSelected(state, itemId));
  const [isClicked, setIsClicked] = (0, react_exports.useState)(false);
  const [actionMenuPortal, setActionMenuPortal] = (0, react_exports.useState)(defaultPortalState);
  const [dragHandlePortal, setDragHandlePortal] = (0, react_exports.useState)(defaultPortalState);
  const layerData = (0, dashboard_context_exports.useLayoutLayers)();
  const {
    layer: itemLayer,
    actionMenu: actionMenuLayer,
    dragHandle: dragHandleLayer
  } = (_a = layerData == null ? void 0 : layerData[itemId]) != null ? _a : {};
  const renderActionMenuInPortal = typeof itemLayer === "number";
  const {
    position: { x, y, w, h }
  } = applyVizPadding({
    item: {
      position: {
        x: originalX,
        y: originalY,
        w: originalW,
        h: originalH
      }
    },
    padding
  });
  const layoutItem = (0, react_exports.useMemo)(
    () => renderLayoutItem(
      itemId,
      {
        width: w,
        height: h,
        y,
        canvasHeight,
        renderActionMenuInPortal,
        actionMenuPortal,
        dragHandlePortal
      },
      type,
      onItemSelected
    ),
    [
      renderLayoutItem,
      itemId,
      type,
      w,
      h,
      y,
      canvasHeight,
      onItemSelected,
      renderActionMenuInPortal,
      actionMenuPortal,
      dragHandlePortal
    ]
  );
  const handleFocus = (0, react_exports.useCallback)(
    (event) => {
      if (isMouseDownEventRef == null ? void 0 : isMouseDownEventRef.current) {
        setIsClicked(true);
        return;
      }
      if (!event.relatedTarget) {
        return;
      }
      setIsClicked(false);
      if (
        // The tabindex was moved to the SelectableContainer (in renderLayoutItem), so we'll check that thats the target
        event.target.getAttribute("data-test") === "select-outline" && event.target.getAttribute("data-id") === itemId
      ) {
        onItemSelected(event, [{ id: itemId, type }]);
      }
    },
    [isMouseDownEventRef, itemId, onItemSelected, type]
  );
  const trackActionMenuPortal = (0, react_exports.useCallback)((element) => {
    setActionMenuPortal({ current: element });
  }, []);
  const trackDragHandlePortal = (0, react_exports.useCallback)((element) => {
    setDragHandlePortal({ current: element });
  }, []);
  const isHidden = appearance === "hidden";
  return /* @__PURE__ */ react_default.createElement(
    dashboard_ui_exports.AbsoluteItemContainer,
    {
      "data-id": itemId,
      "data-selected": `${isSelected}`,
      "data-click-focus": `${isClicked && isSelected && mode === "view"}`
    },
    /* @__PURE__ */ react_default.createElement(
      ResponsiveBox_default,
      {
        "data-test": "absolute-item",
        itemId,
        appearance,
        x,
        y,
        w,
        h,
        onFocusCapture: handleFocus
      },
      layoutItem,
      /* @__PURE__ */ react_default.createElement(ErrorMessageOverlay, { messages: errorMessages })
    ),
    renderActionMenuInPortal && /* @__PURE__ */ react_default.createElement(
      ActionMenuPortal_default,
      {
        x,
        y,
        w,
        h,
        ref: trackActionMenuPortal,
        zIndex: actionMenuLayer
      }
    ),
    /* @__PURE__ */ react_default.createElement(
      DragHandlePortal,
      {
        x,
        y,
        w,
        ref: trackDragHandlePortal,
        zIndex: dragHandleLayer,
        isHidden
      }
    )
  );
};
var ResponsiveBlockItem_default = react_default.memo(ResponsiveBlockItem);

// esm-externals:@splunk/react-icons/enterprise/Plus
var Plus_exports = {};
__export(Plus_exports, {
  default: () => Plus_default
});
__reExport(Plus_exports, Plus_star);
import * as defaultImport6 from "@splunk/react-icons/enterprise/Plus";
import * as Plus_star from "@splunk/react-icons/enterprise/Plus";
var Plus_default = "default" in defaultImport6 ? defaultImport6.default : defaultImport6;

// src/utils/outlineUtils.ts
var getVerticalPosition = (dir, offset) => {
  if (dir[0] === "n") {
    return `top: -${offset}px`;
  }
  if (dir[0] === "s") {
    return `bottom: -${offset}px`;
  }
  return `top: calc(50% - ${offset}px)`;
};
var getHorizontalPosition = (dir, offset) => {
  if (dir.slice(-1) === "w") {
    return `left: -${offset}px`;
  }
  if (dir.slice(-1) === "e") {
    return `right: -${offset}px`;
  }
  return `left: calc(50% - ${offset}px)`;
};

// src/components/Port.tsx
var portOffset = 8;
var PORT_DIRECTIONS = ["n", "e", "s", "w"];
var PortContainer = styled_components_default.div.attrs(({ port }) => ({
  "data-test": "port",
  "data-test-port": port
}))`
    text-align: center;
    width: 16px;
    height: 16px;
    line-height: 16px;
    box-sizing: border-box;
    border-radius: 50%;
    position: absolute;
    color: ${(0, themes_exports.pick)({
  enterprise: themes_exports.variables.accentColorL10,
  prisma: themes_exports.variables.interactiveColorPrimary
})};
    ${({ port }) => getVerticalPosition(port, portOffset)};
    ${({ port }) => getHorizontalPosition(port, portOffset)};
    z-index: 999;
    pointer-events: auto;
    user-select: none;
    &:hover {
        background-color: ${(0, themes_exports.pick)({
  enterprise: themes_exports.variables.accentColorL10,
  prisma: themes_exports.variables.interactiveColorPrimary
})};
    }
`;
var Port = ({
  port,
  onMouseEnter,
  onMouseLeave
}) => /* @__PURE__ */ react_default.createElement(
  PortContainer,
  {
    port,
    onMouseEnter,
    onMouseLeave
  },
  /* @__PURE__ */ react_default.createElement(Plus_default, null)
);

// esm-externals:@splunk/ui-utils/format
var format_exports = {};
__export(format_exports, {
  default: () => format_default
});
__reExport(format_exports, format_star);
import * as defaultImport7 from "@splunk/ui-utils/format";
import * as format_star from "@splunk/ui-utils/format";
var format_default = "default" in defaultImport7 ? defaultImport7.default : defaultImport7;

// src/components/ResizeHandle.tsx
var resizeHandleOffset = 7;
var computeCursor = (dir) => {
  switch (dir) {
    case "n":
    case "s":
      return "ns-resize";
    case "e":
    case "w":
      return "ew-resize";
    case "ne":
    case "sw":
      return "nesw-resize";
    case "se":
    case "nw":
    default:
      return "nwse-resize";
  }
};
var computeDirection = (dir) => {
  switch (dir) {
    case "n":
      return (0, i18n_exports._)("North");
    case "s":
      return (0, i18n_exports._)("South");
    case "e":
      return (0, i18n_exports._)("East");
    case "w":
      return (0, i18n_exports._)("West");
    case "ne":
      return (0, i18n_exports._)("North-East");
    case "sw":
      return (0, i18n_exports._)("South-West");
    case "se":
      return (0, i18n_exports._)("South-East");
    case "nw":
      return (0, i18n_exports._)("North-West");
    default:
      return "";
  }
};
var Handle = styled_components_default.button.attrs(({ direction }) => ({
  "data-test": "handle",
  "data-test-direction": direction
}))`
    width: 10px;
    height: 10px;
    background-color: ${(0, themes_exports.pick)({
  enterprise: themes_exports.variables.accentColorL10,
  prisma: themes_exports.variables.interactiveColorPrimary
})};
    border: none;
    border-radius: 20%;
    padding: 0;
    position: absolute;
    ${({ direction }) => getVerticalPosition(direction, resizeHandleOffset)};
    ${({ direction }) => getHorizontalPosition(direction, resizeHandleOffset)};
    cursor: ${({ direction }) => computeCursor(direction)};
    z-index: 999;
    pointer-events: auto;
    user-select: none;
`;
var ResizeHandle = ({
  onMouseDown,
  onKeyDown,
  direction
}) => {
  const handleMouseDown = (0, react_exports.useCallback)(
    (e) => {
      onMouseDown(e, direction);
    },
    [onMouseDown, direction]
  );
  const handleKeyDown = (0, react_exports.useCallback)(
    (e) => {
      onKeyDown(e, direction);
    },
    [onKeyDown, direction]
  );
  return /* @__PURE__ */ react_default.createElement(
    Handle,
    {
      "aria-label": (0, format_exports.sprintf)((0, i18n_exports._)("Move %(direction)s direction"), {
        direction: computeDirection(direction)
      }),
      direction,
      onMouseDown: handleMouseDown,
      onKeyDown: handleKeyDown
    }
  );
};

// src/components/ResponsiveBlockOutline.tsx
var keyDirectionMap = {
  ArrowUp: ["n", "ne", "se", "s", "sw", "nw"],
  ArrowDown: ["n", "ne", "se", "s", "sw", "nw"],
  ArrowLeft: ["ne", "e", "se", "sw", "w", "nw"],
  ArrowRight: ["ne", "e", "se", "sw", "w", "nw"]
};
var isKeyAllowedCheck = (key) => Object.keys(keyDirectionMap).includes(key);
var noop4 = () => void 0;
var ResponsiveBlockOutline = ({
  itemId,
  scale = 1,
  connectable = false,
  resizable = false,
  onResize = noop4,
  onResized = noop4,
  onLineConnect = noop4,
  onLineDisconnect = noop4,
  handleDirections,
  x,
  y,
  w,
  h
}) => {
  const startPosition = (0, react_exports.useRef)(
    null
  );
  const resizing = (0, react_exports.useRef)(false);
  const resizeDir = (0, react_exports.useRef)(null);
  const keyUpOffset = (0, react_exports.useRef)({ offsetX: 0, offsetY: 0 });
  const handleResizeMouseDown = (0, react_exports.useCallback)(
    (e, dir) => {
      e.preventDefault();
      e.stopPropagation();
      startPosition.current = getClientPosition(e, scale);
      resizing.current = true;
      resizeDir.current = dir;
    },
    [scale]
  );
  const handleMouseMove = (0, react_exports.useCallback)(
    (e) => {
      if (startPosition.current && resizing.current && resizeDir.current) {
        e.preventDefault();
        const currentPosition = getClientPosition(e, scale);
        const offset = getOffset(
          currentPosition,
          startPosition.current
        );
        onResize(e, itemId, offset, resizeDir.current);
      }
    },
    [scale, onResize, itemId]
  );
  const handleMouseUp = (0, react_exports.useCallback)(
    (e) => {
      if (startPosition.current && resizing.current && resizeDir.current) {
        e.preventDefault();
        const currentPosition = getClientPosition(e, scale);
        const offset = getOffset(
          currentPosition,
          startPosition.current
        );
        startPosition.current = null;
        resizing.current = false;
        onResized(e, itemId, offset, resizeDir.current);
        resizeDir.current = null;
      }
    },
    [itemId, scale, onResized]
  );
  const handleResizeKeyDown = (0, react_exports.useCallback)(
    (e, dir) => {
      const { key } = e;
      const isKeyAllowed = isKeyAllowedCheck(key);
      const isDirectionAllowed = isKeyAllowed && keyDirectionMap[key].includes(dir);
      if (!isDirectionAllowed) {
        return;
      }
      e.preventDefault();
      e.stopPropagation();
      resizeDir.current = dir;
      if (e.key === "ArrowUp") {
        keyUpOffset.current.offsetY -= GRID_SIZE;
      }
      if (e.key === "ArrowDown") {
        keyUpOffset.current.offsetY += GRID_SIZE;
      }
      if (e.key === "ArrowLeft") {
        keyUpOffset.current.offsetX -= GRID_SIZE;
      }
      if (e.key === "ArrowRight") {
        keyUpOffset.current.offsetX += GRID_SIZE;
      }
      onResize(e, itemId, keyUpOffset.current, dir);
    },
    [itemId, onResize]
  );
  const handleKeyUp = (0, react_exports.useCallback)(
    (e) => {
      if (keyUpOffset.current && resizeDir.current) {
        onResized(e, itemId, keyUpOffset.current, resizeDir.current);
        resizeDir.current = null;
        keyUpOffset.current = { offsetX: 0, offsetY: 0 };
      }
    },
    [itemId, onResized]
  );
  const handlePortEnter = (0, react_exports.useCallback)(
    (port) => (e) => {
      e.preventDefault();
      onLineConnect(itemId, port);
    },
    [itemId, onLineConnect]
  );
  const handlePortLeave = (0, react_exports.useCallback)(
    (port) => (e) => {
      e.preventDefault();
      onLineDisconnect(itemId, port);
    },
    [itemId, onLineDisconnect]
  );
  (0, react_exports.useLayoutEffect)(() => {
    if (resizable) {
      document.addEventListener("mousemove", handleMouseMove);
    }
    return () => {
      document.removeEventListener("mousemove", handleMouseMove);
    };
  }, [resizable, handleMouseMove]);
  (0, react_exports.useLayoutEffect)(() => {
    if (resizable) {
      document.addEventListener("mouseup", handleMouseUp);
    }
    return () => {
      document.removeEventListener("mouseup", handleMouseUp);
    };
  }, [resizable, handleMouseUp]);
  (0, react_exports.useLayoutEffect)(() => {
    if (resizable) {
      document.addEventListener("keyup", handleKeyUp);
    }
    return () => {
      document.removeEventListener("keyup", handleKeyUp);
    };
  }, [resizable, handleKeyUp]);
  const ResizeHandles = (0, react_exports.useMemo)(() => {
    if (!resizable) {
      return null;
    }
    return handleDirections.map((dir) => /* @__PURE__ */ react_default.createElement(
      ResizeHandle,
      {
        key: `handle-${dir}`,
        onMouseDown: handleResizeMouseDown,
        onKeyDown: handleResizeKeyDown,
        direction: dir
      }
    ));
  }, [
    resizable,
    handleDirections,
    handleResizeMouseDown,
    handleResizeKeyDown
  ]);
  const Ports = (0, react_exports.useMemo)(() => {
    if (!connectable) {
      return null;
    }
    return PORT_DIRECTIONS.map((port) => /* @__PURE__ */ react_default.createElement(
      Port,
      {
        key: `port-${port}`,
        port,
        onMouseEnter: handlePortEnter(port),
        onMouseLeave: handlePortLeave(port)
      }
    ));
  }, [connectable, handlePortEnter, handlePortLeave]);
  return /* @__PURE__ */ react_default.createElement(ResponsiveBox_default, { itemId, x, y, w, h }, ResizeHandles, Ports);
};
var ResponsiveBlockOutline_default = ResponsiveBlockOutline;

// src/components/ResponsiveLine.tsx
var allowedKeys = ["ArrowDown", "ArrowUp", "ArrowLeft", "ArrowRight"];
var LineBox = styled_components_default.div`
    position: absolute;
    pointer-events: none;
    z-index: ${(props) => props.zIndex};
`;
var AdjustHandle = styled_components_default.button.attrs(({ x, y }) => ({
  style: {
    transform: `translate(${x}px, ${y}px)`
  }
}))`
    width: 10px;
    height: 10px;
    left: -5px;
    top: -5px;
    background-color: ${(0, themes_exports.pick)({
  enterprise: themes_exports.variables.accentColorL10,
  prisma: themes_exports.variables.interactiveColorPrimary
})};
    border: none;
    padding: 0;
    border-radius: 50%;
    position: absolute;
    z-index: 999;
    pointer-events: auto;
    cursor: move;
    user-select: none;
`;
var ResponsiveLine = ({
  fromX,
  fromY,
  fromItem,
  fromPort,
  toX,
  toY,
  toItem,
  toPort,
  lineId,
  scale = 1,
  selectable = false,
  editable = false,
  renderLayoutItem,
  onItemSelected = dashboard_utils_exports.noop,
  onLineDragStart = dashboard_utils_exports.noop,
  onLineMove = dashboard_utils_exports.noop,
  onLineMoved = dashboard_utils_exports.noop,
  onLineDrag = dashboard_utils_exports.noop,
  onLineDragged = dashboard_utils_exports.noop
}) => {
  const [startPosition, setStartPosition] = (0, react_exports.useState)(null);
  const [action, setAction] = (0, react_exports.useState)(null);
  const keyUpOffset = (0, react_exports.useRef)({ offsetX: 0, offsetY: 0 });
  const layerData = (0, dashboard_context_exports.useLayoutLayers)();
  const lineLayer = (0, react_exports.useMemo)(() => {
    const selectionLayer = Object.values(layerData != null ? layerData : {}).map((entry) => entry.selection).filter(Boolean)[0];
    return typeof selectionLayer === "number" ? selectionLayer + 1 : void 0;
  }, [layerData]);
  const handleLineSelected = (0, react_exports.useCallback)(
    (e) => {
      e.stopPropagation();
      setStartPosition(getClientPosition(e, scale));
      setAction("move");
      onItemSelected(e, [{ id: lineId, type: "line" }]);
    },
    [lineId, onItemSelected, scale]
  );
  const handleLineSelectedOnEnter = (0, react_exports.useCallback)(
    (e) => {
      if (e.key === "Enter") {
        onItemSelected(e, [{ id: lineId, type: "line" }]);
      }
    },
    [lineId, onItemSelected]
  );
  const handleKeyDown = (0, react_exports.useCallback)(
    (e) => {
      e.preventDefault();
      if (e.key === "ArrowUp") {
        keyUpOffset.current.offsetY -= 1;
      }
      if (e.key === "ArrowDown") {
        keyUpOffset.current.offsetY += 1;
      }
      if (e.key === "ArrowLeft") {
        keyUpOffset.current.offsetX -= 1;
      }
      if (e.key === "ArrowRight") {
        keyUpOffset.current.offsetX += 1;
      }
      onLineDrag(e, keyUpOffset.current);
    },
    [onLineDrag]
  );
  const handleKeyDownFrom = (0, react_exports.useCallback)(
    (e) => {
      if (!allowedKeys.includes(e.key)) {
        return;
      }
      if (keyUpOffset.current.offsetX === 0 && keyUpOffset.current.offsetY === 0) {
        onLineDragStart(e, "from");
      }
      handleKeyDown(e);
    },
    [onLineDragStart, handleKeyDown]
  );
  const handleKeyDownTo = (0, react_exports.useCallback)(
    (e) => {
      if (!allowedKeys.includes(e.key)) {
        return;
      }
      if (keyUpOffset.current.offsetX === 0 && keyUpOffset.current.offsetY === 0) {
        onLineDragStart(e, "to");
      }
      handleKeyDown(e);
    },
    [onLineDragStart, handleKeyDown]
  );
  const handleKeyUp = (0, react_exports.useCallback)(
    (e) => {
      var _a, _b, _c;
      const isLineHandleFocused = ((_a = document.activeElement) == null ? void 0 : _a.tagName) === "BUTTON" && ((_c = (_b = document.activeElement) == null ? void 0 : _b.getAttribute("data-test")) == null ? void 0 : _c.startsWith("line-handle"));
      if (!allowedKeys.includes(e.key) || keyUpOffset.current.offsetX === 0 && keyUpOffset.current.offsetY === 0 || !isLineHandleFocused) {
        return;
      }
      onLineDragged(e, keyUpOffset.current);
      keyUpOffset.current = { offsetX: 0, offsetY: 0 };
    },
    [onLineDragged]
  );
  const handleMouseDownFrom = (0, react_exports.useCallback)(
    (e) => {
      e.stopPropagation();
      setStartPosition(getClientPosition(e, scale));
      setAction("drag");
      onLineDragStart(e, "from");
    },
    [onLineDragStart, scale]
  );
  const handleMouseDownTo = (0, react_exports.useCallback)(
    (e) => {
      e.stopPropagation();
      setStartPosition(getClientPosition(e, scale));
      setAction("drag");
      onLineDragStart(e, "to");
    },
    [onLineDragStart, scale]
  );
  const handleMouseUp = (0, react_exports.useCallback)(
    (e) => {
      if (startPosition == null || action == null) {
        return;
      }
      setStartPosition(null);
      const currentPosition = getClientPosition(e, scale);
      const offset = getOffset(currentPosition, startPosition);
      if (!offset.offsetX && !offset.offsetY) {
        return;
      }
      switch (action) {
        case "drag":
          onLineDragged(e, offset);
          break;
        case "move":
          onLineMoved(e, offset);
          break;
        default:
          break;
      }
    },
    [action, onLineDragged, onLineMoved, scale, startPosition]
  );
  const handleMouseMove = (0, react_exports.useCallback)(
    (e) => {
      if (startPosition == null || action == null) {
        return;
      }
      if (e.buttons <= 0 || e.buttons % 2 === 0) {
        handleMouseUp(e);
        return;
      }
      e.preventDefault();
      const currentPosition = getClientPosition(e, scale);
      const offset = getOffset(currentPosition, startPosition);
      if (!offset.offsetX && !offset.offsetY) {
        return;
      }
      switch (action) {
        case "drag":
          onLineDrag(e, offset);
          break;
        case "move":
          onLineMove(e, offset);
          break;
        default:
          break;
      }
    },
    [action, handleMouseUp, onLineDrag, onLineMove, scale, startPosition]
  );
  const from = { x: fromX, y: fromY };
  const to = { x: toX, y: toY };
  const boxPos = computeLineBoxPosition(from, to);
  const relativePos = computeLineRelativePosition(from, to, boxPos);
  const lineBoxStyle = (0, react_exports.useMemo)(
    () => ({
      transform: `translate(${boxPos.x}px, ${boxPos.y}px)`
    }),
    [boxPos.x, boxPos.y]
  );
  const fromHandleElement = (0, react_exports.useMemo)(() => {
    if (!editable) {
      return null;
    }
    return /* @__PURE__ */ react_default.createElement(
      AdjustHandle,
      {
        "data-test": "line-handle-from",
        "data-test-item": fromItem,
        "data-test-port": fromPort,
        x: relativePos.from.x,
        y: relativePos.from.y,
        onMouseDown: handleMouseDownFrom,
        onKeyDown: handleKeyDownFrom,
        "aria-label": (0, i18n_exports._)("From line handle")
      }
    );
  }, [
    editable,
    fromItem,
    fromPort,
    relativePos.from.x,
    relativePos.from.y,
    handleMouseDownFrom,
    handleKeyDownFrom
  ]);
  const toHandleElement = (0, react_exports.useMemo)(() => {
    if (!editable) {
      return null;
    }
    return /* @__PURE__ */ react_default.createElement(
      AdjustHandle,
      {
        "data-test": "line-handle-to",
        "data-test-item": toItem,
        "data-test-port": toPort,
        x: relativePos.to.x,
        y: relativePos.to.y,
        onMouseDown: handleMouseDownTo,
        onKeyDown: handleKeyDownTo,
        "aria-label": (0, i18n_exports._)("To line handle")
      }
    );
  }, [
    editable,
    toItem,
    toPort,
    relativePos.to.x,
    relativePos.to.y,
    handleMouseDownTo,
    handleKeyDownTo
  ]);
  const lineElement = (0, react_exports.useMemo)(
    () => renderLayoutItem(
      lineId,
      {
        from: {
          x: relativePos.from.x,
          y: relativePos.from.y
        },
        to: {
          x: relativePos.to.x,
          y: relativePos.to.y
        },
        // todo: this is inconsistent with the ResponsiveBlockItem which accepts the `onSelect` callback as the 4th argument of `renderLayoutItem`.
        onLineSelect: selectable ? handleLineSelected : dashboard_utils_exports.noop
      },
      "line"
    ),
    [
      handleLineSelected,
      lineId,
      relativePos.from.x,
      relativePos.from.y,
      relativePos.to.x,
      relativePos.to.y,
      renderLayoutItem,
      selectable
    ]
  );
  (0, react_exports.useEffect)(() => {
    if (editable) {
      document.addEventListener("mousemove", handleMouseMove);
      document.addEventListener("mouseup", handleMouseUp);
    }
    return () => {
      document.removeEventListener("mousemove", handleMouseMove);
      document.removeEventListener("mouseup", handleMouseUp);
    };
  }, [editable, handleMouseMove, handleMouseUp]);
  (0, react_exports.useEffect)(() => {
    if (editable) {
      document.addEventListener("keyup", handleKeyUp);
    }
    return () => {
      document.removeEventListener("keyup", handleKeyUp);
    };
  }, [editable, handleKeyUp]);
  return /* @__PURE__ */ react_default.createElement(
    LineBox,
    {
      "data-test": "line-box",
      tabIndex: !editable ? 0 : void 0,
      onKeyDown: handleLineSelectedOnEnter,
      style: lineBoxStyle,
      "data-viz-type": "abslayout.line",
      "data-id": lineId,
      "data-test-line-position": `${from.x},${from.y}-${to.x},${to.y}`,
      zIndex: lineLayer
    },
    fromHandleElement,
    toHandleElement,
    lineElement
  );
};
var ResponsiveLine_default = ResponsiveLine;

export {
  lodash_exports,
  dashboard_telemetry_exports,
  getNodes,
  computeEdges,
  applyVizPadding,
  getVerticalBoundaries,
  getHorizontalBoundaries,
  moveHorizontalEdge,
  moveVerticalEdge,
  findSnappableEdges,
  findEdgesInBoundary,
  computeMaxHeight,
  removeInvalidItems,
  getClientPosition,
  getOffset,
  findTopBlockItemByPosition,
  snapOffsetToXY,
  snapOffsetToWH,
  updateBlockItemSize,
  updateBlockItemPosition,
  createOffset,
  computeScaleToFit,
  moveLayoutItem,
  isLineConnected,
  disconnectLine,
  connectLine,
  updateLineAbsPosition,
  cloneBlockItem,
  cloneLine,
  shiftViewportOnZoom,
  scaleGridLayoutStructureByWidth,
  isFromImageRegistry,
  validateBackgroundImage,
  useBackgroundImage,
  useLayoutShowHide,
  useZoom,
  Canvas_default,
  AbsoluteCanvas_default,
  AbsoluteItem_default,
  Edge,
  GridCanvas_default,
  GridItem_default,
  ItemDragPlaceholder,
  ItemDropTarget,
  Layer_default,
  PreviewPlaceholderItem,
  ResponsiveBlockItem_default,
  ResponsiveBlockOutline_default,
  ResponsiveLine_default
};
//# sourceMappingURL=chunk-NJ3ZVGVA.js.map
