/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  computeLineAbsPosition
} from "./chunk-X4NAUHLE.js";
import {
  AbsoluteItem_default,
  Canvas_default,
  ResponsiveLine_default,
  computeScaleToFit,
  validateBackgroundImage
} from "./chunk-NJ3ZVGVA.js";
import {
  react_default,
  react_exports
} from "./chunk-D2UIVRV7.js";
import {
  absoluteLayoutOptions
} from "./chunk-5YVOBQNO.js";
import {
  dashboard_utils_exports
} from "./chunk-QCW3O7QD.js";
import {
  __spreadValues
} from "./chunk-W3Q7A2F6.js";

// src/AbsoluteLayoutViewer.tsx
var AbsoluteLayoutViewer = ({
  containerWidth,
  containerHeight,
  layoutStructure = [],
  renderLayoutItem,
  options = {}
}) => {
  const {
    width,
    height,
    backgroundColor,
    backgroundImage = {},
    display
  } = __spreadValues(__spreadValues({}, absoluteLayoutOptions), options);
  const scrollbarWidth = (0, react_exports.useMemo)(() => (0, dashboard_utils_exports.getScrollbarWidth)(), []);
  const scale = (0, react_exports.useMemo)(() => {
    switch (display) {
      case "fit-to-width":
      case "auto-scale":
        return computeScaleToFit({
          canvasWidth: width,
          canvasHeight: height,
          containerWidth,
          containerHeight,
          scrollbarWidth
        });
      case "actual-size":
        return 1;
      default:
        return computeScaleToFit({
          canvasWidth: width,
          canvasHeight: height,
          containerWidth,
          containerHeight,
          scrollbarWidth,
          max: 1
        });
    }
  }, [
    display,
    height,
    width,
    containerWidth,
    containerHeight,
    scrollbarWidth
  ]);
  const image = (0, react_exports.useMemo)(
    () => backgroundImage ? validateBackgroundImage({
      backgroundImage,
      canvasWidth: width,
      canvasHeight: height
    }) : {},
    [backgroundImage, height, width]
  );
  const renderLines = (0, react_exports.useMemo)(() => {
    const lineItems = layoutStructure.filter(
      ({ type }) => type === "line"
    );
    return lineItems.map(({ item, position }) => {
      const absPosition = computeLineAbsPosition({
        layoutStructure,
        position
      });
      return /* @__PURE__ */ react_default.createElement(
        ResponsiveLine_default,
        {
          key: item,
          lineId: item,
          scale,
          fromX: absPosition.from.x,
          toX: absPosition.to.x,
          fromY: absPosition.from.y,
          toY: absPosition.to.y,
          renderLayoutItem
        }
      );
    });
  }, [layoutStructure, scale, renderLayoutItem]);
  const renderBlockItems = (0, react_exports.useMemo)(() => {
    const blockItems = layoutStructure.filter(
      ({ type }) => type !== "line"
    );
    return blockItems.map(({ item, type, position }) => {
      const { x, y, w, h } = position;
      return /* @__PURE__ */ react_default.createElement(
        AbsoluteItem_default,
        {
          itemId: item,
          type,
          key: item,
          x,
          y,
          w,
          h,
          renderLayoutItem
        }
      );
    });
  }, [renderLayoutItem, layoutStructure]);
  return /* @__PURE__ */ react_default.createElement(
    Canvas_default,
    {
      "data-layout-type": "absolute",
      width,
      height,
      scale,
      showGrid: false,
      backgroundColor,
      backgroundImageSrc: image.src,
      backgroundImageSizeType: image.sizeType,
      backgroundImageWidth: image.w,
      backgroundImageHeight: image.h,
      backgroundImagePositionX: image.x,
      backgroundImagePositionY: image.y,
      backgroundImageOpacity: image.opacity
    },
    renderBlockItems,
    renderLines
  );
};
var AbsoluteLayoutViewer_default = AbsoluteLayoutViewer;

export {
  AbsoluteLayoutViewer_default
};
//# sourceMappingURL=chunk-RHSS7LVQ.js.map
