/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  i18n_exports
} from "./chunk-PLOB2GBC.js";
import {
  __export,
  __reExport,
  __spreadProps,
  __spreadValues
} from "./chunk-W3Q7A2F6.js";

// esm-externals:@splunk/visualization-color-palettes/editors/PresetPalettes
var PresetPalettes_exports = {};
__export(PresetPalettes_exports, {
  default: () => PresetPalettes_default
});
__reExport(PresetPalettes_exports, PresetPalettes_star);
import * as defaultImport from "@splunk/visualization-color-palettes/editors/PresetPalettes";
import * as PresetPalettes_star from "@splunk/visualization-color-palettes/editors/PresetPalettes";
var PresetPalettes_default = "default" in defaultImport ? defaultImport.default : defaultImport;

// esm-externals:@splunk/visualizations/common/utils/configUtils
var configUtils_exports = {};
__export(configUtils_exports, {
  default: () => configUtils_default
});
__reExport(configUtils_exports, configUtils_star);
import * as defaultImport2 from "@splunk/visualizations/common/utils/configUtils";
import * as configUtils_star from "@splunk/visualizations/common/utils/configUtils";
var configUtils_default = "default" in defaultImport2 ? defaultImport2.default : defaultImport2;

// src/visualizations/ConnectedLineEditorConfig.ts
var defaultContext = {
  defaultRangeValue: [
    { to: 20, value: "#D41F1F" },
    { from: 20, to: 40, value: "#D94E17" },
    { from: 40, to: 60, value: "#CBA700" },
    { from: 60, to: 80, value: "#669922" },
    { from: 80, value: "#118832" }
  ]
};
var basePresetContext = {
  strokeColorEditorConfig: void 0
};
var basePresetOptions = {
  strokeColor: void 0
};
var presets = [
  {
    label: (0, i18n_exports._)("Static"),
    name: "lineColor.static",
    value: {
      context: __spreadValues({}, basePresetContext),
      options: __spreadValues({}, basePresetOptions)
    }
  },
  {
    label: (0, i18n_exports._)("Dynamic"),
    name: "lineColor.dynamic",
    value: {
      context: __spreadProps(__spreadValues({}, basePresetContext), {
        strokeColorEditorConfig: defaultContext.defaultRangeValue
      }),
      options: __spreadProps(__spreadValues({}, basePresetOptions), {
        strokeColor: "> strokeDataPoint | rangeValue(strokeColorEditorConfig)"
      })
    }
  }
];
var StaticColorEditor = {
  label: (0, i18n_exports._)("Color value"),
  option: "strokeColor",
  editor: "editor.color",
  showEditor: ({ options }) => !(0, configUtils_exports.isDynamicOption)(options == null ? void 0 : options.strokeColor),
  editorProps: {
    labelPosition: "top",
    themeVariable: "@splunk/dashboard-layouts/ConnectedLine.strokeColor"
  }
};
var DynamicColorEditor = {
  label: (0, i18n_exports._)("Color range"),
  option: "strokeColor",
  context: "defaultContext",
  editor: "editor.dynamicColor",
  showEditor: ({ options }) => (0, configUtils_exports.isDynamicOption)(options == null ? void 0 : options.strokeColor),
  editorProps: {
    labelPosition: "top",
    flyoutTitle: (0, i18n_exports._)("Dynamic line color"),
    formatters: [
      {
        label: (0, i18n_exports._)("Ranges"),
        value: "rangeValue",
        isDefault: true,
        // show it by default when open dynamic color flyout
        defaults: {
          strokeDataPoint: defaultContext.defaultRangeValue
        }
      }
    ],
    dataSelectors: [
      {
        label: (0, i18n_exports._)("Color field"),
        value: "strokeDataPoint"
      }
    ],
    defaultPalettesConfig: PresetPalettes_exports.defaultPalettesConfig
  }
};
var editorConfig = [
  {
    label: (0, i18n_exports._)("Selected data field"),
    layout: [
      [
        {
          label: (0, i18n_exports._)("Data field"),
          editor: "editor.columnSelector",
          option: "strokeDataSeries",
          context: "strokeDataContext",
          editorProps: {
            dataSourceKey: "primary",
            prepareValue: (definition) => {
              const {
                options: { strokeDataSeries }
              } = definition;
              return {
                context: {},
                options: {
                  strokeDataPoint: `> strokeDataSeries | lastPoint()`,
                  strokeDataSeries
                }
              };
            }
          }
        }
      ]
    ]
  },
  {
    label: (0, i18n_exports._)("Color and style"),
    layout: [
      [
        {
          label: (0, i18n_exports._)("Mode"),
          key: "presetSelector",
          editor: "editor.presetSelector",
          editorProps: {
            presets,
            value: ({
              context,
              options
            }) => {
              const initialPreset = (0, configUtils_exports.getInitialPreset)(
                context,
                options,
                presets
              );
              return (initialPreset == null ? void 0 : initialPreset.label) || null;
            }
          }
        }
      ],
      [
        __spreadValues({}, StaticColorEditor)
      ],
      [
        __spreadValues({}, DynamicColorEditor)
      ],
      [
        {
          label: (0, i18n_exports._)("Stroke opacity"),
          option: "strokeOpacity",
          editor: "editor.percent"
        }
      ],
      [
        {
          label: (0, i18n_exports._)("Stroke thickness (px)"),
          option: "strokeWidth",
          editor: "editor.number",
          editorProps: {
            min: 1
          }
        }
      ],
      [
        {
          label: (0, i18n_exports._)("Stroke dash style"),
          option: "strokeDasharray",
          editor: "editor.number",
          editorProps: {
            min: 0
          }
        }
      ],
      [
        {
          label: (0, i18n_exports._)("Starting arrow point"),
          option: "fromArrow",
          editor: "editor.toggle",
          editorProps: {
            labelPosition: "right"
          }
        }
      ],
      [
        {
          label: (0, i18n_exports._)("Ending arrow point"),
          option: "toArrow",
          editor: "editor.toggle",
          editorProps: {
            labelPosition: "right"
          }
        }
      ]
    ]
  }
];
var ConnectedLineEditorConfig_default = editorConfig;

export {
  presets,
  ConnectedLineEditorConfig_default
};
//# sourceMappingURL=chunk-SQHLMK4Z.js.map
