/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  absoluteLayoutOptionsSchema_default
} from "./chunk-2VLZOQVK.js";
import {
  AbsoluteLayoutApi_default,
  withLayoutShowHide
} from "./chunk-NRG3QARU.js";
import {
  dashboard_definition_exports
} from "./chunk-UUGRNOSW.js";
import {
  InvalidBlockItemError,
  computeLineAbsPosition,
  findSelectedBlockItems,
  findSelectedLineItems,
  getAllBlockItems,
  handleSingleLineDragStart,
  handleSingleLineMove
} from "./chunk-X4NAUHLE.js";
import {
  AbsoluteCanvas_default,
  Layer_default,
  ResponsiveBlockItem_default,
  ResponsiveBlockOutline_default,
  ResponsiveLine_default,
  connectLine,
  createOffset,
  dashboard_telemetry_exports,
  disconnectLine,
  isLineConnected,
  lodash_exports,
  shiftViewportOnZoom,
  snapOffsetToWH,
  snapOffsetToXY,
  updateBlockItemPosition,
  updateBlockItemSize,
  useBackgroundImage,
  useZoom,
  validateBackgroundImage
} from "./chunk-NJ3ZVGVA.js";
import {
  dashboard_context_exports
} from "./chunk-2OPMAP46.js";
import {
  dashboard_ui_exports
} from "./chunk-V6LCPHRJ.js";
import {
  GRID_PADDING,
  GRID_SIZE,
  MIN_HEIGHT,
  MIN_WIDTH
} from "./chunk-WSRZLQBK.js";
import {
  react_default,
  react_exports
} from "./chunk-D2UIVRV7.js";
import {
  absoluteLayoutOptions
} from "./chunk-5YVOBQNO.js";
import {
  dashboard_utils_exports
} from "./chunk-QCW3O7QD.js";
import {
  __spreadProps,
  __spreadValues
} from "./chunk-W3Q7A2F6.js";

// src/utils/absoluteLayoutUtils.ts
var initializeLayoutStructureState = (layoutStructure) => layoutStructure.reduce((obj, item) => {
  obj[item.item] = __spreadValues({}, item);
  return obj;
}, {});
var reducer = (state, action) => {
  switch (action.type) {
    case "lineMove":
      return __spreadProps(__spreadValues({}, state), {
        [action.payload.item]: action.payload
      });
    case "lineDragStart":
      return __spreadProps(__spreadValues({}, state), {
        [action.payload.item]: action.payload
      });
    case "lineDrag": {
      const lineToDrag = state[action.payload.id];
      return isLineConnected({
        line: lineToDrag,
        dir: action.payload.dir
      }) ? state : __spreadProps(__spreadValues({}, state), {
        [action.payload.id]: __spreadProps(__spreadValues({}, lineToDrag), {
          position: __spreadProps(__spreadValues({}, lineToDrag.position), {
            [action.payload.dir]: action.payload.absPos
          })
        })
      });
    }
    case "lineConnect": {
      const lineToConnect = state[action.payload.lineId];
      return isLineConnected({
        line: lineToConnect,
        dir: action.payload.lineDir
      }) ? state : __spreadProps(__spreadValues({}, state), {
        [action.payload.lineId]: connectLine({
          line: lineToConnect,
          dir: action.payload.lineDir,
          itemId: action.payload.itemId,
          port: action.payload.port
        })
      });
    }
    case "lineDisconnect": {
      const lineToDisconnect = state[action.payload.lineId];
      return isLineConnected({
        line: lineToDisconnect,
        dir: action.payload.lineDir
      }) ? __spreadProps(__spreadValues({}, state), {
        [action.payload.lineId]: disconnectLine({
          line: lineToDisconnect,
          dir: action.payload.lineDir,
          absPos: computeLineAbsPosition({
            layoutStructure: Object.values(state),
            position: lineToDisconnect.position
          })[action.payload.lineDir]
        })
      }) : state;
    }
    case "blockResize":
      return __spreadProps(__spreadValues({}, state), {
        [action.payload.item]: action.payload
      });
    case "blocksMove":
      return action.payload.reduce(
        (s, blockItem) => __spreadProps(__spreadValues({}, s), {
          [blockItem.item]: blockItem
        }),
        __spreadValues({}, state)
      );
    case "reset":
      return initializeLayoutStructureState(action.payload);
    default:
      return state;
  }
};
var updateLayoutStructureOnKeyboardMove = ({
  selectedLineItems,
  selectedBlockItems,
  layoutStructure,
  dir,
  snap,
  gridSize
}) => {
  const updatedLines = selectedLineItems.map(({ item }) => {
    const offset = createOffset(dir, 1, 1);
    return handleSingleLineMove({
      layoutStructure,
      lineId: item,
      offset
    });
  });
  const updatedBlocks = selectedBlockItems.map((blockItem) => {
    const { position } = blockItem;
    const snapOffset = snap ? snapOffsetToXY({
      position,
      offset: createOffset(dir, gridSize, gridSize),
      gridWidth: gridSize,
      gridHeight: gridSize,
      spacing: 0,
      padding: 0
    }) : createOffset(dir, 1, 1);
    return updateBlockItemPosition(blockItem, snapOffset);
  });
  const updatedItems = [...updatedLines, ...updatedBlocks];
  const updatedLayoutStructure = layoutStructure.map(
    (item) => {
      var _a;
      return (_a = updatedItems.find((line) => line.item === item.item)) != null ? _a : item;
    }
  );
  return updatedLayoutStructure;
};

// src/AbsoluteLayout.tsx
var emptySelectedItems = [];
var emptyLayoutStructure = [];
var AbsoluteLayout = (props) => {
  const {
    mode = "view",
    showGrid = true,
    options: {
      width = absoluteLayoutOptions.width,
      height = absoluteLayoutOptions.height,
      display = absoluteLayoutOptions.display,
      backgroundColor,
      backgroundImage
    } = {},
    layoutStructure = emptyLayoutStructure,
    rawLayoutStructure = emptyLayoutStructure,
    containerWidth,
    containerHeight,
    selectedItems = emptySelectedItems,
    renderLayoutItem,
    onLayoutItemsSelect = dashboard_utils_exports.noop,
    onLayoutStructureChange = dashboard_utils_exports.noop,
    layoutApiRef = dashboard_utils_exports.noop
  } = props;
  const activeLine = (0, react_exports.useRef)(null);
  const [layoutStructureState, dispatch] = (0, react_exports.useReducer)(
    reducer,
    layoutStructure,
    initializeLayoutStructureState
  );
  const canvasRef = (0, react_exports.useRef)(null);
  const layoutApi = (0, react_exports.useRef)(null);
  const savedScroll = (0, react_exports.useRef)(null);
  const isMouseDownEventRef = (0, react_exports.useRef)(false);
  const layoutStructureRef = (0, react_exports.useRef)(layoutStructure);
  layoutStructureRef.current = layoutStructure;
  const keyboardListener = (0, dashboard_context_exports.useKeyboardListener)();
  const featureFlags = (0, dashboard_context_exports.useFeatureFlags)();
  const preset = (0, dashboard_context_exports.usePreset)();
  const dashboardApi = (0, dashboard_context_exports.useDashboardCoreApi)();
  const telemetry = (0, dashboard_telemetry_exports.useTelemetryApi)();
  const profiler = (0, dashboard_telemetry_exports.useDashboardProfiler)();
  const {
    scale,
    setScale,
    fitToWidth,
    getZoomObserver,
    zoomIn,
    zoomOut,
    setFitToWidthAndScrollToTopLeft
  } = useZoom({
    canvasRef,
    width,
    height,
    containerWidth,
    containerHeight
  });
  const previousScale = (0, dashboard_ui_exports.usePrevious)(scale);
  const previousLayoutStructure = (0, dashboard_ui_exports.usePrevious)(layoutStructure);
  const getCanvasDomElement = (0, react_exports.useCallback)(() => {
    return canvasRef.current;
  }, []);
  const handleKeyboardMove = (0, react_exports.useCallback)(
    ({
      dir,
      snap,
      preventDefault
    }) => {
      var _a, _b, _c, _d, _e;
      if (!(0, dashboard_utils_exports.allowKeyboardShortcut)()) {
        return;
      }
      const selectedLineItems = findSelectedLineItems({
        layoutStructure,
        selectedItems
      });
      const selectedBlockItems = findSelectedBlockItems({
        layoutStructure,
        selectedItems
      });
      const isResizeHandleFocused = ((_a = document.activeElement) == null ? void 0 : _a.tagName) === "BUTTON" && ((_b = document.activeElement) == null ? void 0 : _b.hasAttribute("direction"));
      const isLineHandleFocused = ((_c = document.activeElement) == null ? void 0 : _c.tagName) === "BUTTON" && ((_e = (_d = document.activeElement) == null ? void 0 : _d.getAttribute("data-test")) == null ? void 0 : _e.startsWith("line-handle"));
      if (selectedLineItems.length === 0 && selectedBlockItems.length === 0 || mode !== "edit" || isResizeHandleFocused || isLineHandleFocused) {
        return;
      }
      const updatedLayoutStructure = updateLayoutStructureOnKeyboardMove({
        selectedBlockItems,
        selectedLineItems,
        dir,
        snap,
        gridSize: GRID_SIZE,
        layoutStructure
      });
      onLayoutStructureChange(updatedLayoutStructure);
      preventDefault();
    },
    [layoutStructure, mode, onLayoutStructureChange, selectedItems]
  );
  const handleLayoutStructureChange = (0, react_exports.useCallback)(() => {
    const updatedLayoutStructure = layoutStructure.map(
      (item) => {
        var _a;
        return (_a = layoutStructureState[item.item]) != null ? _a : item;
      }
    );
    onLayoutStructureChange(updatedLayoutStructure);
  }, [layoutStructure, layoutStructureState, onLayoutStructureChange]);
  const handleLineMove = (0, react_exports.useCallback)(
    (_e, offset) => {
      const selectedLineItems = findSelectedLineItems({
        layoutStructure,
        selectedItems
      });
      if (selectedLineItems.length === 0) {
        return;
      }
      const lineId = selectedLineItems[0].item;
      dispatch({
        type: "lineMove",
        payload: handleSingleLineMove({
          layoutStructure,
          lineId,
          offset
        })
      });
    },
    [layoutStructure, selectedItems]
  );
  const handleLineMoved = (0, react_exports.useCallback)(() => {
    handleLayoutStructureChange();
  }, [handleLayoutStructureChange]);
  const handleLineDragStart = (0, react_exports.useCallback)(
    (_e, dir) => {
      const selectedLineItems = findSelectedLineItems({
        layoutStructure,
        selectedItems
      });
      if (selectedLineItems.length !== 1) {
        return;
      }
      const lineId = selectedLineItems[0].item;
      const line = handleSingleLineDragStart({
        lineId,
        layoutStructure,
        lineDir: dir
      });
      dispatch({
        type: "lineDragStart",
        payload: line
      });
      activeLine.current = { id: lineId, dir };
    },
    [layoutStructure, selectedItems]
  );
  const handleLineDrag = (0, react_exports.useCallback)(
    (_e, offset) => {
      if (!activeLine.current) {
        return;
      }
      const line = layoutStructure.find(
        (item) => {
          var _a;
          return item.item === ((_a = activeLine.current) == null ? void 0 : _a.id);
        }
      );
      const absPos = computeLineAbsPosition({
        layoutStructure,
        position: line.position
      })[activeLine.current.dir];
      dispatch({
        type: "lineDrag",
        payload: {
          id: activeLine.current.id,
          dir: activeLine.current.dir,
          absPos: {
            x: absPos.x + offset.offsetX,
            y: absPos.y + offset.offsetY
          }
        }
      });
    },
    [layoutStructure]
  );
  const handleLineDragged = (0, react_exports.useCallback)(() => {
    activeLine.current = null;
    handleLayoutStructureChange();
  }, [handleLayoutStructureChange]);
  const handleLineConnected = (0, react_exports.useCallback)((itemId, port) => {
    if (!activeLine.current) {
      return;
    }
    const lineId = activeLine.current.id;
    dispatch({
      type: "lineConnect",
      payload: {
        lineId,
        lineDir: activeLine.current.dir,
        itemId,
        port
      }
    });
  }, []);
  const handleLineDisconnected = (0, react_exports.useCallback)(() => {
    if (!activeLine.current) {
      return;
    }
    dispatch({
      type: "lineDisconnect",
      payload: {
        lineId: activeLine.current.id,
        lineDir: activeLine.current.dir
      }
    });
  }, []);
  const handleItemSelected = (0, react_exports.useCallback)(
    (_e, newSelectedItems = emptySelectedItems) => {
      onLayoutItemsSelect(newSelectedItems);
    },
    [onLayoutItemsSelect]
  );
  const handleBlockItemResize = (0, react_exports.useCallback)(
    (_e, itemId, offset, dir) => {
      const item = layoutStructure.find(
        (itm) => itm.item === itemId
      );
      const snapOffset = snapOffsetToWH({
        position: item.position,
        offset,
        gridWidth: GRID_SIZE,
        gridHeight: GRID_SIZE,
        spacing: 0,
        padding: 0
      });
      const newItem = updateBlockItemSize({
        item,
        offset: snapOffset,
        dir,
        options: {
          minHeight: MIN_HEIGHT,
          minWidth: MIN_WIDTH
        }
      });
      if ((0, lodash_exports.isEqual)(
        newItem.position,
        layoutStructureState[newItem.item].position
      )) {
        return;
      }
      profiler == null ? void 0 : profiler.startMeasurement(dashboard_telemetry_exports.VIZ_RESIZE_EVENT);
      dispatch({
        type: "blockResize",
        payload: newItem
      });
    },
    [layoutStructure, layoutStructureState, profiler]
  );
  const handleBlockItemResized = (0, react_exports.useCallback)(() => {
    profiler == null ? void 0 : profiler.emitAndClearMeasurements({
      measurementName: dashboard_telemetry_exports.VIZ_RESIZE_EVENT
    });
    handleLayoutStructureChange();
  }, [handleLayoutStructureChange, profiler]);
  const handleBlockItemMove = (0, react_exports.useCallback)(
    (_e, offset) => {
      const selectedBlockItems = findSelectedBlockItems({
        layoutStructure,
        selectedItems
      });
      if (selectedBlockItems.length === 0) {
        return;
      }
      const firstBlockItem = selectedBlockItems[0];
      const moveOffset = snapOffsetToXY({
        position: firstBlockItem.position,
        offset,
        gridWidth: GRID_SIZE,
        gridHeight: GRID_SIZE,
        spacing: 0,
        padding: 0
      });
      const updatedSelectedBlockItems = selectedBlockItems.map(
        (blockItem) => updateBlockItemPosition(blockItem, moveOffset)
      );
      const firstUpdatedBlockItem = updatedSelectedBlockItems[0];
      if ((0, lodash_exports.isEqual)(
        firstUpdatedBlockItem.position,
        layoutStructureState[firstUpdatedBlockItem.item].position
      )) {
        return;
      }
      profiler == null ? void 0 : profiler.startMeasurement(dashboard_telemetry_exports.VIZ_MOVE_EVENT);
      dispatch({
        type: "blocksMove",
        payload: updatedSelectedBlockItems
      });
    },
    [layoutStructure, layoutStructureState, profiler, selectedItems]
  );
  const handleBlockItemMoved = (0, react_exports.useCallback)(() => {
    profiler == null ? void 0 : profiler.emitAndClearMeasurements({
      measurementName: dashboard_telemetry_exports.VIZ_MOVE_EVENT,
      metadata: { numVisualizationsMoved: selectedItems == null ? void 0 : selectedItems.length }
    });
    handleLayoutStructureChange();
  }, [handleLayoutStructureChange, profiler, selectedItems]);
  const blockItems = (0, react_exports.useMemo)(
    () => layoutStructure.filter((item) => (0, dashboard_utils_exports.isBlockItem)(item)).map(
      (block) => {
        var _a;
        return (_a = layoutStructureState[block.item]) != null ? _a : block;
      }
    ),
    [layoutStructure, layoutStructureState]
  );
  const lineItems = (0, react_exports.useMemo)(
    () => layoutStructure.filter(({ type }) => type === "line").map(
      (line) => {
        var _a;
        return (_a = layoutStructureState[line.item]) != null ? _a : line;
      }
    ),
    [layoutStructure, layoutStructureState]
  );
  const optionsRef = (0, react_exports.useRef)(props.options);
  optionsRef.current = props.options;
  const bgImageSrcRef = (0, react_exports.useRef)();
  const realBGImageUrl = useBackgroundImage(backgroundImage == null ? void 0 : backgroundImage.src);
  bgImageSrcRef.current = realBGImageUrl;
  const getBgImageSrc = (0, react_exports.useCallback)(() => {
    return bgImageSrcRef.current;
  }, []);
  if (!layoutApi.current) {
    layoutApi.current = new AbsoluteLayoutApi_default({
      getZoomObserver,
      setFitToWidthAndScrollToTopLeft,
      zoomIn,
      zoomOut,
      setScale,
      optionsRef,
      layoutStructureRef,
      getCanvasDomElement,
      telemetry,
      getBgImageSrc
    });
  }
  (0, react_exports.useEffect)(() => {
    if (!(0, lodash_exports.isEqual)(layoutStructure, previousLayoutStructure)) {
      dispatch({
        type: "reset",
        payload: layoutStructure
      });
    }
  }, [layoutStructure, previousLayoutStructure]);
  (0, react_exports.useEffect)(() => {
    layoutApiRef(layoutApi.current);
    return () => {
      layoutApiRef(null);
    };
  }, [layoutApiRef]);
  (0, react_exports.useEffect)(() => {
    if (!keyboardListener) {
      return dashboard_utils_exports.noop;
    }
    const unsubscribeMove = keyboardListener.subscribe(
      "move",
      handleKeyboardMove
    );
    return () => {
      unsubscribeMove();
    };
  }, [handleKeyboardMove, keyboardListener]);
  (0, react_exports.useEffect)(() => {
    if (!keyboardListener) {
      return dashboard_utils_exports.noop;
    }
    const unsubscribeCancel = keyboardListener.subscribe(
      "cancel",
      () => handleItemSelected(null, [])
    );
    return () => {
      unsubscribeCancel();
    };
  }, [handleItemSelected, keyboardListener]);
  (0, react_exports.useEffect)(() => {
    if (display === "fit-to-width" || display === "auto-scale") {
      setFitToWidthAndScrollToTopLeft();
    } else if (display === "actual-size") {
      setScale(1);
    } else {
      setFitToWidthAndScrollToTopLeft(1);
    }
  }, [display, setFitToWidthAndScrollToTopLeft, setScale]);
  if (previousScale != null && previousScale >= 1 && scale < 1 && featureFlags.enableZoomCenter) {
    const previousLayout = getCanvasDomElement().parentElement;
    if (previousLayout) {
      savedScroll.current = {
        x: previousLayout.scrollLeft + previousLayout.offsetWidth / 2,
        y: previousLayout.scrollTop + previousLayout.offsetHeight / 2
      };
    }
  }
  (0, react_exports.useLayoutEffect)(() => {
    var _a, _b;
    if (fitToWidth || previousScale == null || previousScale === scale) {
      return;
    }
    const layout = getCanvasDomElement().parentElement;
    if (!layout) {
      return;
    }
    const { offsetWidth, offsetHeight, scrollTop, scrollLeft } = layout;
    if (previousScale < 1 && scale > 1 && ((_a = savedScroll == null ? void 0 : savedScroll.current) == null ? void 0 : _a.x) != null && ((_b = savedScroll == null ? void 0 : savedScroll.current) == null ? void 0 : _b.y) != null && featureFlags.enableZoomCenter) {
      layout.scrollLeft = savedScroll.current.x - layout.offsetWidth / 2;
      layout.scrollTop = savedScroll.current.y - layout.offsetHeight / 2;
    } else {
      const scaleRatio = scale / previousScale;
      const newScroll = shiftViewportOnZoom({
        offsetWidth,
        offsetHeight,
        scrollTop,
        scrollLeft,
        scaleRatio
      });
      layout.scrollLeft = newScroll.scrollLeft;
      layout.scrollTop = newScroll.scrollTop;
    }
  }, [
    featureFlags.enableZoomCenter,
    fitToWidth,
    getCanvasDomElement,
    previousScale,
    scale
  ]);
  (0, react_exports.useEffect)(() => {
    if (activeLine.current && layoutStructureState[activeLine.current.id] == null) {
      activeLine.current = null;
    }
  }, [layoutStructureState]);
  const image = (0, react_exports.useMemo)(
    () => backgroundImage ? validateBackgroundImage({
      backgroundImage,
      canvasWidth: width,
      canvasHeight: height
    }) : {},
    [backgroundImage, height, width]
  );
  const modeSpecificProps = (0, react_exports.useMemo)(
    () => mode === "edit" ? {
      movable: true,
      showOverflowContent: true,
      showGrid,
      showBorder: true,
      gridLineWidth: 1,
      gridPadding: GRID_PADDING,
      onItemMove: handleBlockItemMove,
      onItemMoved: handleBlockItemMoved,
      userSelect: false,
      allowMultiselect: true
    } : {
      movable: false,
      showOverflowContent: false,
      showGrid: false,
      showBorder: false,
      userSelect: true,
      allowMultiselect: false
    },
    [handleBlockItemMove, handleBlockItemMoved, mode, showGrid]
  );
  const allBlockItems = (0, react_exports.useMemo)(
    () => getAllBlockItems({ layoutStructure, layoutStructureState }),
    [layoutStructure, layoutStructureState]
  );
  const blockItemElements = (0, react_exports.useMemo)(() => {
    return allBlockItems.map(({ item, type, position: { x, y, w, h } }) => /* @__PURE__ */ react_default.createElement(
      ResponsiveBlockItem_default,
      {
        key: item,
        itemId: item,
        type,
        x,
        y,
        w,
        h,
        canvasHeight: height,
        renderLayoutItem,
        onItemSelected: handleItemSelected,
        isMouseDownEventRef
      }
    ));
  }, [handleItemSelected, height, allBlockItems, renderLayoutItem]);
  const lineElements = (0, react_exports.useMemo)(() => {
    const selectedLineItems = findSelectedLineItems({
      layoutStructure,
      selectedItems
    });
    const selectable = mode === "edit";
    return lineItems.map((line) => {
      const editable = selectedLineItems.length === 1 && selectedLineItems[0].item === line.item && selectable;
      let absPos = { from: { x: 0, y: 0 }, to: { x: 0, y: 0 } };
      try {
        absPos = computeLineAbsPosition({
          layoutStructure: Object.values(layoutStructureState),
          position: line.position
        });
      } catch (err) {
        if (err instanceof Error && err.message.endsWith(InvalidBlockItemError)) {
          absPos = computeLineAbsPosition({
            layoutStructure: rawLayoutStructure,
            position: line.position
          });
        }
      }
      const fromTestHooks = "item" in line.position.from && "port" in line.position.from ? {
        fromItem: line.position.from.item,
        fromPort: line.position.from.port
      } : {};
      const toTestHooks = "item" in line.position.to && "port" in line.position.to ? {
        toItem: line.position.to.item,
        toPort: line.position.to.port
      } : {};
      return /* @__PURE__ */ react_default.createElement(
        ResponsiveLine_default,
        __spreadValues(__spreadValues({
          key: line.item,
          lineId: line.item,
          scale,
          selectable,
          editable,
          renderLayoutItem,
          onLineMove: handleLineMove,
          onLineMoved: handleLineMoved,
          onLineDragStart: handleLineDragStart,
          onLineDrag: handleLineDrag,
          onLineDragged: handleLineDragged,
          onItemSelected: handleItemSelected,
          fromX: absPos.from.x,
          fromY: absPos.from.y,
          toX: absPos.to.x,
          toY: absPos.to.y
        }, fromTestHooks), toTestHooks)
      );
    });
  }, [
    handleItemSelected,
    handleLineDrag,
    handleLineDragStart,
    handleLineDragged,
    handleLineMove,
    handleLineMoved,
    rawLayoutStructure,
    layoutStructure,
    layoutStructureState,
    lineItems,
    mode,
    renderLayoutItem,
    scale,
    selectedItems
  ]);
  const outlineElements = (0, react_exports.useMemo)(() => {
    const selectedLineItems = findSelectedLineItems({
      layoutStructure: Object.values(layoutStructureState),
      selectedItems
    });
    const selectedBlockItems = findSelectedBlockItems({
      layoutStructure: Object.values(layoutStructureState),
      selectedItems
    });
    return allBlockItems.map((block) => {
      const connectable = mode === "edit" && selectedBlockItems.length === 0 && selectedLineItems.length === 1 && activeLine != null;
      const resizable = mode === "edit" && selectedBlockItems.length === 1 && selectedBlockItems[0].item === block.item;
      const selected = selectedBlockItems.findIndex((b) => b.item === block.item) >= 0;
      if (!connectable && !resizable && !selected) {
        return null;
      }
      const definition = dashboard_definition_exports.DashboardDefinition.fromJSON(
        dashboardApi == null ? void 0 : dashboardApi.getDefinition()
      );
      const itemPresetType = definition.getItemPresetType(block.item);
      const handleDirections = preset.getResizeHandleDirections(itemPresetType);
      return /* @__PURE__ */ react_default.createElement(
        ResponsiveBlockOutline_default,
        {
          key: block.item,
          itemId: block.item,
          x: block.position.x,
          y: block.position.y,
          w: block.position.w,
          h: block.position.h,
          scale,
          connectable,
          resizable,
          onResize: handleBlockItemResize,
          onResized: handleBlockItemResized,
          onLineConnect: handleLineConnected,
          onLineDisconnect: handleLineDisconnected,
          handleDirections
        }
      );
    });
  }, [
    layoutStructureState,
    selectedItems,
    allBlockItems,
    mode,
    activeLine,
    dashboardApi,
    preset,
    scale,
    handleBlockItemResize,
    handleBlockItemResized,
    handleLineConnected,
    handleLineDisconnected
  ]);
  const handleMouseDownCapture = (0, react_exports.useCallback)(() => {
    isMouseDownEventRef.current = true;
  }, []);
  const handleMouseUpCapture = (0, react_exports.useCallback)(() => {
    isMouseDownEventRef.current = false;
  }, []);
  return /* @__PURE__ */ react_default.createElement(
    AbsoluteCanvas_default,
    __spreadValues({
      "data-test": "absolute-layout",
      blockItems,
      width,
      height,
      scale,
      gridWidth: GRID_SIZE - 1,
      gridHeight: GRID_SIZE - 1,
      backgroundColor,
      backgroundImageSrc: image.src,
      backgroundImageSizeType: image.sizeType,
      backgroundImageWidth: image.w,
      backgroundImageHeight: image.h,
      backgroundImagePositionX: image.x,
      backgroundImagePositionY: image.y,
      backgroundImageOpacity: image.opacity,
      selectedLayoutItems: selectedItems,
      onItemSelected: handleItemSelected,
      ref: canvasRef,
      onMouseDownCapture: handleMouseDownCapture,
      onMouseUpCapture: handleMouseUpCapture
    }, modeSpecificProps),
    /* @__PURE__ */ react_default.createElement(
      Layer_default,
      {
        key: "block-item-layer",
        "data-test": "block-item-layer",
        zIndex: 0
      },
      lineElements,
      blockItemElements.map((el, i) => /* @__PURE__ */ react_default.createElement(react_default.Fragment, { key: el.key }, el, /* @__PURE__ */ react_default.createElement(Layer_default, { key: "outline-layer", "data-test": "outline-layer" }, outlineElements[i])))
    )
  );
};
var AbsoluteLayout_default = withLayoutShowHide(AbsoluteLayout, {
  schema: absoluteLayoutOptionsSchema_default
});

export {
  AbsoluteLayout_default
};
//# sourceMappingURL=chunk-TF3DLY5V.js.map
