/* Copyright © 2020 Splunk Inc.
SPLUNK CONFIDENTIAL – Use or disclosure of this material in whole or
in part without a valid written license from Splunk Inc. is PROHIBITED. */
import {
  disconnectLine,
  isLineConnected,
  updateLineAbsPosition
} from "./chunk-NJ3ZVGVA.js";
import {
  dashboard_utils_exports
} from "./chunk-QCW3O7QD.js";
import {
  i18n_exports
} from "./chunk-PLOB2GBC.js";

// src/utils/blockUtils.ts
var findSelectedBlockItems = ({
  layoutStructure,
  selectedItems
}) => layoutStructure.filter(
  (item) => selectedItems.findIndex(
    ({ id }) => item.item === id && (0, dashboard_utils_exports.isBlockItem)(item)
  ) >= 0
);
var getAllBlockItems = ({
  layoutStructure,
  layoutStructureState = {}
}) => {
  return layoutStructure.map((item) => {
    var _a;
    return (_a = layoutStructureState[item.item]) != null ? _a : item;
  }).filter((item) => (0, dashboard_utils_exports.isBlockItem)(item));
};
var computePortPosition = (blockItem, port) => {
  const blockPosition = blockItem.position;
  switch (port) {
    case "n":
      return {
        x: Math.round(blockPosition.x + blockPosition.w / 2),
        y: blockPosition.y
      };
    case "w":
      return {
        x: blockPosition.x,
        y: Math.round(blockPosition.y + blockPosition.h / 2)
      };
    case "e":
      return {
        x: blockPosition.x + blockPosition.w,
        y: Math.round(blockPosition.y + blockPosition.h / 2)
      };
    case "s":
    default:
      return {
        x: Math.round(blockPosition.x + blockPosition.w / 2),
        y: blockPosition.y + blockPosition.h
      };
  }
};
var InvalidBlockItemError = "refers to an invalid block item";
var InvalidBlockItemErrorMessage = (0, i18n_exports._)(`%s ${InvalidBlockItemError}`);
var getBlockItem = ({
  layoutStructure,
  id
}) => {
  const blockItem = layoutStructure.find(({ item }) => item === id);
  if (!blockItem || blockItem.type === "line") {
    throw Error(InvalidBlockItemErrorMessage.replace("%s", id));
  }
  return blockItem;
};

// src/utils/lineUtils.ts
var computeLineAbsPosition = ({
  layoutStructure,
  position
}) => {
  let { from, to } = position;
  if ("item" in from && "port" in from) {
    const blockItem = getBlockItem({ layoutStructure, id: from.item });
    from = computePortPosition(blockItem, from.port);
  }
  if ("item" in to && "port" in to) {
    const blockItem = getBlockItem({ layoutStructure, id: to.item });
    to = computePortPosition(blockItem, to.port);
  }
  return {
    from,
    to
  };
};
var findSelectedLineItems = ({
  layoutStructure,
  selectedItems
}) => layoutStructure.filter(
  ({ item, type }) => selectedItems.findIndex(
    ({ id }) => item === id && type === "line"
  ) >= 0
);
var handleSingleLineMove = ({
  lineId,
  offset,
  layoutStructure
}) => {
  let line = layoutStructure.find(
    (item) => item.item === lineId
  );
  const linePosition = computeLineAbsPosition({
    layoutStructure,
    position: line.position
  });
  if (isLineConnected({ line, dir: "from" })) {
    line = disconnectLine({
      line,
      dir: "from",
      absPos: linePosition.from
    });
  }
  if (isLineConnected({ line, dir: "to" })) {
    line = disconnectLine({
      line,
      dir: "to",
      absPos: linePosition.to
    });
  }
  line = updateLineAbsPosition({
    line,
    dir: "from",
    offset
  });
  line = updateLineAbsPosition({
    line,
    dir: "to",
    offset
  });
  return line;
};
var handleSingleLineDragStart = ({
  lineId,
  layoutStructure,
  lineDir
}) => {
  let line = layoutStructure.find(
    (item) => item.item === lineId
  );
  const linePosition = computeLineAbsPosition({
    layoutStructure,
    position: line.position
  });
  if (isLineConnected({
    line,
    dir: lineDir
  })) {
    line = disconnectLine({
      line,
      dir: lineDir,
      absPos: linePosition[lineDir]
    });
  }
  return line;
};
var getAllLineItems = ({
  layoutStructure,
  layoutStructureState = {}
}) => {
  return layoutStructure.map((item) => {
    var _a;
    return (_a = layoutStructureState[item.item]) != null ? _a : item;
  }).filter(({ type }) => type === "line");
};

export {
  findSelectedBlockItems,
  getAllBlockItems,
  InvalidBlockItemError,
  getBlockItem,
  computeLineAbsPosition,
  findSelectedLineItems,
  handleSingleLineMove,
  handleSingleLineDragStart,
  getAllLineItems
};
//# sourceMappingURL=chunk-X4NAUHLE.js.map
