import type { ReactNode } from 'react';
import type { Coordinate, SelectedItem, AbsoluteLayoutItem, AbsoluteBlockItem, StructureItemType } from '@splunk/dashboard-types';
export interface Dimension {
    width: number | string;
    height: number | string;
}
export interface ShouldRenderActionMenuBelowVizProps {
    y: number;
    canvasHeight: number;
}
export interface RenderResponsiveLineProps {
    from: Coordinate;
    to: Coordinate;
    onLineSelect: (e: React.MouseEvent) => void;
}
export interface GridCanvasState {
    startPosition: Coordinate | null;
    isMoving: boolean;
}
export type OnItemSelected = (e: MouseEvent | React.MouseEvent | React.KeyboardEvent | React.FocusEvent, items: SelectedItem[]) => void;
export interface GridCanvasProps extends CanvasProps {
    panelPadding?: number;
    movable?: boolean;
    selectable?: boolean;
    blockItems: AbsoluteBlockItem[];
    selectedLayoutItems?: SelectedItem[];
    onItemSelected: OnItemSelected;
    onItemMove?: (e: MouseEvent, { currentPosition }: {
        currentPosition: Coordinate;
    }) => void;
    onItemMoved?: (e: MouseEvent, offset: Offset) => void;
    children: React.ReactNode;
}
export type ActionMenuRenderMetadata = {
    actionMenuPortal?: React.MutableRefObject<HTMLDivElement | null>;
    renderActionMenuInPortal?: boolean;
};
export type DragHandleRenderMetadata = {
    dragHandlePortal?: React.MutableRefObject<HTMLDivElement | null>;
};
export type RenderLayoutItem = (itemId: string, props: ActionMenuRenderMetadata & DragHandleRenderMetadata & (Dimension | (Dimension & ShouldRenderActionMenuBelowVizProps) | RenderResponsiveLineProps), type: StructureItemType, onSelected?: OnItemSelected) => JSX.Element;
export interface Offset {
    offsetX: number;
    offsetY: number;
}
export type LineDirection = 'from' | 'to';
export interface LayoutStructureState {
    [key: string]: AbsoluteLayoutItem;
}
export interface BackgroundImageProps {
    src?: string;
    imageWidth?: number;
    imageHeight?: number;
    sizeType?: 'contain' | 'custom' | 'cover' | 'auto';
    imageX?: number;
    imageY?: number;
    opacity?: number;
}
export interface CanvasProps {
    width: number;
    height: number;
    scale?: number;
    backgroundColor?: string;
    backgroundImageSrc?: BackgroundImageProps['src'];
    backgroundImageSizeType?: BackgroundImageProps['sizeType'];
    backgroundImageWidth?: BackgroundImageProps['imageWidth'];
    backgroundImageHeight?: BackgroundImageProps['imageHeight'];
    backgroundImagePositionX?: BackgroundImageProps['imageX'];
    backgroundImagePositionY?: BackgroundImageProps['imageY'];
    backgroundImageOpacity?: BackgroundImageProps['opacity'];
    showGrid?: boolean;
    gridLineColor?: string;
    gridPadding?: number;
    gridLineWidth?: number;
    gridWidth?: number;
    gridHeight?: number;
    showOverflowContent?: boolean;
    userSelect?: boolean;
    gridLineOpacity?: number;
    showBorder?: boolean;
    cssScaling?: boolean;
    children: ReactNode;
    onMouseDown?: (e: React.MouseEvent) => void;
    onContextMenu?: (e: React.MouseEvent) => void;
    onMouseDownCapture?: (e: React.MouseEvent) => void;
    onMouseUpCapture?: (e: React.MouseEvent) => void;
}
export interface EdgeItem {
    item: string;
    orientation: 'horizontal' | 'vertical';
    edgeStart: Coordinate;
    edgeEnd: Coordinate;
    visualizations: AbsoluteBlockItem[];
    isCanvasEdge?: boolean;
}
export type Quadrant = 'w' | 's' | 'n' | 'e';
export type LayoutError = {
    itemId: string;
    messages: string[];
};
export type HorizontalBoundaries = {
    leftBoundary: number;
    rightBoundary: number;
};
export type VerticalBoundaries = {
    upperBoundary: number;
    lowerBoundary: number;
};
export type EdgeBoundaries = HorizontalBoundaries | VerticalBoundaries;
export type EdgeAppearance = 'invalid' | 'dropTarget' | 'normal' | 'hidden';
//# sourceMappingURL=index.d.ts.map