import type { AbsoluteBlockItem, Coordinate } from '@splunk/dashboard-types';
import type { SplitGridEntry } from './gridOrderUtils';
/**
 * Mapping of block item ID to block item which contains changes to be made to a
 * grid structure in order to apply the appropriate resize behavior
 */
export type ResizeChanges = Record<string, AbsoluteBlockItem>;
type RecursiveDirectionalResizeArgs = {
    container: SplitGridEntry;
    newSize: number;
    position: number;
    translation: Coordinate;
};
type RecursiveDirectionalResizeFn = (args: RecursiveDirectionalResizeArgs) => ResizeChanges;
declare const HorizontalResizeConfig: {
    readonly dimension: "width";
    readonly blockSizeDimension: "w";
    readonly blockPosDimension: "x";
    readonly splitGridConfig: "columns";
    readonly splitGridKey: "splitByColumns";
    readonly recursiveResizeFn: RecursiveDirectionalResizeFn;
};
declare const VerticalResizeConfig: {
    readonly dimension: "height";
    readonly blockSizeDimension: "h";
    readonly blockPosDimension: "y";
    readonly splitGridConfig: "rows";
    readonly splitGridKey: "splitByRows";
    readonly recursiveResizeFn: RecursiveDirectionalResizeFn;
};
type HandleResizingArgs = {
    container?: SplitGridEntry;
    itemToRemove: AbsoluteBlockItem;
    config: typeof HorizontalResizeConfig | typeof VerticalResizeConfig;
};
/**
 * An object containing properties:
 *
 * `container` - the result of calling `getContainingRowColumn`
 *
 * `itemToRemove` - the {@link AbsoluteBlockItem block item} being removed from the grid, causing the resize
 */
export type HandleDirectionalResizingArgs = Pick<HandleResizingArgs, 'container' | 'itemToRemove'>;
/**
 * Handles horizontal resizing of a container.
 * The width of itemToRemove will be proportionally redistributed to all other column splits
 * in the container based upon the splits' width as a percentage of the total container width
 * @param {HandleDirectionalResizingArgs} args container and itemToRemove
 * @param {SplitGridEntry} args.container The result of `getContainingRowColumn`
 * @param {AbsoluteBlockItem} args.itemToRemove The block item being removed from the structure, causing the resize
 * @returns {ResizeChanges} Object detailing block item changes needed for the resize
 */
export declare const handleHorizontalResizing: (args: HandleDirectionalResizingArgs) => ResizeChanges;
/**
 * Handles vertical resizing of a container.
 * The height of itemToRemove will be proportionally redistributed to all other row splits in
 * the container based upon the splits' height as a percentage of the total container height
 * @param {HandleDirectionalResizingArgs} args container and itemToRemove
 * @param {SplitGridEntry} args.container The non-nullable result of `getContainingRowColumn`
 * @param {AbsoluteBlockItem} args.itemToRemove The block item being removed from the structure, causing the resize
 * @returns {ResizeChanges} Object detailing block item changes needed for the resize
 */
export declare const handleVerticalResizing: (args: HandleDirectionalResizingArgs) => ResizeChanges;
export {};
//# sourceMappingURL=gridLayoutAutoResize.d.ts.map