import type { AbsoluteBlockItem, Coordinate, GridLayoutStructure } from '@splunk/dashboard-types';
import type { Intersections } from './edgeUtils';
import { type SplitGrid, type SplitGridEntry } from './gridOrderUtils';
export type GridInfo = {
    width: number;
    height: number;
    originItem: AbsoluteBlockItem;
    itemToRemove: AbsoluteBlockItem;
    nodes: Intersections;
    splitByRows: SplitGrid;
    splitByColumns: SplitGrid;
};
export declare const DEFAULT_BLOCK_ITEM: AbsoluteBlockItem;
/**
 * Translate an AbsoluteBlockItem object by the provided X and Y values
 * @param {AbsoluteBlockItem} item An AbsoluteBlockItem object to be translated
 * @param {Coordinate} translation An X and Y delta by which to translate the item
 * @returns A cloned object with the position translated by the provided delta
 */
export declare const translateBlockItem: (item: AbsoluteBlockItem, translation: Coordinate) => AbsoluteBlockItem;
/**
 * Translate an array of AbsoluteBlockItem objects by the provided X and Y values
 * @param {AbsoluteBlockItem[]} items Array of AbsoluteBlockItem objects to be translated
 * @param {Coordinate} translation An X and Y delta by which to translate the items
 * @returns A mapped array with each entry translated by the provided delta
 */
export declare const translateBlockItems: (items: AbsoluteBlockItem[], translation: Coordinate) => AbsoluteBlockItem[];
/**
 * Translate a SplitGridEntry object by the provided X and Y values
 * @param {SplitGridEntry} gridSplit Grid split to be translated
 * @param {Coordinate} translation An X and Y delta by which to translate the split. Applied to all content and the split origin.
 * @returns The SplitGridEntry with an updated origin and content.items[].position values
 */
export declare const translateGridSplit: (gridSplit: SplitGridEntry, translation: Coordinate) => SplitGridEntry;
/**
 * Convert a grid structure into an object which contains the grid width, height,
 * origin block item, intersection nodes, and the result of splitGridByRows and/or
 * splitGridByColumns. If a value for `idToRemove` is provided then the AbsoluteBlockItem
 * with that ID will also be returned
 * @param {GridLayoutStructure} param0.layoutStructure The layout structure of a grid layout to be split
 * @param {String | undefined} param0.idToRemove The ID of a block item to be fetched from the grid structure
 * @param {"both" | "none" | "rows" | "columns"} param0.splitConfig If the grid should be split by rows, columns, both, or neither
 * @returns An object which describes how the grid structure can be split into rows and/or columns
 */
export declare const splitGrid: ({ layoutStructure, idToRemove, splitConfig, alwaysIncludeLastRow, }: {
    layoutStructure: GridLayoutStructure;
    idToRemove?: string;
    splitConfig?: "both" | "none" | "rows" | "columns";
    alwaysIncludeLastRow?: boolean;
}) => GridInfo;
//# sourceMappingURL=splitGridStructure.d.ts.map